@@ -1,16 +1,18 @@
 #ifndef BACKGROUND_HPP_INCLUDED
 #define BACKGROUND_HPP_INCLUDED
 
+#include <stdint.h>
+
 #include <string>
 #include <vector>
 
 #include "boost/shared_ptr.hpp"
 
-#include "SDL.h"
+#include "graphics.hpp"
 
 #include "geometry.hpp"
 #include "texture.hpp"
-#include "wml_node_fwd.hpp"
+#include "variant.hpp"
 
 class level;
 
@@ -18,15 +20,17 @@ class level;
 class background
 {
 public:
+	static void load_modified_backgrounds();
+
 	//gets a background associated with a given ID.
 	static boost::shared_ptr<background> get(const std::string& id, int palette_id);
 
 	//all available backgrounds.
 	static std::vector<std::string> get_available_backgrounds();
 
-	background(const wml::const_node_ptr& node, int palette);
+	background(variant node, int palette);
 	const std::string& id() const { return id_; }
-	wml::node_ptr write() const;
+	variant write() const;
 	void draw(int x, int y, const rect& area, const std::vector<rect>& opaque_areas, int rotation, int cycle) const;
 	void draw_foreground(double x, double y, int rotation, int cycle) const;
 
@@ -34,7 +38,7 @@ public:
 private:
 
 	void draw_layers(int x, int y, const rect& area, const std::vector<rect>& opaque_areas, int rotation, int cycle) const;
-	std::string id_;
+	std::string id_, file_;
 	SDL_Color top_, bot_;
 	int width_, height_;
 	point offset_;
@@ -68,6 +72,9 @@ private:
 		//if false we can disable blending while this is drawn
 		bool blend;
 
+		//if true prevents the image being tiled.
+		bool notile;
+
 		bool tile_upwards, tile_downwards;
 	};
 
