@@ -2,6 +2,7 @@
 #define COLLISION_UTILS_HPP_INCLUDED
 
 #include "entity.hpp"
+#include "level_solid_map.hpp"
 #include "solid_map.hpp"
 
 class level;
@@ -12,8 +13,12 @@ int get_solid_dimension_id(const std::string& key);
 
 //struct which provides information about a surface we collide with.
 struct collision_info {
-	collision_info() : friction(0), traction(0), damage(0), adjust_y(0), platform(false), area_id(0), collide_with_area_id(0)
+	collision_info() : surf_info(0), friction(0), traction(0), damage(0), adjust_y(0), platform(false), area_id(0), collide_with_area_id(0)
 	{}
+
+	void read_surf_info();
+
+	const surface_info* surf_info;
 	int friction;
 	int traction;
 	int damage;
@@ -53,6 +58,8 @@ bool point_standable(const level& lvl, const entity& e, int x, int y, collision_
 //'dir' is MOVE_NONE, then all pixels will be checked.
 bool entity_collides(level& lvl, const entity& e, MOVE_DIRECTION dir, collision_info* info=NULL);
 
+void debug_check_entity_solidity(const level& lvl, const entity& e);
+
 //function which finds if one entity collides with another given entity.
 bool entity_collides_with_entity(const entity& e, const entity& other, collision_info* info=NULL);
 
