@@ -2,6 +2,7 @@
 #define CONCURRENT_CACHE_HPP_INCLUDED
 
 #include <map>
+#include <vector>
 
 #include "thread.hpp"
 
@@ -11,13 +12,14 @@ class concurrent_cache
 public:
 	typedef std::map<Key, Value> map_type;
 	size_t size() const { threading::lock l(mutex_); return map_.size(); }
-	Value get(const Key& key) {
+	const Value& get(const Key& key) {
 		threading::lock l(mutex_);
 		typename map_type::const_iterator itor = map_.find(key);
 		if(itor != map_.end()) {
 			return itor->second;
 		} else {
-			return Value();
+			static const Value empty_result = Value();
+			return empty_result;
 		}
 	}
 
@@ -26,6 +28,11 @@ public:
 		map_[key] = value;
 	}
 
+	void erase(const Key& key) {
+		threading::lock l(mutex_);
+		map_.erase(key);
+	}
+
 	int count(const Key& key) const {
 		threading::lock l(mutex_);
 		return map_.count(key);
@@ -36,6 +43,16 @@ public:
 		map_.clear();
 	}
 
+	std::vector<Key> get_keys() {
+		std::vector<Key> result;
+		threading::lock l(mutex_);
+		for(typename map_type::const_iterator i = map_.begin(); i != map_.end(); ++i) {
+			result.push_back(i->first);
+		}
+
+		return result;
+	}
+
 	struct lock : public threading::lock {
 		explicit lock(concurrent_cache& cache) : threading::lock(cache.mutex_), cache_(cache) {
 		}
