@@ -1,19 +1,30 @@
 #ifdef _WIN32
 #include <winsock2.h>
+#elif defined(__native_client__)
+#include <stdint.h>
+uint32_t ntohl(uint32_t hl)
+{
+   return (((hl&0x000000FF)<<24)+((hl&0x0000FF00)<<8)+
+   ((hl&0x00FF0000)>>8)+((hl&0xFF000000)>>24));
+}
+uint32_t htonl(uint32_t nl)
+{
+   return (((nl&0x000000FF)<<24)+((nl&0x0000FF00)<<8)+
+   ((nl&0x00FF0000)>>8)+((nl&0xFF000000)>>24));
+}
 #else
 #include <netinet/in.h>
 #endif
 
 #include <assert.h>
-#include <inttypes.h>
+#include <boost/cstdint.hpp>
 
 #include <stdio.h>
 
 #include <stack>
 #include <vector>
 
-#include "SDL.h"
-
+#include "graphics.hpp"
 #include "asserts.hpp"
 #include "controls.hpp"
 #include "foreach.hpp"
@@ -21,6 +32,7 @@
 #include "key.hpp"
 #include "level_runner.hpp"
 #include "multiplayer.hpp"
+#include "preferences.hpp"
 #include "iphone_controls.hpp"
 
 namespace controls {
@@ -64,6 +76,14 @@ SDLKey sdlk[NUM_CONTROLS] = {
 #endif
 };
 
+//If any of these keys are held, we ignore other keyboard input.
+SDLKey control_keys[] = {
+	SDLK_LCTRL,
+	SDLK_RCTRL,
+	SDLK_LALT,
+	SDLK_RALT,
+};
+
 int32_t our_highest_confirmed() {
 	int32_t res = -1;
 	for(int n = 0; n != nplayers; ++n) {
@@ -81,6 +101,30 @@ CKey& keyboard() {
 }
 }
 
+struct control_backup_scope_impl {
+	std::vector<unsigned char> controls[MAX_PLAYERS];
+	int32_t highest_confirmed[MAX_PLAYERS];
+	int starting_cycles;
+};
+
+control_backup_scope::control_backup_scope() : impl_(new control_backup_scope_impl)
+{
+	impl_->starting_cycles = starting_cycles;
+	for(int n = 0; n != MAX_PLAYERS; ++n) {
+		impl_->controls[n] = controls[n];
+		impl_->highest_confirmed[n] = highest_confirmed[n];
+	}
+}
+
+control_backup_scope::~control_backup_scope()
+{
+	starting_cycles = impl_->starting_cycles;
+	for(int n = 0; n != MAX_PLAYERS; ++n) {
+		controls[n] = impl_->controls[n];
+		highest_confirmed[n] = impl_->highest_confirmed[n];
+	}
+}
+
 int their_highest_confirmed() {
 	int32_t res = -1;
 	for(int n = 0; n != nplayers; ++n) {
@@ -146,18 +190,53 @@ void ignore_current_keypresses()
 	}
 }
 
+void read_until(int ncycle)
+{
+	if(local_player < 0 || local_player >= nplayers) {
+		return;
+	}
+
+	while(controls[local_player].size() <= ncycle) {
+		read_local_controls();
+	}
+
+	while(controls[local_player].size() > ncycle+1) {
+		unread_local_controls();
+	}
+}
+
+int local_controls_end()
+{
+	if(local_player < 0 || local_player >= nplayers) {
+		return 0;
+	}
+
+	return controls[local_player].size();
+}
+
 void read_local_controls()
 {
 	if(local_player < 0 || local_player >= nplayers) {
 		return;
 	}
 
-	iphone_controls::read_controls();
+	if(preferences::no_iphone_controls() == false && level::current().allow_touch_controls() == true) {
+		iphone_controls::read_controls();
+	}
 
 	unsigned char state = 0;
 	if(local_control_locks.empty()) {
+#if !defined(__ANDROID__)
+		bool ignore_keypresses = false;
+		foreach(const SDLKey& k, control_keys) {
+			if(keyboard()[k]) {
+				ignore_keypresses = true;
+				break;
+			}
+		}
+
 		for(int n = 0; n < NUM_CONTROLS; ++n) {
-			if(keyboard()[sdlk[n]]) {
+			if(keyboard()[sdlk[n]] && !ignore_keypresses) {
 				if(!key_ignore[n]) {
 					state |= (1 << n);
 				}
@@ -165,14 +244,27 @@ void read_local_controls()
 				key_ignore[n] = false;
 			}
 		}
+#endif
 
-		if(joystick::up() || iphone_controls::up()) { state |= (1 << CONTROL_UP); }
-		if(joystick::down() || iphone_controls::down()) { state |= (1 << CONTROL_DOWN); }
-		if(joystick::left() || iphone_controls::left()) { state |= (1 << CONTROL_LEFT); }
-		if(joystick::right() || iphone_controls::right()) { state |= (1 << CONTROL_RIGHT); }
-		if(joystick::button(0) || iphone_controls::attack()) { state |= (1 << CONTROL_ATTACK); }
-		if(joystick::button(1) || iphone_controls::jump()) { state |= (1 << CONTROL_JUMP); }
-		if(joystick::button(2) || iphone_controls::tongue()) { state |= (1 << CONTROL_TONGUE); }
+		if(preferences::no_iphone_controls() == false && level::current().allow_touch_controls() == true) {
+			if(iphone_controls::up()) { state |= (1 << CONTROL_UP);}
+			if(iphone_controls::down()) { state |= (1 << CONTROL_DOWN);}
+			if(iphone_controls::left()) { state |= (1 << CONTROL_LEFT);}
+			if(iphone_controls::right()) { state |= (1 << CONTROL_RIGHT);}
+			if(iphone_controls::attack()) { state |= (1 << CONTROL_ATTACK);}
+			if(iphone_controls::jump()) { state |= (1 << CONTROL_JUMP);}
+			if(iphone_controls::tongue()) { state |= (1 << CONTROL_TONGUE);}
+		}
+
+#if !defined(__ANDROID__)
+		if(joystick::up()) { state |= (1 << CONTROL_UP);}
+		if(joystick::down()) { state |= (1 << CONTROL_DOWN);}
+		if(joystick::left()) { state |= (1 << CONTROL_LEFT);}
+		if(joystick::right()) { state |= (1 << CONTROL_RIGHT);}
+		if(joystick::button(0)) { state |= (1 << CONTROL_ATTACK);}
+		if(joystick::button(1)) { state |= (1 << CONTROL_JUMP);}
+		if(joystick::button(2)) { state |= (1 << CONTROL_TONGUE);}
+#endif
 	} else {
 		//we have the controls locked into a specific state.
 		state = local_control_locks.top();
@@ -195,6 +287,16 @@ void read_local_controls()
 	}
 }
 
+void unread_local_controls()
+{
+	if(local_player < 0 || local_player >= nplayers || controls[local_player].empty()) {
+		return;
+	}
+
+	controls[local_player].pop_back();
+	highest_confirmed[local_player]--;
+}
+
 void get_control_status(int cycle, int player, bool* output)
 {
 	--cycle;
@@ -214,9 +316,11 @@ void get_control_status(int cycle, int player, bool* output)
 			const int end_time = SDL_GetTicks() + max_delay;
 
 			const pause_scope pause;
+#if !defined(__native_client__)
 			while(cycle > highest_confirmed[player] && SDL_GetTicks() < end_time) {
 				multiplayer::receive();
 			}
+#endif
 		}
 
 		if(cycle > highest_confirmed[player]) {
@@ -301,7 +405,7 @@ void read_control_packet(const char* buf, size_t len)
 	buf += 4;
 
 	if(end_buf - buf != ncycles) {
-		fprintf(stderr, "ERROR: BAD NUMBER OF CYCLES: %d vs %d\n", ncycles, (end_buf - buf));
+		fprintf(stderr, "ERROR: BAD NUMBER OF CYCLES: %d vs %d\n", (int)ncycles, (int)(end_buf - buf));
 		return;
 	}
 
