@@ -1,14 +1,18 @@
 #ifndef CUSTOM_OBJECT_HPP_INCLUDED
 #define CUSTOM_OBJECT_HPP_INCLUDED
 
+#include <set>
+
+#include <boost/scoped_ptr.hpp>
 #include <boost/shared_ptr.hpp>
-#include <inttypes.h>
+#include <boost/cstdint.hpp>
 #include <stack>
 
 #include "blur.hpp"
 #include "color_utils.hpp"
 #include "custom_object_type.hpp"
 #include "decimal.hpp"
+#include "draw_scene.hpp"
 #include "entity.hpp"
 #include "formula.hpp"
 #include "formula_callable.hpp"
@@ -17,9 +21,10 @@
 #include "particle_system.hpp"
 #include "raster_distortion.hpp"
 #include "variant.hpp"
-#include "wml_node_fwd.hpp"
+#include "vector_text.hpp"
+#include "widget.hpp"
 
-class collision_info;
+struct collision_info;
 class level;
 
 struct custom_object_text;
@@ -27,17 +32,29 @@ struct custom_object_text;
 class custom_object : public entity
 {
 public:
+	static const std::string* current_debug_error();
+	static void reset_current_debug_error();
+
+	static std::set<custom_object*>& get_all();
+	static std::set<custom_object*>& get_all(const std::string& type);
 	static void init();
 
-	explicit custom_object(wml::const_node_ptr node);
+	static void run_garbage_collection();
+
+	explicit custom_object(variant node);
 	custom_object(const std::string& type, int x, int y, bool face_right);
 	custom_object(const custom_object& o);
 	virtual ~custom_object();
-	virtual wml::node_ptr write() const;
+
+	//finish_loading(): called when a level finishes loading all objects,
+	//and allows us to do any final setup such as finding our parent.
+	void finish_loading();
+	virtual variant write() const;
 	virtual void setup_drawing() const;
-	virtual void draw() const;
+	virtual void draw(int x, int y) const;
 	virtual void draw_group() const;
 	virtual void process(level& lvl);
+	virtual void create_object();
 	void set_level(level& lvl) { }
 
 	int parallax_scale_millis_x() const {
@@ -91,7 +108,7 @@ public:
 	virtual bool is_active(const rect& screen_area) const;
 	bool dies_on_inactive() const;
 	bool always_active() const;
-	void move_to_standing(level& lvl);
+	bool move_to_standing(level& lvl, int max_displace=10000);
 
 	bool body_harmful() const;
 	bool body_passthrough() const;
@@ -106,7 +123,7 @@ public:
 	int surface_friction() const;
 	int surface_traction() const;
 
-	wml::const_node_ptr get_child(const std::string& key) const {
+	variant get_child(const std::string& key) const {
 		return type_->get_child(key);
 	}
 
@@ -123,11 +140,16 @@ public:
 	std::string debug_description() const;
 
 	void map_entities(const std::map<entity_ptr, entity_ptr>& m);
+	void cleanup_references();
 
 	void add_particle_system(const std::string& key, const std::string& type);
 	void remove_particle_system(const std::string& key);
 
-	void set_text(const std::string& text, const std::string& font, int size, bool centered);
+	void set_text(const std::string& text, const std::string& font, int size, int align);
+	void add_vector_text(const gui::vector_text_ptr& txtp) {
+		vector_text_.push_back(txtp);
+	}
+	void clear_vector_text() { vector_text_.clear(); }
 
 	virtual int hitpoints() const { return hitpoints_; }
 
@@ -141,12 +163,20 @@ public:
 
 	void set_driver_position();
 
+	virtual bool use_absolute_screen_coordinates() const { return use_absolute_screen_coordinates_; }
+
 	virtual int current_animation_id() const { return current_animation_id_; }
 
+	virtual bool handle_sdl_event(const SDL_Event& event, bool claimed);
+#ifndef NO_EDITOR
 	virtual const_editor_entity_info_ptr editor_info() const;
+#endif // !NO_EDITOR
 
-	virtual void handle_event(const std::string& event, const formula_callable* context=NULL);
-	virtual void handle_event(int event, const formula_callable* context=NULL);
+	virtual bool handle_event(const std::string& event, const formula_callable* context=NULL);
+	virtual bool handle_event(int event, const formula_callable* context=NULL);
+	virtual bool handle_event_delay(int event, const formula_callable* context=NULL);
+
+	virtual void resolve_delayed_events();
 
 	virtual bool serializable() const;
 
@@ -156,6 +186,8 @@ public:
 	
 	bool execute_command(const variant& var);
 
+	virtual game_logic::formula_ptr create_formula(const variant& v);
+
 	bool allow_level_collisions() const;
 
 	//statistic on how many FFL events are handled every second.
@@ -173,7 +205,28 @@ public:
 
 	void set_difficulty(int min, int max) { min_difficulty_ = min; max_difficulty_ = max; }
 
+	void update_type(const_custom_object_type_ptr old_type,
+	                 const_custom_object_type_ptr new_type);
+
+	bool mouse_event_swallowed() const {return swallow_mouse_event_;}
+	void reset_mouse_event() {swallow_mouse_event_ = false;}
+	void add_widget(const gui::widget_ptr& w) { widgets_.push_back(w); std::sort(widgets_.begin(), widgets_.end(), gui::widget_sort_zorder()); }
+	void add_widgets(std::vector<gui::widget_ptr>* widgets) { widgets_.swap(*widgets); std::sort(widgets_.begin(), widgets_.end(), gui::widget_sort_zorder()); }
+	void clear_widgets() { widgets_.clear(); }
+	gui::widget_ptr get_widget_by_id(const std::string& id);
+	bool get_clip_area(rect* clip_area) {
+		if(clip_area_ && clip_area) {
+			*clip_area = *clip_area_.get();
+			return true;
+		}
+		return false;
+	}
+
 protected:
+	//components of per-cycle process() that can be done even on
+	//static objects.
+	void static_process(level& lvl);
+
 	virtual void control(const level& lvl);
 	variant get_value(const std::string& key) const;
 	variant get_value_by_slot(int slot) const;
@@ -195,24 +248,51 @@ protected:
 
 	void set_parent(entity_ptr e, const std::string& pivot_point);
 
-	virtual int parent_depth(int cur_depth=0) const;
+	virtual int parent_depth(bool* has_human_parent=NULL, int cur_depth=0) const;
+
+	virtual bool editor_force_standing() const;
+
+	virtual const game_logic::formula_callable_definition* get_definition() const;
+
+	entity_ptr standing_on() const { return standing_on_; }
+	virtual void add_to_level();
+
+	virtual rect platform_rect_at(int xpos) const;
+	virtual int platform_slope_at(int xpos) const;
+
+	virtual bool solid_platform() const;
+
 
 private:
 	custom_object& operator=(const custom_object& o);
 	struct Accessor;
 
+	struct gc_object_reference {
+		entity* owner;
+		entity* target;
+		variant* from_variant;
+		entity_ptr* from_ptr;
+	};
+
+	void extract_gc_object_references(std::vector<gc_object_reference>& v);
+	void extract_gc_object_references(entity_ptr& e, std::vector<gc_object_reference>& v);
+	void extract_gc_object_references(variant& var, std::vector<gc_object_reference>& v);
+	static void restore_gc_object_reference(gc_object_reference ref);
+
+	bool move_to_standing_internal(level& lvl, int max_displace);
+
 	void process_frame();
 
-	const solid_info* calculate_solid() const;
-	const solid_info* calculate_platform() const;
+	const_solid_info_ptr calculate_solid() const;
+	const_solid_info_ptr calculate_platform() const;
 
-	void get_inputs(std::vector<game_logic::formula_input>* inputs) const;
+	virtual void get_inputs(std::vector<game_logic::formula_input>* inputs) const;
 
 	int slope_standing_on(int range) const;
 
 	int previous_y_;
 
-	wml::const_node_ptr custom_type_;
+	variant custom_type_;
 	const_custom_object_type_ptr type_; //the type after variations are applied
 	const_custom_object_type_ptr base_type_; //the type without any variation
 	std::vector<std::string> current_variation_;
@@ -237,6 +317,8 @@ private:
 	bool has_feet_;
 
 	int invincible_;
+
+	bool use_absolute_screen_coordinates_;
 	
 	int sound_volume_;	//see sound.cpp; valid values are 0-128, note that this affects all sounds spawned by this object
 
@@ -251,6 +333,8 @@ private:
 
 	int cycle_;
 
+	bool created_;
+
 	//variable which is always set to false on construction, and then the
 	//first time process is called will fire the on_load event and set to false
 	bool loaded_;
@@ -278,6 +362,8 @@ private:
 	typedef boost::shared_ptr<custom_object_text> custom_object_text_ptr;
 	custom_object_text_ptr text_;
 
+	std::vector<gui::vector_text_ptr> vector_text_;
+
 	entity_ptr driver_;
 
 	boost::shared_ptr<blur_info> blur_;
@@ -286,11 +372,12 @@ private:
 	//at the end of every cycle.
 	int fall_through_platforms_;
 
+#ifdef USE_GLES2
 	//current shader we're using to draw with.
-	std::vector<std::string> fragment_shaders_, vertex_shaders_;
-	mutable GLuint shader_;
-
-	mutable game_logic::map_formula_callable_ptr shader_vars_;
+	gles2::shader_ptr shader_;
+	// List of shader effects to run.
+	std::vector<gles2::shader_ptr> effects_;
+#endif
 
 	bool always_active_;
 
@@ -299,8 +386,9 @@ private:
 	int last_cycle_active_;
 
 	struct position_schedule {
-		position_schedule() : speed(1) {}
-		int speed;
+		position_schedule() : speed(1), base_cycle(0), expires(false) {}
+		int speed, base_cycle;
+		bool expires;
 		std::vector<int> x_pos;
 		std::vector<int> y_pos;
 		std::vector<decimal> rotation;
@@ -315,6 +403,9 @@ private:
 
 	point parent_position() const;
 
+	//storage of the parent object while we're loading the object still.
+	variant parent_loading_;
+
 	entity_ptr parent_;
 	std::string parent_pivot_;
 	int parent_prev_x_, parent_prev_y_;
@@ -322,7 +413,24 @@ private:
 
 	int min_difficulty_, max_difficulty_;
 
+	boost::shared_ptr<const std::vector<frame::CustomPoint> > custom_draw_;
+
 	void set_platform_area(const rect& area);
+
+	std::vector<int> platform_offsets_;
+
+	bool swallow_mouse_event_;
+
+	bool handle_event_internal(int event, const formula_callable* context, bool execute_commands_now=true);
+	std::vector<variant> delayed_commands_;
+
+	int currently_handling_die_event_;
+
+	std::vector<gui::widget_ptr> widgets_;
+
+	rect previous_water_bounds_;
+
+	mutable screen_position adjusted_draw_position_;
 };
 
 #endif
