@@ -16,19 +16,21 @@
 #include "texture.hpp"
 #include "widget.hpp"
 
-#include <boost/function.hpp>
-
 #include <string>
 #include <vector>
 
 namespace gui {
 
-class dialog : public widget, public input::listener_container
+std::string get_dialog_file(const std::string& fname);
+void reset_dialog_paths();
+
+class dialog : public virtual widget, public virtual input::listener_container
 {
 public:
 	typedef std::vector<widget_ptr>::const_iterator child_iterator;
 
-	dialog(int x, int y, int w, int h);
+	explicit dialog(int x, int y, int w, int h);
+	explicit dialog(const variant& v, game_logic::formula_callable* e);
 	virtual ~dialog();
 	virtual void show_modal();
 	void show();
@@ -55,6 +57,14 @@ public:
     bool process_event(const SDL_Event& e, bool claimed);
 	
 	void set_on_quit(boost::function<void ()> onquit) { on_quit_ = onquit; }
+
+	void set_background_frame(const std::string& id) { background_framed_gui_element_ = id; }
+	void set_draw_background_fn(boost::function<void()> fn) { draw_background_fn_ = fn; }
+
+	virtual bool has_focus() const;
+	void set_process_hook(boost::function<void()> fn) { on_process_ = fn; }
+	static void draw_last_scene();
+	virtual widget_ptr get_widget_by_id(const std::string& id);
 protected:
 	virtual bool handle_event(const SDL_Event& event, bool claimed);
 	virtual bool handle_event_children(const SDL_Event& event, bool claimed);
@@ -65,6 +75,11 @@ protected:
 	void set_clear_bg(bool clear) { clear_bg_ = clear; };
 	void set_clear_bg_amount(int amount) { clear_bg_ = amount; }
 	int clear_bg() const { return clear_bg_; };
+
+	virtual void set_value(const std::string& key, const variant& v);
+	virtual variant get_value(const std::string& key) const;
+
+	virtual void handle_process();
 private:
 	std::vector<widget_ptr> widgets_;
 	bool opened_;
@@ -73,6 +88,10 @@ private:
 	
 	boost::function<void ()> on_quit_;
 
+	void quit_delegate();
+
+	game_logic::formula_ptr ffl_on_quit_;
+
 	//default padding between widgets
 	int padding_;
 
@@ -83,9 +102,12 @@ private:
 	mutable GLfloat bg_alpha_;
 
 	int last_draw_;
+
+	std::string background_framed_gui_element_;
+	boost::function<void()> draw_background_fn_;
 };
 
-typedef boost::shared_ptr<dialog> dialog_ptr;
+typedef boost::intrusive_ptr<dialog> dialog_ptr;
 
 }
 
