@@ -1,11 +1,5 @@
-#ifdef TARGET_PANDORA
-#include <GLES/gl.h>
-#include <GLES/glues.h>
-#else
-#include <GL/gl.h>
-#include <GL/glu.h>
-#endif
-#include <SDL.h>
+#include "graphics.hpp"
+
 #include <algorithm>
 #include <cmath>
 #include <iostream>
@@ -18,8 +12,10 @@
 #include "debug_console.hpp"
 #include "draw_number.hpp"
 #include "draw_scene.hpp"
+#include "editor.hpp"
 #include "font.hpp"
 #include "foreach.hpp"
+#include "formula_profiler.hpp"
 #include "globals.h"
 #include "graphical_font.hpp"
 #include "gui_section.hpp"
@@ -35,6 +31,9 @@
 
 namespace {
 
+std::vector<rect> current_debug_rects;
+int current_debug_rects_valid_cycle = -1;
+
 std::string& scene_title() {
 	static std::string title;
 	return title;
@@ -68,10 +67,26 @@ screen_position& last_draw_position()
 	return last_position;
 }
 
+namespace {
+int g_flash_disable = 0;
+}
+
+disable_flashes_scope::disable_flashes_scope()
+{
+	++g_flash_disable;
+}
+
+disable_flashes_scope::~disable_flashes_scope()
+{
+	--g_flash_disable;
+}
+
 void screen_color_flash(const graphics::color_transform& color, const graphics::color_transform& color_delta, int duration)
 {
-	screen_flash f = { color, color_delta, duration };
-	flashes().push_back(f);
+	if(!g_flash_disable) {
+		screen_flash f = { color, color_delta, duration };
+		flashes().push_back(f);
+	}
 }
 
 void set_scene_title(const std::string& msg, int duration) {
@@ -88,36 +103,31 @@ void set_displayed_achievement(achievement_ptr a)
 
 GLfloat hp_ratio = -1.0;
 void draw_scene(const level& lvl, screen_position& pos, const entity* focus, bool do_draw) {
+	const bool draw_ready = update_camera_position(lvl, pos, focus, do_draw);
+	if(draw_ready) {
+		render_scene(lvl, pos, focus, do_draw);
+	}
+}
+
+bool update_camera_position(const level& lvl, screen_position& pos, const entity* focus, bool do_draw) {
 	if(focus == NULL && lvl.player()) {
 		focus = &lvl.player()->get_entity();
 	}
 
-	last_position = pos;
-
 	//flag which gets set to false if we abort drawing, due to the
 	//screen position being initialized now.
 	const bool draw_level = do_draw && pos.init;
+	
+#ifndef NO_EDITOR
+	const int sidebar_width = editor::sidebar_width();
+	const int codebar_height = editor::codebar_height();
+#else
+	const int sidebar_width = 0;
+	const int codebar_height = 0;
+#endif
 
-	graphics::prepare_raster();
-	glPushMatrix();
-
-	const int camera_rotation = lvl.camera_rotation();
-	if(camera_rotation) {
-		GLfloat rotate = GLfloat(camera_rotation)/1000.0;
-		glRotatef(rotate, 0.0, 0.0, 1.0);
-	}
-
-	if(pos.flip_rotate) {
-		glClearColor(0.0, 0.0, 0.0, 0.0);
-		glClear(GL_COLOR_BUFFER_BIT);
-
-		const SDL_Surface* fb = SDL_GetVideoSurface();
-		const double angle = sin(0.5*3.141592653589*GLfloat(pos.flip_rotate)/1000.0);
-		const int pixels = (preferences::actual_screen_width()/2)*angle;
-		
-		//then squish all future drawing inwards
-		glViewport(pixels, 0, preferences::actual_screen_width() - pixels*2, preferences::actual_screen_height());
-	}
+	const int screen_width = graphics::screen_width() - (lvl.in_editor() ? sidebar_width : 0);
+	const int screen_height = graphics::screen_height() - (lvl.in_editor() ? codebar_height : 0);
 
 	if(focus) {
 		// If the camera is automatically moved along by the level (e.g. a 
@@ -128,19 +138,19 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 		//find how much padding will have to be on the edge of the screen due
 		//to the level being wider than the screen. This value will be 0
 		//if the level is larger than the screen (i.e. most cases)
-		const int x_screen_pad = std::max<int>(0, graphics::screen_width() - lvl.boundaries().w());
+		const int x_screen_pad = std::max<int>(0, screen_width - lvl.boundaries().w());
 
-		const int y_screen_pad = std::max<int>(0, graphics::screen_height() - lvl.boundaries().h());
+		const int y_screen_pad = std::max<int>(0, screen_height - lvl.boundaries().h());
 
 		//find the boundary values for the camera position based on the size
 		//of the level. These boundaries keep the camera from ever going out
 		//of the bounds of the level.
-		const int min_x = lvl.boundaries().x() + graphics::screen_width()/2 - x_screen_pad/2;
-		const int max_x = lvl.boundaries().x2() - graphics::screen_width()/2 + x_screen_pad/2;
-		const int min_y = lvl.boundaries().y() + graphics::screen_height()/2 - y_screen_pad/2;
-		const int max_y = lvl.boundaries().y2() - graphics::screen_height()/2 + y_screen_pad/2;
+		const int min_x = lvl.boundaries().x() + screen_width/2 - x_screen_pad/2;
+		const int max_x = lvl.boundaries().x2() - screen_width/2 + x_screen_pad/2;
+		const int min_y = lvl.boundaries().y() + screen_height/2 - y_screen_pad/2;
+		const int max_y = lvl.boundaries().y2() - screen_height/2 + y_screen_pad/2;
 
-		//std::cerr << "BOUNDARIES: " << lvl.boundaries().x() << ", " << lvl.boundaries().x2() << " WIDTH: " << graphics::screen_width() << " PAD: " << x_screen_pad << "\n";
+		//std::cerr << "BOUNDARIES: " << lvl.boundaries().x() << ", " << lvl.boundaries().x2() << " WIDTH: " << screen_width << " PAD: " << x_screen_pad << "\n";
 
 		//we look a certain number of frames ahead -- assuming the focus
 		//keeps moving at the current velocity, we converge toward the point
@@ -167,7 +177,7 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 		const int vertical_look = focus->vertical_look();
 
 		//find the y point for the camera to converge toward
-		int y = std::min(std::max(focus->feet_y() - graphics::screen_height()/(5*lvl.zoom_level()) + displacement_y*PredictiveFramesVert + vertical_look, min_y), max_y);
+		int y = std::min(std::max(focus->feet_y() - (screen_height/(5*lvl.zoom_level())).as_int() + displacement_y*PredictiveFramesVert + vertical_look, min_y), max_y);
 
 		//std::cerr << "POSITION: " << x << "," << y << " IN " << min_x << "," << min_y << "," << max_x << "," << max_y << "\n";
 
@@ -187,7 +197,7 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 				}
 
 				const int BorderSize = 20;
-				if(v.size() == 1 || right - left < graphics::screen_width()/lvl.zoom_level() - BorderSize && bottom - top < graphics::screen_height()/lvl.zoom_level() - BorderSize) {
+				if(v.size() == 1 || right - left < screen_width/lvl.zoom_level() - BorderSize && bottom - top < screen_height/lvl.zoom_level() - BorderSize) {
 					break;
 				}
 
@@ -197,7 +207,7 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 			}
 
 			x = std::min(std::max((left + right)/2, min_x), max_x);
-			y = std::min(std::max((top + bottom)/2 - graphics::screen_height()/(5*lvl.zoom_level()), min_y), max_y);
+			y = std::min(std::max(((top + bottom)/2 - screen_height/(5*lvl.zoom_level())).as_int(), min_y), max_y);
 		}
 
 
@@ -211,15 +221,15 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 		//for small screens the speech dialog arrows cover the entities they are
 		//pointing to. adjust to that by looking up a little bit.
 		if (lvl.current_speech_dialog() && preferences::virtual_screen_height() < 600)
-			y = std::min(y + (600 - graphics::screen_height())/(2*lvl.zoom_level()), max_y);
+			y = std::min((y + (600 - screen_height)/(2*lvl.zoom_level())).as_int(), max_y);
 
 		//find the target x,y position of the camera in centi-pixels. Note that
 		//(x,y) represents the position the camera should center on, while
 		//now we're calculating the top-left point.
 		//
 		//the actual camera position will converge toward this point
-		const int target_xpos = 100*(x - graphics::screen_width()/2);
-		const int target_ypos = 100*(y - graphics::screen_height()/2);
+		const int target_xpos = 100*(x - screen_width/2);
+		const int target_ypos = 100*(y - screen_height/2);
 
 		if(pos.init == false) {
 			pos.x = target_xpos;
@@ -230,8 +240,8 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 			//We do this by moving asymptotically toward the target, which
 			//makes the camera have a nice acceleration/decceleration effect
 			//as the target position moves.
-			const int horizontal_move_speed = 30/lvl.zoom_level();
-			const int vertical_move_speed = 10/lvl.zoom_level();
+			const int horizontal_move_speed = (30/lvl.zoom_level()).as_int();
+			const int vertical_move_speed = (10/lvl.zoom_level()).as_int();
 			int xdiff = (target_xpos - pos.x)/horizontal_move_speed;
 			int ydiff = (target_ypos - pos.y)/vertical_move_speed;
 
@@ -281,7 +291,7 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 			}
 		}
 
-		const int target_zoom = lvl.zoom_level();
+		const float target_zoom = lvl.zoom_level().as_float();
 		const float ZoomSpeed = 0.03;
 		if(std::abs(target_zoom - pos.zoom) < ZoomSpeed) {
 			pos.zoom = target_zoom;
@@ -292,24 +302,71 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 		}
 	}
 
-	if(!draw_level) {
-		glPopMatrix();
-		return;
+	last_position = pos;
+
+	return draw_level;
+}
+
+void render_scene(const level& lvl, screen_position& pos, const entity* focus, bool do_draw) {
+		formula_profiler::instrument instrumentation("DRAW");
+#ifndef NO_EDITOR
+	const int sidebar_width = editor::sidebar_width();
+#else
+	const int sidebar_width = 0;
+#endif
+	const int screen_width = graphics::screen_width() - (lvl.in_editor() ? sidebar_width : 0);
+
+	graphics::prepare_raster();
+	glPushMatrix();
+
+	const int camera_rotation = lvl.camera_rotation();
+	if(camera_rotation) {
+		GLfloat rotate = GLfloat(camera_rotation)/1000.0;
+		glRotatef(rotate, 0.0, 0.0, 1.0);
+	}
+
+	if(pos.flip_rotate) {
+		glClearColor(0.0, 0.0, 0.0, 0.0);
+		glClear(GL_COLOR_BUFFER_BIT);
+
+		const SDL_Surface* fb = SDL_GetVideoSurface();
+		const double angle = sin(0.5*3.141592653589*GLfloat(pos.flip_rotate)/1000.0);
+		const int pixels = (preferences::actual_screen_width()/2)*angle;
+		
+		//then squish all future drawing inwards
+		glViewport(pixels, 0, preferences::actual_screen_width() - pixels*2, preferences::actual_screen_height());
 	}
 
 	int xscroll = (pos.x/100)&preferences::xypos_draw_mask;
 	int yscroll = (pos.y/100)&preferences::xypos_draw_mask;
 
-	if(pos.zoom > 1.0) {
+//	if(pos.zoom > 1.0) {
 		glScalef(pos.zoom, pos.zoom, 0);
-		xscroll += (graphics::screen_width()/2)*(-1.0/pos.zoom + 1.0);
+		xscroll += (screen_width/2)*(-1.0/pos.zoom + 1.0);
 		yscroll += (graphics::screen_height()/2)*(-1.0/pos.zoom + 1.0);
-	}
+//	}
 
 	glTranslatef(-xscroll, -yscroll, 0);
 	lvl.draw_background(xscroll, yscroll, camera_rotation);
 
-	lvl.draw(xscroll, yscroll, graphics::screen_width(), graphics::screen_height());
+	int draw_width = screen_width;
+	int draw_height = graphics::screen_height();
+	if(pos.zoom < 1.0) {
+		draw_width /= pos.zoom;
+		draw_height /= pos.zoom;
+	}
+	lvl.draw(xscroll, yscroll, draw_width, draw_height);
+
+	foreach(const rect& r, current_debug_rects) {
+		graphics::draw_rect(r, graphics::color(0, 0, 255, 175));
+	}
+
+	if(current_debug_rects_valid_cycle != lvl.cycle()) {
+		current_debug_rects.clear();
+	}
+
+	current_debug_rects_valid_cycle = lvl.cycle();
+
 	graphics::clear_raster_distortion();
 	glPopMatrix();
 
@@ -332,7 +389,11 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 		}
 	}
 
+#ifndef NO_EDITOR
 	debug_console::draw();
+#endif
+
+	debug_console::draw_graph();
 
 	if (!pause_stack) lvl.draw_status();
 
@@ -406,6 +467,16 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 		GLshort varray1[8] = {0,0,  pixels,0,  pixels,preferences::actual_screen_height(),   0,preferences::actual_screen_height()};
 		GLshort varray2[8] = {preferences::actual_screen_width() - pixels,0,  preferences::actual_screen_width(),0,   preferences::actual_screen_width(),preferences::actual_screen_height(),  preferences::actual_screen_width() - pixels,preferences::actual_screen_height()};
 		glColor4ub(0, 0, 0, 255);
+#if defined(USE_GLES2)
+		gles2::manager gles2_manager(gles2::get_simple_shader());
+		glViewport(0, 0, preferences::actual_screen_width(), preferences::actual_screen_height());
+
+		gles2::active_shader()->shader()->vertex_array(2, GL_SHORT, 0, 0,varray1);
+		glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
+
+		gles2::active_shader()->shader()->vertex_array(2, GL_SHORT, 0, 0, varray2);
+		glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
+#else
 		glDisable(GL_TEXTURE_2D);
 		glDisableClientState(GL_TEXTURE_COORD_ARRAY);		
 		
@@ -417,10 +488,55 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 		glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
 		glEnable(GL_TEXTURE_2D);
 		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+#endif
 		glColor4ub(255, 255, 255, 255);
 	}
 }
 
+namespace {
+boost::intrusive_ptr<performance_data> current_perf_data;
+}
+
+variant performance_data::get_value(const std::string& key) const
+{
+#define PERF_ATTR(m) if(key == #m) return variant(m);
+	PERF_ATTR(fps);
+	PERF_ATTR(cycles_per_second);
+	PERF_ATTR(delay);
+	PERF_ATTR(draw);
+	PERF_ATTR(process);
+	PERF_ATTR(flip);
+	PERF_ATTR(cycle);
+	PERF_ATTR(nevents);
+#undef PERF_ATTR
+
+	return variant();
+}
+
+void performance_data::get_inputs(std::vector<game_logic::formula_input>* inputs) const
+{
+#define PERF_ATTR(m) inputs->push_back(std::string(#m))
+	PERF_ATTR(fps);
+	PERF_ATTR(cycles_per_second);
+	PERF_ATTR(delay);
+	PERF_ATTR(draw);
+	PERF_ATTR(process);
+	PERF_ATTR(flip);
+	PERF_ATTR(cycle);
+	PERF_ATTR(nevents);
+#undef PERF_ATTR
+}
+
+void performance_data::set_current(const performance_data& d)
+{
+	current_perf_data.reset(new performance_data(d));
+}
+
+performance_data* performance_data::current()
+{
+	return current_perf_data.get();
+}
+
 void draw_fps(const level& lvl, const performance_data& data)
 {
 	if(!preferences::debug()) {
@@ -448,3 +564,8 @@ void draw_fps(const level& lvl, const performance_data& data)
 		font->draw(10, area.y2() + 5, data.profiling_info);
 	}
 }
+
+void add_debug_rect(const rect& r)
+{
+	current_debug_rects.push_back(r);
+}
