@@ -1,5 +1,6 @@
 #ifndef EDITOR_FORMULA_FUNCTIONS_HPP_INCLUDED
 #define EDITOR_FORMULA_FUNCTIONS_HPP_INCLUDED
+#ifndef NO_EDITOR
 
 #include <string>
 #include <vector>
@@ -18,4 +19,5 @@ void execute(const std::string& id, editor& e);
 
 }
 
+#endif // !NO_EDITOR
 #endif
