@@ -1,4 +1,5 @@
-#include "SDL.h"
+#ifndef NO_EDITOR
+#include "graphics.hpp"
 
 #include <boost/bind.hpp>
 
@@ -17,25 +18,59 @@
 #include "label.hpp"
 #include "raster.hpp"
 #include "stats.hpp"
-#include "text_entry_widget.hpp"
+#include "text_editor_widget.hpp"
 
 namespace editor_dialogs
 {
 
 namespace {
-void set_segmented_level(level* lvl, bool value)
+
+void set_segmented_level_width(editor_level_properties_dialog* d, editor* e, bool value)
 {
-	if(value) {
-		lvl->set_segment_width(lvl->boundaries().w());
-	} else {
-		lvl->set_segment_width(0);
+	foreach(level_ptr lvl, e->get_level_list()) {
+		if(value) {
+			//make sure the segment width is divisible by the tile size.
+			int width = lvl->boundaries().w();
+			while(width%32) {
+				++width;
+			}
+			lvl->set_segment_width(width);
+			lvl->set_boundaries(rect(lvl->boundaries().x(), lvl->boundaries().y(),
+			                         width, lvl->boundaries().h()));
+		} else {
+			lvl->set_segment_width(0);
+		}
 	}
+
+	d->init();
 }
+
+void set_segmented_level_height(editor_level_properties_dialog* d, editor* e, bool value)
+{
+	foreach(level_ptr lvl, e->get_level_list()) {
+		if(value) {
+			//make sure the segment height is divisible by the tile size.
+			int height = lvl->boundaries().h();
+			while(height%32) {
+				++height;
+			}
+			lvl->set_segment_height(height);
+			lvl->set_boundaries(rect(lvl->boundaries().x(), lvl->boundaries().y(),
+			                         lvl->boundaries().w(), height));
+		} else {
+			lvl->set_segment_height(0);
+		}
+	}
+	
+	d->init();
+}
+
 }
 
 editor_level_properties_dialog::editor_level_properties_dialog(editor& e)
   : dialog(0, 0, graphics::screen_width(), graphics::screen_height()), editor_(e)
 {
+	set_clear_bg_amount(255);
 	init();
 }
 
@@ -46,9 +81,14 @@ void editor_level_properties_dialog::init()
 
 	add_widget(widget_ptr(new label("Level Properties", graphics::color_white(), 48)), 10, 10);
 
+	text_editor_widget* change_title_entry(new text_editor_widget(200, 30));
+	change_title_entry->set_text(editor_.get_level().title());
+	change_title_entry->set_on_change_handler(boost::bind(&editor_level_properties_dialog::change_title, this, change_title_entry));
+	change_title_entry->set_on_enter_handler(boost::bind(&dialog::close, this));
+
 	grid_ptr g(new grid(2));
-	g->add_col(widget_ptr(new label(editor_.get_level().title(), graphics::color_white(), 36)))
-	  .add_col(widget_ptr(new button(widget_ptr(new label("Change Title", graphics::color_white())), boost::bind(&editor_level_properties_dialog::change_title, this))));
+	g->add_col(widget_ptr(new label("Change Title", graphics::color_white(), 36)))
+	  .add_col(widget_ptr(change_title_entry));
 
 	add_widget(g);
 
@@ -72,29 +112,30 @@ void editor_level_properties_dialog::init()
 	g->add_col(widget_ptr(new button(widget_ptr(new label("Set", graphics::color_white())), boost::bind(&editor_level_properties_dialog::change_previous_level, this))));
 	add_widget(g);
 
-	checkbox* segmented_checkbox = new checkbox("Segmented Level", editor_.get_level().segment_width() != 0, boost::bind(set_segmented_level, &editor_.get_level(), _1));
-	add_widget(widget_ptr(segmented_checkbox));
-}
+	checkbox* hz_segmented_checkbox = new checkbox("Horizontally Segmented Level", editor_.get_level().segment_width() != 0, boost::bind(set_segmented_level_width, this, &editor_, _1));
+	widget_ptr hz_checkbox(hz_segmented_checkbox);
+	add_widget(hz_checkbox);
 
-void editor_level_properties_dialog::change_title()
-{
-	using namespace gui;
-	dialog d(0, 0, graphics::screen_width(), graphics::screen_height());
-	d.add_widget(widget_ptr(new label("Change Title", graphics::color_white(), 48)));
-	text_entry_widget* entry = new text_entry_widget;
-	d.add_widget(widget_ptr(new label("Name:", graphics::color_white())))
-	 .add_widget(widget_ptr(entry));
-	d.show_modal();
+	checkbox* vt_segmented_checkbox = new checkbox("Vertically Segmented Level", editor_.get_level().segment_height() != 0, boost::bind(set_segmented_level_height, this, &editor_, _1));
+	widget_ptr vt_checkbox(vt_segmented_checkbox);
+	add_widget(vt_checkbox);
 
-	if(d.cancelled()) {
-		return;
+	if(editor_.get_level().segment_height() != 0) {
+		remove_widget(hz_checkbox);
 	}
 
-	std::string title = entry->text();
+	if(editor_.get_level().segment_width() != 0) {
+		remove_widget(vt_checkbox);
+	}
+}
 
-	editor_.get_level().set_title(title);
+void editor_level_properties_dialog::change_title(const gui::text_editor_widget_ptr editor)
+{
+	std::string title = editor->text();
 
-	init();
+	foreach(level_ptr lvl, editor_.get_level_list()) {
+		lvl->set_title(title);
+	}
 }
 
 void editor_level_properties_dialog::change_background()
@@ -139,7 +180,10 @@ void editor_level_properties_dialog::execute_change_background(const std::vector
 		return;
 	}
 
-	editor_.get_level().set_background_by_id(choices[index]);
+	foreach(level_ptr lvl, editor_.get_level_list()) {
+		lvl->set_background_by_id(choices[index]);
+	}
+
 	init();
 }
 
@@ -147,7 +191,9 @@ void editor_level_properties_dialog::change_next_level()
 {
 	std::string result = show_choose_level_dialog("Next Level");
 	if(result.empty() == false) {
-		editor_.get_level().set_next_level(result);
+		foreach(level_ptr lvl, editor_.get_level_list()) {
+			lvl->set_next_level(result);
+		}
 	}
 
 	init();
@@ -157,10 +203,14 @@ void editor_level_properties_dialog::change_previous_level()
 {
 	std::string result = show_choose_level_dialog("Previous Level");
 	if(result.empty() == false) {
-		editor_.get_level().set_previous_level(result);
+		foreach(level_ptr lvl, editor_.get_level_list()) {
+			lvl->set_previous_level(result);
+		}
 	}
 
 	init();
 }
 
 }
+#endif // !NO_EDITOR
+
