@@ -14,10 +14,13 @@
 #include "geometry.hpp"
 #include "key.hpp"
 #include "light.hpp"
-#include "powerup_fwd.hpp"
 #include "solid_map_fwd.hpp"
 #include "wml_formula_callable.hpp"
-#include "wml_node_fwd.hpp"
+#include "variant.hpp"
+
+namespace game_logic {
+class formula_callable_definition;
+}
 
 class character;
 class frame;
@@ -30,13 +33,17 @@ typedef boost::intrusive_ptr<character> character_ptr;
 class entity : public game_logic::wml_serializable_formula_callable
 {
 public:
-	static entity_ptr build(wml::const_node_ptr node);
-	explicit entity(wml::const_node_ptr node);
+	static entity_ptr build(variant node);
+	explicit entity(variant node);
 	entity(int x, int y, bool face_right);
 	virtual ~entity() {}
-	virtual wml::node_ptr write() const = 0;
+
+	virtual void add_to_level();
+
+	virtual void finish_loading() {}
+	virtual variant write() const = 0;
 	virtual void setup_drawing() const {}
-	virtual void draw() const = 0;
+	virtual void draw(int x, int y) const = 0;
 	virtual void draw_group() const = 0;
 	player_info* get_player_info() { return is_human(); }
 	const player_info* get_player_info() const { return is_human(); }
@@ -89,11 +96,14 @@ public:
 
 	virtual bool point_collides(int x, int y) const = 0;
 	virtual bool rect_collides(const rect& r) const = 0;
-	const solid_info* platform() const { return platform_; }
-	const solid_info* solid() const { return solid_; }
+	const solid_info* platform() const { return platform_.get(); }
+	const solid_info* solid() const { return solid_.get(); }
 	const rect& solid_rect() const { return solid_rect_; }
 	const rect& frame_rect() const { return frame_rect_; }
 	rect platform_rect() const { return platform_rect_; }
+	virtual rect platform_rect_at(int xpos) const { return platform_rect(); }
+	virtual int platform_slope_at(int xpos) const { return 0; }
+	virtual bool solid_platform() const { return false; }
 	rect body_rect() const;
 	rect hit_rect() const;
 	point midpoint() const;
@@ -114,6 +124,7 @@ public:
 
 	void set_platform_motion_x(int value);
 	int platform_motion_x() const;
+	int map_platform_pos(int xpos) const;
 
 	bool face_right() const { return face_right_; }
 	virtual void set_face_right(bool facing);
@@ -132,6 +143,8 @@ public:
 	virtual bool no_move_to_standing() const { return 0; }
 	virtual bool reverse_global_vertical_zordering() const { return 0; }
 
+	virtual entity_ptr standing_on() const = 0;
+
 	virtual void die_with_no_event() = 0;
 	virtual bool is_active(const rect& screen_area) const = 0;
 	virtual bool dies_on_inactive() const { return false; } 
@@ -165,7 +178,9 @@ public:
 
 	void draw_debug_rects() const;
 
+#ifndef NO_EDITOR
 	virtual const_editor_entity_info_ptr editor_info() const { return const_editor_entity_info_ptr(); }
+#endif // !NO_EDITOR
 
 	virtual entity_ptr clone() const { return entity_ptr(); }
 	virtual entity_ptr backup() const = 0;
@@ -175,8 +190,10 @@ public:
 	virtual game_logic::const_formula_ptr get_event_handler(int key) const { return game_logic::const_formula_ptr(); }
 	virtual void set_event_handler(int, game_logic::const_formula_ptr f) { return; }
 
-	virtual void handle_event(const std::string& id, const formula_callable* context=NULL) {}
-	virtual void handle_event(int id, const formula_callable* context=NULL) {}
+	virtual bool handle_event(const std::string& id, const formula_callable* context=NULL) { return false; }
+	virtual bool handle_event(int id, const formula_callable* context=NULL) { return false; }
+	virtual bool handle_event_delay(int id, const formula_callable* context=NULL) { return false; }
+	virtual void resolve_delayed_events() = 0;
 
 	//function which returns true if this object can be 'interacted' with.
 	//i.e. if the player ovelaps with the object and presses up if they will
@@ -191,31 +208,19 @@ public:
 	//that we hold, and map them according to the mapping given. This is useful
 	//when we back up an entire level and want to make references match.
 	virtual void map_entities(const std::map<entity_ptr, entity_ptr>& m) {}
+	virtual void cleanup_references() {}
 
 	void add_scheduled_command(int cycle, variant cmd);
-	variant get_scheduled_command(int cycle);
+	std::vector<variant> pop_scheduled_commands();
 
 	virtual void save_game() {}
 
 	virtual entity_ptr driver() { return entity_ptr(); }
 	virtual const_entity_ptr driver() const { return const_entity_ptr(); }
 
-	virtual void move_to_standing(level& lvl) {}
+	virtual bool move_to_standing(level& lvl, int max_displace=10000) { return false; }
 	virtual int hitpoints() const { return 1; }
 	virtual int max_hitpoints() const { return 1; }
-	virtual int num_powerups() const { return 0; }
-
-	virtual void get_powerup(const std::string& id) {}
-	virtual void get_powerup(const_powerup_ptr powerup) {}
-	virtual void remove_powerup() {}
-	virtual int remove_powerup(const_powerup_ptr powerup) { return 0; }
-	virtual const std::vector<const_powerup_ptr>& powerups() const;
-	virtual const std::vector<const_powerup_ptr>& abilities() const;
-
-	//function to perform preloading of a powerup so it'll be ready to apply
-	//to this type of object when needed
-	virtual void preload_powerup(const_powerup_ptr powerup) {}
-	virtual bool is_powerup_loaded(const_powerup_ptr powerup) const { return true; }
 
 	void set_control_status(const std::string& key, bool value);
 	void set_control_status(controls::CONTROL_ITEM ctrl, bool value) { controls_[ctrl] = value; }
@@ -246,16 +251,36 @@ public:
 	virtual const std::vector<light_ptr>& lights() const = 0;
 	virtual void swap_lights(std::vector<light_ptr>& lights) = 0;
 
-	point pivot(const std::string& name) const;
+	point pivot(const std::string& name, bool reverse_facing=false) const;
 
 	virtual bool appears_at_difficulty(int difficulty) const = 0;
 
-	virtual int parent_depth(int cur_depth=0) const { return 0; }
+	virtual int parent_depth(bool* has_human_parent=NULL, int cur_depth=0) const { return 0; }
+
+	virtual bool editor_force_standing() const = 0;
+
+	void set_spawned_by(const std::string& key);
+	const std::string& spawned_by() const;
+
+	virtual bool mouse_event_swallowed() const {return false;}
+
+	bool is_mouse_over_entity() const { return mouse_over_entity_; }
+	void set_mouse_over_entity(bool val=true) { mouse_over_entity_=val; }
+	void set_mouse_buttons(Uint8 buttons) { mouse_button_state_ = buttons; }
+	Uint8 get_mouse_buttons() const { return mouse_button_state_; }
+	bool is_being_dragged() const { return being_dragged_; }
+	void set_being_dragged(bool val=true) { being_dragged_ = val; }
+	virtual bool get_clip_area(rect* clip_area) = 0;
+
+	virtual const game_logic::formula_callable_definition* get_definition() const = 0;
+
+	virtual void create_object() = 0;
+
+	virtual bool use_absolute_screen_coordinates() const = 0;
 
 protected:
-
-	virtual const solid_info* calculate_solid() const = 0;
-	virtual const solid_info* calculate_platform() const = 0;
+	virtual const_solid_info_ptr calculate_solid() const = 0;
+	virtual const_solid_info_ptr calculate_platform() const = 0;
 	void calculate_solid_rect();
 
 	bool control_status(controls::CONTROL_ITEM ctrl) const { return controls_[ctrl]; }
@@ -274,17 +299,21 @@ protected:
 
 	const std::vector<entity_ptr>& attached_objects() const { return attached_objects_; }
 
-
-private:
 	virtual void control(const level& lvl) = 0;
 
-	wml::node_ptr serialize_to_wml() const { return write(); }
+	variant serialize_to_wml() const { return write(); }
 
+	int prev_feet_x() const { return prev_feet_x_; }
+	int prev_feet_y() const { return prev_feet_y_; }
+
+
+private:
 	std::string label_;
 
 	int x_, y_;
 
 	int prev_feet_x_, prev_feet_y_;
+	int last_move_x_, last_move_y_;
 
 	bool face_right_;
 	bool upside_down_;
@@ -296,6 +325,10 @@ private:
 
 	bool respawn_;
 
+	bool mouse_over_entity_;
+	Uint8 mouse_button_state_;
+	bool being_dragged_;
+
 	unsigned int solid_dimensions_, collide_dimensions_;
 	unsigned int weak_solid_dimensions_, weak_collide_dimensions_;
 
@@ -312,11 +345,19 @@ private:
 	std::vector<entity_ptr> attached_objects_;
 
 	//caches of commonly queried rects.
-	rect solid_rect_, frame_rect_, platform_rect_;
-	const solid_info* solid_;
-	const solid_info* platform_;
+	rect solid_rect_, frame_rect_, platform_rect_, prev_platform_rect_;
+	const_solid_info_ptr solid_;
+	const_solid_info_ptr platform_;
 
 	int platform_motion_x_;
+
+	std::string spawned_by_;
+};
+
+bool zorder_compare(const entity_ptr& e1, const entity_ptr& e2);	
+struct entity_zorder_compare
+{
+	bool operator()(const entity_ptr& lhs, const entity_ptr& rhs);
 };
 
 #endif
