@@ -15,8 +15,6 @@
 
 #include <sstream>
 
-#include "wml_value.hpp"
-
 class formatter
 {
 public:
@@ -38,9 +36,6 @@ public:
 		return stream_.str();
 	}
 
-	operator wml::value() {
-		return wml::value(stream_.str());
-	}
 private:
 	std::ostringstream stream_;
 };
