@@ -8,6 +8,18 @@
 namespace formula_profiler
 {
 
+void dump_instrumentation() {}
+
+class instrument
+{
+public:
+	explicit instrument(const char* id) {}
+	~instrument() {}
+}
+
+//should be called every cycle while the profiler is running.
+void pump();
+
 class manager
 {
 public:
@@ -27,11 +39,33 @@ inline std::string get_profile_summary() { return ""; }
 
 #include <vector>
 
+#if defined(_WINDOWS)
+#include "utils.hpp"
+#else
+#include <sys/time.h>
+#endif
+
 class custom_object_type;
 
 namespace formula_profiler
 {
 
+//instruments inside a given scope.
+class instrument
+{
+public:
+	explicit instrument(const char* id);
+	~instrument();
+private:
+	const char* id_;
+	struct timeval tv_;
+};
+
+void dump_instrumentation();
+
+//should be called every cycle while the profiler is running.
+void pump();
+
 struct custom_object_event_frame {
 	const custom_object_type* type;
 	int event_id;
@@ -50,6 +84,8 @@ public:
 	~manager();
 };
 
+void end_profiling();
+
 class suspend_scope
 {
 public:
