@@ -15,6 +15,7 @@
 #define FORMULA_TOKENIZER_HPP_INCLUDED
 
 #include <string>
+#include <vector>
 
 namespace formula_tokenizer
 {
@@ -29,7 +30,8 @@ enum FFL_TOKEN_TYPE { TOKEN_OPERATOR, TOKEN_STRING_LITERAL,
 				  TOKEN_LBRACKET, TOKEN_RBRACKET,
 				  TOKEN_COMMA, TOKEN_SEMICOLON, TOKEN_COLON, 
 				  TOKEN_WHITESPACE, TOKEN_KEYWORD,
-				  TOKEN_COMMENT, TOKEN_POINTER,
+				  TOKEN_COMMENT, TOKEN_POINTER, TOKEN_LEFT_POINTER,
+				  TOKEN_PIPE,
 				  TOKEN_INVALID  };
 
 struct token {
@@ -39,7 +41,28 @@ struct token {
 
 token get_token(iterator& i1, iterator i2);
 
-struct token_error {};
+struct token_error {
+	token_error(const std::string& m);
+	std::string msg;
+};
+
+//A special interface for searching for and matching tokens.
+class token_matcher {
+public:
+	token_matcher();
+	explicit token_matcher(FFL_TOKEN_TYPE type);
+	token_matcher& add(FFL_TOKEN_TYPE type);
+	token_matcher& add(const std::string& str);
+
+	bool match(const token& t) const;
+
+	//Find the first matching token within the given range and return it.
+	//Does not return tokens that are inside any kinds of brackets.
+	bool find_match(const token*& i1, const token* i2) const;
+private:
+	std::vector<FFL_TOKEN_TYPE> types_;
+	std::vector<std::string> str_;
+};
 
 }
 
