@@ -8,14 +8,14 @@
 #include "frame.hpp"
 #include "object_events.hpp"
 #include "raster.hpp"
+#include "rectangle_rotator.hpp"
 #include "solid_map.hpp"
 #include "sound.hpp"
 #include "string_utils.hpp"
 #include "surface_formula.hpp"
 #include "surface_palette.hpp"
 #include "texture.hpp"
-#include "wml_node.hpp"
-#include "wml_utils.hpp"
+#include "variant_utils.hpp"
 
 namespace {
 std::set<frame*>& palette_frames() {
@@ -26,62 +26,63 @@ std::set<frame*>& palette_frames() {
 unsigned int current_palette_mask = 0;
 }
 
-frame::frame(wml::const_node_ptr node)
-   : id_(node->attr("id")),
-     image_(node->attr("image")),
+frame::frame(variant node)
+   : id_(node["id"].as_string()),
+     image_(node["image"].as_string()),
      variant_id_(id_),
      enter_event_id_(get_object_event_id("enter_" + id_ + "_anim")),
 	 end_event_id_(get_object_event_id("end_" + id_ + "_anim")),
 	 leave_event_id_(get_object_event_id("leave_" + id_ + "_anim")),
 	 process_event_id_(get_object_event_id("process_" + id_)),
-     texture_(graphics::texture::get(image_, node->attr("image_formula"))),
+     texture_(graphics::texture::get(image_, node["image_formula"].as_string_default())),
 	 solid_(solid_info::create(node)),
-     collide_rect_(node->has_attr("collide") ? rect(node->attr("collide")) :
-	               rect(wml::get_int(node, "collide_x"),
-                        wml::get_int(node, "collide_y"),
-                        wml::get_int(node, "collide_w"),
-                        wml::get_int(node, "collide_h"))),
-	 hit_rect_(node->has_attr("hit") ? rect(node->attr("hit")) :
-	               rect(wml::get_int(node, "hit_x"),
-				        wml::get_int(node, "hit_y"),
-				        wml::get_int(node, "hit_w"),
-				        wml::get_int(node, "hit_h"))),
-	 platform_rect_(node->has_attr("platform") ? rect(node->attr("platform")) :
-	                rect(wml::get_int(node, "platform_x"),
-	                     wml::get_int(node, "platform_y"),
-	                     wml::get_int(node, "platform_w"), 1)),
-	 img_rect_(node->has_attr("rect") ? rect(node->attr("rect")) :
-	           rect(wml::get_int(node, "x"),
-	                wml::get_int(node, "y"),
-	                wml::get_int(node, "w"),
-	                wml::get_int(node, "h"))),
-	 feet_x_(wml::get_int(node, "feet_x")),
-	 feet_y_(wml::get_int(node, "feet_y")),
-	 accel_x_(wml::get_int(node, "accel_x", INT_MIN)),
-	 accel_y_(wml::get_int(node, "accel_y", INT_MIN)),
-	 velocity_x_(wml::get_int(node, "velocity_x", INT_MIN)),
-	 velocity_y_(wml::get_int(node, "velocity_y", INT_MIN)),
-	 nframes_(wml::get_int(node, "frames", 1)),
-	 nframes_per_row_(wml::get_int(node, "frames_per_row", -1)),
-	 frame_time_(wml::get_int(node, "duration", -1)),
-	 reverse_frame_(wml::get_bool(node, "reverse")),
-	 play_backwards_(wml::get_bool(node, "play_backwards")),
-	 scale_(wml::get_int(node, "scale", 2)),
-	 pad_(wml::get_int(node, "pad")),
-	 rotate_(wml::get_int(node, "rotate")),
-	 blur_(wml::get_int(node, "blur")),
-	 rotate_on_slope_(wml::get_bool(node, "rotate_on_slope")),
-	 damage_(wml::get_int(node, "damage")),
-	 sounds_(util::split(node->attr("sound"))),
-	 no_remove_alpha_borders_(wml::get_bool(node, "no_remove_alpha_borders", false)),
+     collide_rect_(node.has_key("collide") ? rect(node["collide"]) :
+	               rect(node["collide_x"].as_int(),
+                        node["collide_y"].as_int(),
+                        node["collide_w"].as_int(),
+                        node["collide_h"].as_int())),
+	 hit_rect_(node.has_key("hit") ? rect(node["hit"]) :
+	               rect(node["hit_x"].as_int(),
+				        node["hit_y"].as_int(),
+				        node["hit_w"].as_int(),
+				        node["hit_h"].as_int())),
+	 platform_rect_(node.has_key("platform") ? rect(node["platform"]) :
+	                rect(node["platform_x"].as_int(),
+	                     node["platform_y"].as_int(),
+	                     node["platform_w"].as_int(), 1)),
+	 img_rect_(node.has_key("rect") ? rect(node["rect"]) :
+	           rect(node["x"].as_int(),
+	                node["y"].as_int(),
+	                node["w"].as_int(),
+	                node["h"].as_int())),
+	 feet_x_(node["feet_x"].as_int()),
+	 feet_y_(node["feet_y"].as_int()),
+	 accel_x_(node["accel_x"].as_int(INT_MIN)),
+	 accel_y_(node["accel_y"].as_int(INT_MIN)),
+	 velocity_x_(node["velocity_x"].as_int(INT_MIN)),
+	 velocity_y_(node["velocity_y"].as_int(INT_MIN)),
+	 nframes_(node["frames"].as_int(1)),
+	 nframes_per_row_(node["frames_per_row"].as_int(-1)),
+	 frame_time_(node["duration"].as_int(-1)),
+	 reverse_frame_(node["reverse"].as_bool()),
+	 play_backwards_(node["play_backwards"].as_bool()),
+	 scale_(node["scale"].as_int(2)),
+	 pad_(node["pad"].as_int()),
+	 rotate_(node["rotate"].as_int()),
+	 blur_(node["blur"].as_int()),
+	 rotate_on_slope_(node["rotate_on_slope"].as_bool()),
+	 damage_(node["damage"].as_int()),
+	 sounds_(util::split(node["sound"].as_string_default())),
+	 no_remove_alpha_borders_(node["no_remove_alpha_borders"].as_bool(false)),
+	 collision_areas_inside_frame_(true),
 	 current_palette_(-1)
 {
-	std::vector<std::string> hit_frames = util::split((*node)["hit_frames"]);
+	std::vector<std::string> hit_frames = util::split(node["hit_frames"].as_string_default());
 	foreach(const std::string& f, hit_frames) {
 		hit_frames_.push_back(boost::lexical_cast<int>(f));
 	}
 
-	const std::string& events = node->attr("events");
+	const std::string& events = node["events"].as_string_default();
 	if(!events.empty()) {
 		//events are in the format time0:time1:...:timen:event0,time0:time1:...:timen:event1,...
 		std::vector<std::string> event_vector = util::split(events);
@@ -108,45 +109,59 @@ frame::frame(wml::const_node_ptr node)
 	}
 
 	static const std::string AreaPostfix = "_area";
-	for(wml::node::const_attr_iterator i = node->begin_attr(); i != node->end_attr(); ++i) {
-		const std::string& attr = i->first;
+	foreach(const variant_pair& val, node.as_map()) {
+		const std::string& attr = val.first.as_string();
 		if(attr.size() <= AreaPostfix.size() || std::equal(AreaPostfix.begin(), AreaPostfix.end(), attr.end() - AreaPostfix.size()) == false || attr == "solid_area" ||attr == "platform_area") {
 			continue;
 		}
 
 		const std::string area_id = std::string(attr.begin(), attr.end() - AreaPostfix.size());
 
+		variant value = val.second;
+
 		bool solid = false;
-		std::string value = i->second;
-
-		static const std::string SolidStr = "solid:";
-		if(value.size() > SolidStr.size() && std::equal(SolidStr.begin(), SolidStr.end(), value.begin())) {
-			solid = true;
-			value.erase(value.begin(), value.begin() + SolidStr.size());
-		}
-
 		rect r;
-		if(value == "none") {
+		if(value.is_null()) {
 			continue;
-		} else if(value == "all") {
+		} else if(value.is_string() && value.as_string() == "all") {
 			r = rect(0, 0, width(), height());
-		} else {
-			r = rect(value);
-			r = rect(r.x()*scale_, r.y()*scale_, r.w()*scale_, r.h()*scale_);
+		} else if(value.is_list()) {
+			std::vector<int> v;
+			foreach(const variant& var, value.as_list()) {
+				if(var.is_int()) {
+					v.push_back(var.as_int());
+				} else if(var.is_string() && var.as_string() == "solid") {
+					solid = true;
+				} else if(var.is_string() && var.as_string() == "all") {
+					r = rect(0, 0, width(), height());
+				} else {
+					ASSERT_LOG(false, "Unrecognized attribute for '" << attr << "': " << value.to_debug_string());
+				}
+			}
+
+			if(v.empty() == false) {
+				r = rect(v);
+				r = rect(r.x()*scale_, r.y()*scale_, r.w()*scale_, r.h()*scale_);
+			}
 		}
+
 		collision_area area = { area_id, r, solid };
 		collision_areas_.push_back(area);
+
+		if(solid && (r.x() < 0 || r.y() < 0 || r.x2() > width() || r.y2() > height())) {
+			collision_areas_inside_frame_ = false;
+		}
 	}
 
-	if(node->has_attr("frame_info")) {
-		int values_buf[1024];
-		int num_values = 1024;
-		util::split_into_ints(node->attr("frame_info").c_str(), values_buf, &num_values);
+	if(node.has_key("frame_info")) {
+		std::vector<int> values = node["frame_info"].as_list_int();
+		int num_values = values.size();
 
+		ASSERT_GT(num_values, 0);
 		ASSERT_EQ(num_values%8, 0);
 		ASSERT_LE(num_values, 1024);
-		const int* i = values_buf;
-		const int* i2 = values_buf + num_values;
+		const int* i = &values[0];
+		const int* i2 = &values[0] + num_values;
 		while(i != i2) {
 			frame_info info;
 			info.x_adjust = *i++;
@@ -172,13 +187,11 @@ frame::frame(wml::const_node_ptr node)
 		build_alpha();
 	}
 
-	std::vector<std::string> palettes = util::split(node->attr("palettes"));
+	std::vector<std::string> palettes = parse_variant_list_or_csv_string(node["palettes"]);
 	foreach(const std::string& p, palettes) {
 		palettes_recognized_.push_back(graphics::get_palette_id(p));
 	}
 
-	//std::cerr << "PALETTES: " << node->attr("palettes").str() << " " << palettes_recognized_.size() << "\n";
-
 	if(palettes_recognized_.empty() == false) {
 		palette_frames().insert(this);
 		if(current_palette_mask) {
@@ -186,22 +199,21 @@ frame::frame(wml::const_node_ptr node)
 		}
 	}
 
-	for(wml::node::const_attr_iterator i = node->begin_attr(); i != node->end_attr(); ++i) {
+	foreach(const variant_pair& value, node.as_map()) {
 		static const std::string PivotPrefix = "pivot_";
-		const std::string& attr = i->first;
+		const std::string& attr = value.first.as_string();
 		if(attr.size() > PivotPrefix.size() && std::equal(PivotPrefix.begin(), PivotPrefix.end(), attr.begin())) {
 			pivot_schedule schedule;
 			schedule.name = std::string(attr.begin() + PivotPrefix.size(), attr.end());
 
-			int buf[1024];
-			int buf_size = 1024;
-			util::split_into_ints(i->second.str().c_str(), buf, &buf_size);
-			ASSERT_LOG(buf_size%2 == 0, "PIVOT POINTS IN INCORRECT FORMAT, ODD NUMBER OF INTEGERS");
-			const int num_points = buf_size/2;
+			std::vector<int> values = value.second.as_list_int();
+
+			ASSERT_LOG(values.size()%2 == 0, "PIVOT POINTS IN INCORRECT FORMAT, ODD NUMBER OF INTEGERS");
+			const int num_points = values.size()/2;
 
 			int repeat = std::max<int>(1, (nframes_*frame_time_)/std::max<int>(1, num_points));
 			for(int n = 0; n != num_points; ++n) {
-				point p(buf[n*2], buf[n*2+1]);
+				point p(values[n*2], values[n*2+1]);
 				for(int m = 0; m != repeat; ++m) {
 					schedule.points.push_back(p);
 				}
@@ -418,8 +430,18 @@ void frame::build_alpha()
 
 bool frame::is_alpha(int x, int y, int time, bool face_right) const
 {
-	if(alpha_.empty()) {
+	std::vector<bool>::const_iterator itor = get_alpha_itor(x, y, time, face_right);
+	if(itor == alpha_.end()) {
 		return true;
+	} else {
+		return *itor;
+	}
+}
+
+std::vector<bool>::const_iterator frame::get_alpha_itor(int x, int y, int time, bool face_right) const
+{
+	if(alpha_.empty()) {
+		return alpha_.end();
 	}
 
 	if(face_right == false) {
@@ -427,7 +449,7 @@ bool frame::is_alpha(int x, int y, int time, bool face_right) const
 	}
 
 	if(x < 0 || y < 0 || x >= width() || y >= height()) {
-		return true;
+		return alpha_.end();
 	}
 
 	x /= scale_;
@@ -438,7 +460,7 @@ bool frame::is_alpha(int x, int y, int time, bool face_right) const
 	
 	const int index = y*img_rect_.w()*nframes_ + x;
 	ASSERT_INDEX_INTO_VECTOR(index, alpha_);
-	return alpha_[index];
+	return alpha_.begin() + index;
 }
 
 void frame::draw_into_blit_queue(graphics::blit_queue& blit, int x, int y, bool face_right, bool upside_down, int time) const
@@ -452,8 +474,14 @@ void frame::draw_into_blit_queue(graphics::blit_queue& blit, int x, int y, bool
 	const int w = info->area.w()*scale_*(face_right ? 1 : -1);
 	const int h = info->area.h()*scale_*(upside_down ? -1 : 1);
 
+	rect[0] = texture_.translate_coord_x(rect[0]);
+	rect[1] = texture_.translate_coord_y(rect[1]);
+	rect[2] = texture_.translate_coord_x(rect[2]);
+	rect[3] = texture_.translate_coord_y(rect[3]);
+
 	blit.set_texture(texture_.get_id());
 
+
 	blit.add(x, y, rect[0], rect[1]);
 	blit.add(x + w, y, rect[2], rect[1]);
 	blit.add(x, y + h, rect[0], rect[3]);
@@ -534,6 +562,129 @@ void frame::draw(int x, int y, const rect& area, bool face_right, bool upside_do
 	                       rect[0], rect[1], rect[2], rect[3]);
 }
 
+void frame::draw_custom(int x, int y, const std::vector<CustomPoint>& points, const rect* area, bool face_right, bool upside_down, int time, GLfloat rotate) const
+{
+	texture_.set_as_current_texture();
+
+	const frame_info* info = NULL;
+	GLfloat rect[4];
+	get_rect_in_texture(time, &rect[0], info);
+	rect[0] = texture_.translate_coord_x(rect[0]);
+	rect[1] = texture_.translate_coord_y(rect[1]);
+	rect[2] = texture_.translate_coord_x(rect[2]);
+	rect[3] = texture_.translate_coord_y(rect[3]);
+
+	x += (face_right ? info->x_adjust : info->x2_adjust)*scale_;
+	y += info->y_adjust*scale_;
+	int w = info->area.w()*scale_*(face_right ? 1 : -1);
+	int h = info->area.h()*scale_*(upside_down ? -1 : 1);
+
+	if(w < 0) {
+		std::swap(rect[0], rect[2]);
+		w *= -1;
+	}
+
+	if(h < 0) {
+		std::swap(rect[1], rect[3]);
+		h *= -1;
+	}
+
+	if(area != NULL) {
+		const int x_adjust = area->x();
+		const int y_adjust = area->y();
+		const int w_adjust = area->w() - img_rect_.w();
+		const int h_adjust = area->h() - img_rect_.h();
+
+		rect[0] += GLfloat(x_adjust)/GLfloat(texture_.width());
+		rect[1] += GLfloat(y_adjust)/GLfloat(texture_.height());
+		rect[2] += GLfloat(x_adjust + w_adjust)/GLfloat(texture_.width());
+		rect[3] += GLfloat(y_adjust + h_adjust)/GLfloat(texture_.height());
+
+		w += w_adjust*scale_;
+		h += h_adjust*scale_;
+	}
+
+	std::vector<GLfloat> tcqueue;
+	std::vector<GLshort> vqueue;
+
+	const GLfloat center_x = x + GLfloat(w)/2.0;
+	const GLfloat center_y = y + GLfloat(h)/2.0;
+
+	glPushMatrix();
+	glTranslatef(center_x, center_y, 0.0);
+	glRotatef(rotate,0.0,0.0,1.0);
+
+	foreach(const CustomPoint& p, points) {
+		GLfloat pos = p.pos;
+
+		if(pos > 4.0) {
+			pos = 4.0;
+		}
+
+		int side = static_cast<int>(pos);
+		GLfloat f = pos - static_cast<GLfloat>(side);
+		if(side >= 4) {
+			side = 0;
+		}
+
+		GLshort xpos, ypos;
+		GLfloat u, v;
+		switch(side) {
+		case 0:
+			u = rect[0] + (rect[2] - rect[0])*f;
+			v = rect[1];
+			xpos = GLfloat(x) + GLfloat(w)*f;
+			ypos = y;
+			break;
+		case 2:
+			u = rect[2] - (rect[2] - rect[0])*f;
+			v = rect[3];
+			xpos = GLfloat(x + w) - GLfloat(w)*f;
+			ypos = y + h;
+			break;
+		case 1:
+			u = rect[2];
+			v = rect[1] + (rect[3] - rect[1])*f;
+			xpos = x + w;
+			ypos = GLfloat(y) + GLfloat(h)*f;
+			break;
+		case 3:
+			u = rect[0];
+			v = rect[3] - (rect[3] - rect[1])*f;
+			xpos = x;
+			ypos = GLfloat(y + h) - GLfloat(h)*f;
+			break;
+		default:
+			ASSERT_LOG(false, "ILLEGAL CUSTOM FRAME POSITION: " << side);
+			break;
+		}
+
+		xpos += p.offset.x;
+		ypos += p.offset.y;
+
+		vqueue.push_back(xpos - center_x);
+		vqueue.push_back(ypos - center_y);
+
+		tcqueue.push_back(u);
+		tcqueue.push_back(v);
+	}
+
+	ASSERT_LOG(vqueue.size() > 4, "ILLEGAL CUSTOM BLIT: " << vqueue.size());
+#if defined(USE_GLES2)
+	{
+		gles2::active_shader()->prepare_draw();
+		gles2::active_shader()->shader()->vertex_array(2, GL_SHORT, 0, 0, &vqueue.front());
+		gles2::active_shader()->shader()->texture_array(2, GL_FLOAT, GL_FALSE, 0, &tcqueue.front());
+		glDrawArrays(GL_TRIANGLE_STRIP, 0, tcqueue.size()/2);
+	}
+#else
+	glVertexPointer(2, GL_SHORT, 0, &vqueue.front());
+	glTexCoordPointer(2, GL_FLOAT, 0, &tcqueue.front());
+	glDrawArrays(GL_TRIANGLE_STRIP, 0, tcqueue.size()/2);
+#endif
+	glPopMatrix();
+}
+
 void frame::get_rect_in_texture(int time, GLfloat* output_rect, const frame_info*& info) const
 {
 	//picks out a single frame to draw from a whole animation, based on time
