@@ -10,7 +10,6 @@
 #include "solid_map_fwd.hpp"
 #include "texture.hpp"
 #include "variant.hpp"
-#include "wml_node_fwd.hpp"
 
 namespace graphics {
 class blit_queue;
@@ -33,23 +32,36 @@ public:
 
 	static void set_color_palette(unsigned int palettes);
 
-	explicit frame(wml::const_node_ptr node);
+	explicit frame(variant node);
 	~frame();
 
 	//ID of the frame. Not unique, but is the name of the element the frame
 	//came from. Useful to tell what kind of frame it is.
 	const std::string& id() const { return id_; }
 	const variant& variant_id() const { return variant_id_; }
+	const std::string& image_name() const { return image_; }
 
 	//play a sound. 'object' is just the address of the object playing the
 	//sound, useful if the sound is later cancelled.
 	void play_sound(const void* object=NULL) const;
-	std::vector<bool>::const_iterator alpha_row(int x, int y, int time, bool face_right) const;
 	bool is_alpha(int x, int y, int time, bool face_right) const;
+
+	//Low level interface to alpha information.
+	std::vector<bool>::const_iterator get_alpha_itor(int x, int y, int time, bool face_right) const;
+	const std::vector<bool>& get_alpha_buf() const { return alpha_; }
+
 	void draw_into_blit_queue(graphics::blit_queue& blit, int x, int y, bool face_right=true, bool upside_down=false, int time=0) const;
 	void draw(int x, int y, bool face_right=true, bool upside_down=false, int time=0, GLfloat rotate=0) const;
 	void draw(int x, int y, bool face_right, bool upside_down, int time, GLfloat rotate, GLfloat scale) const;
 	void draw(int x, int y, const rect& area, bool face_right=true, bool upside_down=false, int time=0, GLfloat rotate=0) const;
+
+	struct CustomPoint {
+		CustomPoint() : pos(0) {}
+		GLfloat pos;
+		point offset;
+	};
+
+	void draw_custom(int x, int y, const std::vector<CustomPoint>& points, const rect* area, bool face_right, bool upside_down, int time, GLfloat rotate) const;
 	void set_image_as_solid();
 	const_solid_info_ptr solid() const { return solid_; }
 	int collide_x() const { return collide_rect_.x()*scale_; }
@@ -86,6 +98,7 @@ public:
 	const std::string* get_event(int time_in_frame) const;
 
 	const std::vector<collision_area>& collision_areas() const { return collision_areas_; }
+	bool collision_areas_inside_frame() const { return collision_areas_inside_frame_; }
 
 	int enter_event_id() const { return enter_event_id_; }
 	int end_event_id() const { return end_event_id_; }
@@ -102,8 +115,8 @@ public:
 	const std::vector<frame_info>& frame_layout() const { return frames_; }
 
 	point pivot(const std::string& name, int time_in_frame) const;
-private:
 	int frame_number(int time_in_frame) const;
+private:
 
 	void get_rect_in_texture(int time, GLfloat* output_rect, const frame_info*& info) const;
 	void get_rect_in_frame_number(int nframe, GLfloat* output_rect, const frame_info*& info) const;
@@ -146,6 +159,7 @@ private:
 	std::vector <std::string> sounds_;
 
 	std::vector<collision_area> collision_areas_;
+	bool collision_areas_inside_frame_;
 
 	void build_alpha_from_frame_info();
 	void build_alpha();
