@@ -1,5 +1,6 @@
 #ifndef GROUP_PROPERTY_EDITOR_DIALOG_HPP_INCLUDED
 #define GROUP_PROPERTY_EDITOR_DIALOG_HPP_INCLUDED
+#ifndef NO_EDITOR
 
 #include <string>
 #include <vector>
@@ -25,6 +26,10 @@ private:
 	std::vector<entity_ptr> group_;
 };
 
+typedef boost::intrusive_ptr<group_property_editor_dialog> group_property_editor_dialog_ptr;
+
 }
 
 #endif
+#endif // !NO_EDITOR
+
