@@ -18,8 +18,12 @@ class key_button : public widget
 {
 public:
 	key_button(SDLKey key, BUTTON_RESOLUTION button_resolution);
+	key_button(const variant& v, game_logic::formula_callable* e);
 
 	SDLKey get_key();
+
+	void set_value(const std::string& key, const variant& v);
+	variant get_value(const std::string& key) const;
 private:
 	bool in_button(int x, int y) const;
 	void handle_draw() const;
@@ -33,7 +37,7 @@ private:
 	const_framed_gui_element_ptr normal_button_image_set_,depressed_button_image_set_,focus_button_image_set_,current_button_image_set_;
 };
 
-typedef boost::shared_ptr<key_button> key_button_ptr;
+typedef boost::intrusive_ptr<key_button> key_button_ptr;
 
 }
 
