@@ -1,4 +1,5 @@
 #include <iostream>
+#include <sstream>
 #include <map>
 #include <string>
 
@@ -6,19 +7,23 @@
 #include "asserts.hpp"
 #include "draw_tile.hpp"
 #include "filesystem.hpp"
+#include "json_parser.hpp"
 #include "level_object.hpp"
+#include "module.hpp"
 #include "preferences.hpp"
 #include "raster.hpp"
 #include "string_utils.hpp"
 #include "surface.hpp"
 #include "surface_cache.hpp"
 #include "surface_palette.hpp"
-#include "wml_parser.hpp"
-#include "wml_utils.hpp"
-#include "wml_writer.hpp"
+#include "unit_test.hpp"
+#include "variant_utils.hpp"
+
 namespace {
 typedef std::map<std::string,const_level_object_ptr> tiles_map;
 tiles_map tiles_cache;
+
+const int BaseTileSize = 16;
 }
 
 bool level_tile::is_solid(int xpos, int ypos) const
@@ -36,25 +41,31 @@ std::vector<const_level_object_ptr> level_object::all()
 	return res;
 }
 
-level_tile level_object::build_tile(wml::const_node_ptr node)
+level_tile level_object::build_tile(variant node)
 {
 	level_tile res;
-	res.x = wml::get_int(node, "x");
-	res.y = wml::get_int(node, "y");
-	res.zorder = wml::get_int(node, "zorder");
-	if(tiles_cache.count(node->attr("tile"))) {
-		res.object = tiles_cache[node->attr("tile")];
+	res.x = node["x"].as_int();
+	res.y = node["y"].as_int();
+	res.zorder = node["zorder"].as_int();
+	if(tiles_cache.count(node["tile"].as_string())) {
+		res.object = tiles_cache[node["tile"].as_string()].get();
 	}
-	res.face_right = wml::get_bool(node, "face_right");
+	res.face_right = node["face_right"].as_bool();
 	return res;
 }
 
 namespace {
-std::vector<wml::node_ptr> level_object_index;
-std::vector<wml::const_node_ptr> original_level_object_nodes;
-std::map<std::pair<wml::const_node_ptr, int>, level_object_ptr> secondary_zorder_objects;
 
-std::map<wml::node_ptr, int> tile_nodes_to_zorders;
+typedef boost::shared_ptr<variant> obj_variant_ptr;
+typedef boost::shared_ptr<const variant> const_obj_variant_ptr;
+
+std::vector<obj_variant_ptr> level_object_index;
+std::vector<const_obj_variant_ptr> original_level_object_nodes;
+std::map<std::pair<const_obj_variant_ptr, int>, level_object_ptr> secondary_zorder_objects;
+
+std::map<obj_variant_ptr, int> tile_nodes_to_zorders;
+
+std::map<std::string, int> tile_str_to_palette;
 
 
 typedef std::pair<std::string, int> filename_palette_pair;
@@ -66,18 +77,28 @@ std::map<tile_id, int> compiled_tile_ids;
 
 //defined in texture.cpp
 namespace graphics {
-void set_alpha_for_transparent_colors_in_rgba_surface(SDL_Surface* s);
+void set_alpha_for_transparent_colors_in_rgba_surface(SDL_Surface* s, int options=0);
 }
 
 void create_compiled_tiles_image()
 {
 	//the number of tiles that can fit in a tiesheet.
-	const int TilesInSheet = (1024*1024)/(16*16);
+	const int TilesInSheet = (1024*1024)/(BaseTileSize*BaseTileSize);
+
+	//which zorders require an alpha channel?
+	std::set<int> zorder_with_alpha_channel;
 
 	//calculate how many tiles are in each zorder
 	std::map<int, int> zorder_to_num_tiles;
-	for(std::map<wml::node_ptr, int>::const_iterator i = tile_nodes_to_zorders.begin(); i != tile_nodes_to_zorders.end(); ++i) {
-		zorder_to_num_tiles[i->second]++;
+	for(std::map<obj_variant_ptr, int>::const_iterator i = tile_nodes_to_zorders.begin(); i != tile_nodes_to_zorders.end(); ++i) {
+		obj_variant_ptr node = i->first;
+		std::vector<std::string> tiles_vec = util::split((*node)["tiles"].as_string(), '|');
+		zorder_to_num_tiles[i->second] += tiles_vec.size();
+
+		const static std::string UsesAlphaChannelStr = "uses_alpha_channel";
+		if(i->first->has_key(UsesAlphaChannelStr)) {
+			zorder_with_alpha_channel.insert(i->second);
+		}
 	}
 
 	//now work out which zorders should go in which tilesheets.
@@ -87,29 +108,51 @@ void create_compiled_tiles_image()
 	std::vector<graphics::surface> sheets;
 	std::vector<int> sheet_next_image_index;
 
-	for(std::map<int, int>::const_iterator i = zorder_to_num_tiles.begin();
-	    i != zorder_to_num_tiles.end(); ++i) {
-		int sheet = 0;
-		for(; sheet != tiles_in_sheet.size(); ++sheet) {
-			if(tiles_in_sheet[sheet] + i->second <= TilesInSheet) {
-				break;
+	//two passes, since we do all zorders with alpha channel first, so
+	//they'll go in the first tilesheet together, then those without.
+	for(int use_alpha_channel = 1; use_alpha_channel >= 0; --use_alpha_channel) {
+		fprintf(stderr, "ZORDER_PROC\n");
+		for(std::map<int, int>::const_iterator i = zorder_to_num_tiles.begin();
+		    i != zorder_to_num_tiles.end(); ++i) {
+			if(zorder_with_alpha_channel.count(i->first) != use_alpha_channel) {
+				continue;
 			}
-		}
 
-		if(sheet == tiles_in_sheet.size()) {
-			tiles_in_sheet.push_back(0);
-			sheet_next_image_index.push_back(0);
-			sheets.push_back(graphics::surface(SDL_CreateRGBSurface(SDL_SWSURFACE, 1024, 1024, 32, SURFACE_MASK)));
-		}
+			fprintf(stderr, "ZORDER_PROC: %d %d\n", i->first, i->second);
 
-		tiles_in_sheet[sheet] += i->second;
-		zorder_to_sheet_number[i->first] = sheet;
+			int sheet = 0;
+			for(; sheet != tiles_in_sheet.size(); ++sheet) {
+				if(tiles_in_sheet[sheet] + i->second <= TilesInSheet) {
+					std::cerr << "ZORDER_ALLOC " << i->first << " (" << i->second << ") -> " << sheet << "\n";
+					break;
+				}
+			}
+	
+
+			if(sheet == tiles_in_sheet.size()) {
+				const int num_sheets = 1 + i->second/TilesInSheet;
+				std::cerr << "ZORDER_ALLOC " << i->first << " (" << i->second << ") -> NEW SHEET " << sheet << "(" << num_sheets << ")\n";
+				for(int n = 0; n != num_sheets; ++n) {
+					tiles_in_sheet.push_back(0);
+					sheet_next_image_index.push_back(0);
+					sheets.push_back(graphics::surface(SDL_CreateRGBSurface(SDL_SWSURFACE, 1024, 1024, 32, SURFACE_MASK)));
+					tiles_in_sheet[sheet+n] += i->second;
+				}
+			} else {
+				tiles_in_sheet[sheet] += i->second;
+			}
+
+			if(!(tiles_in_sheet[sheet] <= TilesInSheet)) {
+				std::cerr << "TOO MANY TILES IN SHEET " << sheet << "/" << tiles_in_sheet.size() << ": " << tiles_in_sheet[sheet] << "/" << TilesInSheet << " (zorder = " << i->first << ")\n";
+			}
+			zorder_to_sheet_number[i->first] = sheet;
+		}
 	}
 
 	std::cerr << "NUM_TILES: " << tile_nodes_to_zorders.size() << " / " << TilesInSheet << "\n";
 
 
-	graphics::surface s(SDL_CreateRGBSurface(SDL_SWSURFACE, 1024, (compiled_tile_ids.size()/64 + 1)*16, 32, SURFACE_MASK));
+	graphics::surface s(SDL_CreateRGBSurface(SDL_SWSURFACE, 1024, (compiled_tile_ids.size()/64 + 1)*BaseTileSize, 32, SURFACE_MASK));
 	for(std::map<tile_id, int>::const_iterator itor = compiled_tile_ids.begin();
 	    itor != compiled_tile_ids.end(); ++itor) {
 		const tile_id& tile_info = itor->first;
@@ -125,16 +168,16 @@ void create_compiled_tiles_image()
 		}
 
 		SDL_SetAlpha(src.get(), 0, SDL_ALPHA_OPAQUE);
-		const int width = std::max<int>(src->w, src->h)/16;
+		const int width = std::max<int>(src->w, src->h)/BaseTileSize;
 
-		const int src_x = (tile_pos%width) * 16;
-		const int src_y = (tile_pos/width) * 16;
+		const int src_x = (tile_pos%width) * BaseTileSize;
+		const int src_y = (tile_pos/width) * BaseTileSize;
 		
-		const int dst_x = (itor->second%64) * 16;
-		const int dst_y = (itor->second/64) * 16;
+		const int dst_x = (itor->second%64) * BaseTileSize;
+		const int dst_y = (itor->second/64) * BaseTileSize;
 
-		SDL_Rect src_rect = { src_x, src_y, 16, 16 };
-		SDL_Rect dst_rect = { dst_x, dst_y, 16, 16 };
+		SDL_Rect src_rect = { src_x, src_y, BaseTileSize, BaseTileSize };
+		SDL_Rect dst_rect = { dst_x, dst_y, BaseTileSize, BaseTileSize };
 
 		SDL_BlitSurface(src.get(), &src_rect, s.get(), &dst_rect);
 	}
@@ -146,33 +189,56 @@ void create_compiled_tiles_image()
 
 	SDL_SetAlpha(s.get(), 0, SDL_ALPHA_OPAQUE);
 
-	for(std::map<wml::node_ptr, int>::const_iterator i = tile_nodes_to_zorders.begin(); i != tile_nodes_to_zorders.end(); ++i) {
-		const int sheet = zorder_to_sheet_number[i->second];
-		wml::node_ptr node = i->first;
+	for(std::map<obj_variant_ptr, int>::const_iterator i = tile_nodes_to_zorders.begin(); i != tile_nodes_to_zorders.end(); ++i) {
+		const int num_tiles = zorder_to_num_tiles[i->second];
+		const int num_sheets = 1 + num_tiles/TilesInSheet;
+		int sheet = zorder_to_sheet_number[i->second];
+
+		obj_variant_ptr node = i->first;
+		if(num_sheets > 1) {
+			int offset = abs(tile_str_to_palette[(*node)["tiles"].as_string()])%num_sheets;
+			
+			const static std::string UsesAlphaChannelStr = "uses_alpha_channel";
+			if(i->first->has_key(UsesAlphaChannelStr)) {
+				//try to put all alpha tiles in the first sheet.
+				offset = 0;
+			}
+
+			int count = 0;
+			while(sheet_next_image_index[sheet + offset] >= TilesInSheet) {
+				offset = (offset+1)%num_sheets;
+				++count;
+				ASSERT_LOG(count <= num_sheets, "COULD NOT PLACE TILES IN SHEET\n");
+			}
+
+			sheet += offset;
+		}
+		std::cerr << "NODE: " << node->write_json() << " -> " << i->second << "\n";
 		std::map<int, int> dst_index_map;
 
-		std::vector<std::string> tiles_vec = util::split(node->attr("tiles").str(), '|');
+		std::vector<std::string> tiles_vec = util::split((*node)["tiles"].as_string(), '|');
 		std::string tiles_val;
 
 		foreach(const std::string& tiles_str, tiles_vec) {
 			ASSERT_EQ(tiles_str[0], '+');
 
 			const int tile_num = atoi(tiles_str.c_str() + 1);
-			const int src_x = (tile_num%64) * 16;
-			const int src_y = (tile_num/64) * 16;
+			const int src_x = (tile_num%64) * BaseTileSize;
+			const int src_y = (tile_num/64) * BaseTileSize;
 
 			int dst_tile;
 			if(dst_index_map.count(tile_num)) {
 				dst_tile = dst_index_map[tile_num];
 			} else {
 				dst_tile = sheet_next_image_index[sheet]++;
+				ASSERT_LOG(dst_tile < tiles_in_sheet[sheet], "TOO MANY TILES ON SHEET: " << sheet << ": " << tiles_in_sheet[sheet] << " ZORDER: " << i->second);
 				dst_index_map[tile_num] = dst_tile;
 
-				const int dst_x = (dst_tile%64) * 16;
-				const int dst_y = (dst_tile/64) * 16;
+				const int dst_x = (dst_tile%64) * BaseTileSize;
+				const int dst_y = (dst_tile/64) * BaseTileSize;
 
-				SDL_Rect src_rect = { src_x, src_y, 16, 16 };
-				SDL_Rect dst_rect = { dst_x, dst_y, 16, 16 };
+				SDL_Rect src_rect = { src_x, src_y, BaseTileSize, BaseTileSize };
+				SDL_Rect dst_rect = { dst_x, dst_y, BaseTileSize, BaseTileSize };
 
 				SDL_BlitSurface(s.get(), &src_rect, sheets[sheet].get(), &dst_rect);
 			}
@@ -186,19 +252,21 @@ void create_compiled_tiles_image()
 			tiles_val += buf;
 		}
 
-		node->set_attr("tiles", tiles_val);
+		*node = node->add_attr(variant("tiles"), variant(tiles_val));
 
-		char buf[128];
-		sprintf(buf, "tiles-compiled-%d.png", sheet);
+		std::stringstream str;
+		str << "tiles-compiled-" << std::dec << sheet << ".png";
 
-		node->set_attr("image", buf);
+		*node = node->add_attr(variant("image"), variant(str.str()));
 	}
 
 	for(int n = 0; n != sheets.size(); ++n) {
-		char buf[64];
-		sprintf(buf, "images/tiles-compiled-%d.png", n);
-
-		IMG_SavePNG(buf, sheets[n], 5);
+		std::stringstream str;
+		str << "images/tiles-compiled-" << std::dec << n << ".png";
+		
+#if !defined(__native_client__)
+		IMG_SavePNG(module::map_file(str.str()).c_str(), sheets[n], 5);
+#endif
 	}
 }
 
@@ -212,16 +280,13 @@ std::set<level_object*>& palette_level_objects() {
 }
 }
 
-palette_scope::palette_scope(const std::string& str)
+palette_scope::palette_scope(const std::vector<std::string>& v)
   : original_value(current_palette_set)
 {
 	current_palette_set = 0;	
-	if(str.empty() == false) {
-		std::vector<std::string> p = util::split(str);
-		foreach(const std::string& pal, p) {
-			const int id = graphics::get_palette_id(pal);
-			current_palette_set |= 1 << id;
-		}
+	foreach(const std::string& pal, v) {
+		const int id = graphics::get_palette_id(pal);
+		current_palette_set |= 1 << id;
 	}
 }
 
@@ -237,24 +302,29 @@ void level_object::set_current_palette(unsigned int palette)
 	}
 }
 
-level_object::level_object(wml::const_node_ptr node)
-  : id_(node->attr("id")), image_(node->attr("image")),
+level_object::level_object(variant node, const char* id)
+  : id_(node["id"].as_string_default()), image_(node["image"].as_string()),
+    info_(node["info"].as_string_default()),
     t_(graphics::texture::get(image_)),
-	all_solid_(node->attr("solid").str() == "yes"),
-    passthrough_(wml::get_bool(node, "passthrough")),
-    flip_(wml::get_bool(node, "flip", false)),
-    friction_(wml::get_int(node, "friction", 100)),
-    traction_(wml::get_int(node, "traction", 100)),
-    damage_(wml::get_int(node, "damage", 0)),
-	opaque_(wml::get_bool(node, "opaque", false)),
-	draw_area_(0, 0, 16, 16),
+	all_solid_(node["solid"].is_bool() ? node["solid"].as_bool() : node["solid"].as_string_default() == "yes"),
+    passthrough_(node["passthrough"].as_bool()),
+    flip_(node["flip"].as_bool(false)),
+    friction_(node["friction"].as_int(100)),
+    traction_(node["traction"].as_int(100)),
+    damage_(node["damage"].as_int(0)),
+	opaque_(node["opaque"].as_bool(false)),
+	draw_area_(0, 0, BaseTileSize, BaseTileSize),
 	tile_index_(-1),
 	palettes_recognized_(current_palette_set),
 	current_palettes_(0)
 {
-	if(node->has_attr("palettes")) {
+	if(id_.empty() && id != NULL && *id) {
+		id_ = id;
+	}
+
+	if(node.has_key("palettes")) {
 		palettes_recognized_ = 0;
-		std::vector<std::string> p = util::split(node->attr("palettes"));
+		std::vector<std::string> p = parse_variant_list_or_csv_string(node["palettes"]);
 		foreach(const std::string& pal, p) {
 			const int id = graphics::get_palette_id(pal);
 			palettes_recognized_ |= 1 << id;
@@ -265,18 +335,27 @@ level_object::level_object(wml::const_node_ptr node)
 		palette_level_objects().insert(this);
 	}
 
-	if(node->has_attr("solid_color")) {
-		solid_color_ = boost::intrusive_ptr<graphics::color>(new graphics::color(node->attr("solid_color")));
+	if(node.has_key("solid_color")) {
+		solid_color_ = boost::intrusive_ptr<graphics::color>(new graphics::color(node["solid_color"].as_string_default()));
 		if(preferences::use_16bpp_textures()) {
 			*solid_color_ = graphics::color(graphics::map_color_to_16bpp(solid_color_->rgba()));
 		}
 	}
 
-	if(node->has_attr("draw_area")) {
-		draw_area_ = rect(node->attr("draw_area"));
+	if(node.has_key("draw_area")) {
+		draw_area_ = rect(node["draw_area"].as_string_default());
 	}
 
-	std::vector<std::string> tile_variations = util::split(node->attr("tiles"), '|');
+	//TODO: Fix up the JSON to be consistent and use a list.
+	std::string tiles_str;
+	variant tiles_variant = node["tiles"];
+	if(tiles_variant.is_int()) {
+		tiles_str = tiles_variant.string_cast();
+	} else {
+		tiles_str = tiles_variant.as_string();
+	}
+
+	std::vector<std::string> tile_variations = util::split(tiles_str, '|');
 	foreach(const std::string& variation, tile_variations) {
 		if(!variation.empty() && variation[0] == '+') {
 			//a + symbol at the start of tiles means that it's just a base-10
@@ -284,9 +363,9 @@ level_object::level_object(wml::const_node_ptr node)
 			tiles_.push_back(strtol(variation.c_str()+1, NULL, 10));
 		} else {
 			const int width = std::max<int>(t_.width(), t_.height());
-			assert(width%16 == 0);
-			const int base = std::min<int>(32, width/16);
-			tiles_.push_back(strtol(variation.c_str(), NULL, base));
+			assert(width%BaseTileSize == 0);
+			const int base = std::min<int>(32, width/BaseTileSize);
+			tiles_.push_back((base == 1) ? 0 : strtol(variation.c_str(), NULL, base));
 		}
 	}
 
@@ -294,9 +373,9 @@ level_object::level_object(wml::const_node_ptr node)
 		tiles_.resize(1);
 	}
 
-	if(node->has_attr("solid_map")) {
+	if(node.has_key("solid_map")) {
 		solid_.resize(width()*height());
-		graphics::surface surf(graphics::surface_cache::get(node->attr("solid_map")).convert_opengl_format());
+		graphics::surface surf(graphics::surface_cache::get(node["solid_map"].as_string()).convert_opengl_format());
 		if(surf.get()) {
 			const uint32_t* p = reinterpret_cast<const uint32_t*>(surf->pixels);
 			for(int n = 0; n != surf->w*surf->h && n != solid_.size(); ++n) {
@@ -309,7 +388,11 @@ level_object::level_object(wml::const_node_ptr node)
 		}
 	}
 	
-	std::vector<std::string> solid_attr = util::split(node->attr("solid").str());
+	std::vector<std::string> solid_attr;
+	
+	if(!node["solid"].is_bool()) {
+		solid_attr = util::split(node["solid"].as_string_default());
+	}
 
 	if(all_solid_ || std::find(solid_attr.begin(), solid_attr.end(), "flat") != solid_attr.end()) {
 		if(passthrough_){
@@ -407,16 +490,15 @@ level_object::level_object(wml::const_node_ptr node)
 		}
 	}
 	
-	if(node->has_attr("solid_heights")) {
-		//this is a csv list of heights which represent the solids
-		std::vector<std::string> heights = util::split(node->attr("solid_heights"));
+	if(node.has_key("solid_heights")) {
+		//this is a list of heights which represent the solids
+		std::vector<int> heights = node["solid_heights"].as_list_int();
 		if(!heights.empty()) {
 			solid_.resize(width()*height());
 			for(int x = 0; x < width(); ++x) {
 				const int heights_index = (heights.size()*x)/width();
 				assert(heights_index >= 0 && heights_index < heights.size());
-				const std::string& height_str = heights[heights_index];
-				const int h = atoi(height_str.c_str());
+				const int h = heights[heights_index];
 				for(int y = height() - h; y < height(); ++y) {
 					const int index = y*width() + x;
 					solid_[index] = true;
@@ -425,13 +507,11 @@ level_object::level_object(wml::const_node_ptr node)
 		}
 	}
 
-	wml::node::const_child_iterator r1 = node->begin_child("rect");
-	wml::node::const_child_iterator r2 = node->end_child("rect");
-	for(; r1 != r2; ++r1) {
-		const int x = wml::get_int(r1->second, "x");
-		const int y = wml::get_int(r1->second, "y");
-		const int w = wml::get_int(r1->second, "w");
-		const int h = wml::get_int(r1->second, "h");
+	foreach(variant r, node["rect"].as_list()) {
+		const int x = r["x"].as_int();
+		const int y = r["y"].as_int();
+		const int w = r["w"].as_int();
+		const int h = r["h"].as_int();
 
 		if(solid_.empty()) {
 			solid_.resize(width()*height());
@@ -454,34 +534,40 @@ level_object::level_object(wml::const_node_ptr node)
 		//solid colors will confuse the compilation.
 		solid_color_ = boost::intrusive_ptr<graphics::color>();
 
+		const bool uses_alpha_channel = calculate_uses_alpha_channel();
+
 		std::vector<int> palettes;
 		palettes.push_back(-1);
 		get_palettes_used(palettes);
 
 		foreach(int palette, palettes) {
-			wml::node_ptr node_copy(wml::deep_copy(node));
-			node_copy->erase_attr("palettes");
+			variant node_copy = node.add_attr(variant("palettes"), variant());
+			node_copy = node_copy.add_attr(variant("id"), variant(id_));
 			if(calculate_opaque()) {
-				node_copy->set_attr("opaque", "yes");
+				node_copy = node_copy.add_attr(variant("opaque"), variant(true));
 				opaque_ = true;
 			}
 
+			if(uses_alpha_channel) {
+				node_copy = node_copy.add_attr(variant("uses_alpha_channel"), variant(true));
+			}
+
 			graphics::color col;
 			if(calculate_is_solid_color(col)) {
 				if(palette >= 0) {
 					col = graphics::map_palette(col, palette);
 				}
-				node_copy->set_attr("solid_color", graphics::color_transform(col).to_string());
+				node_copy = node_copy.add_attr(variant("solid_color"), variant(graphics::color_transform(col).to_string()));
 			}
 
 			if(calculate_draw_area()) {
-				node_copy->set_attr("draw_area", draw_area_.to_string());
+				node_copy = node_copy.add_attr(variant("draw_area"), variant(draw_area_.to_string()));
 			}
 
 			std::string tiles_str;
 
 			foreach(int tile, tiles_) {
-				tile_id id(filename_palette_pair(node_copy->attr("image"), palette), tile);
+				tile_id id(filename_palette_pair(node_copy["image"].as_string(), palette), tile);
 				std::map<tile_id, int>::const_iterator itor = compiled_tile_ids.find(id);
 				if(itor == compiled_tile_ids.end()) {
 					compiled_tile_ids[id] = compiled_tile_ids.size();
@@ -499,29 +585,14 @@ level_object::level_object(wml::const_node_ptr node)
 				tiles_str += tile_pos;
 			}
 
-			node_copy->set_attr("image", "tiles-compiled.png");
-			node_copy->set_attr("tiles", tiles_str);
+//			node_copy = node_copy.add_attr(variant("image"), variant("tiles-compiled.png"));
+			node_copy = node_copy.add_attr(variant("tiles"), variant(tiles_str));
+			tile_str_to_palette[tiles_str] = palette;
 	
-			level_object_index.push_back(node_copy);
-			original_level_object_nodes.push_back(node);
+			level_object_index.push_back(obj_variant_ptr(new variant(node_copy)));
+			original_level_object_nodes.push_back(const_obj_variant_ptr(new variant(node)));
 		}
 	}
-
-	//debug code to output the solidity of a tile in case we need to introspect at some point
-	/*
-	std::cerr << "LEVEL_OBJECT: " << wml::output(node) << ":::\nSOLID:::\n";
-	if(solid_.size() == height()*width()) {
-		for(int y = 0; y != height(); ++y) {
-			for(int x = 0; x != width(); ++x) {
-				std::cerr << (solid_[y*width() + x] ? "1" : "0");
-			}
-			
-			std::cerr << "\n";
-		}
-	} else {
-		std::cerr << "SOLID SIZE: " << solid_.size() << "\n";
-	}
-	*/
 }
 
 level_object::~level_object()
@@ -538,7 +609,7 @@ std::vector<int> create_base64_char_to_int()
 {
 	std::vector<int> result(256);
 	int index = 0;
-	foreach(char c, base64_chars) {
+	foreach(char c, (char const *)base64_chars) {
 		result[c] = index++;
 	}
 
@@ -578,17 +649,15 @@ void level_object::write_compiled()
 	create_compiled_tiles_image();
 
 	for(int n = 0; n <= level_object_index.size()/64; ++n) {
-		char buf[128];
-		sprintf(buf, "%d", n);
-		const std::string filename = std::string(buf) + ".cfg";
-		wml::node_ptr tiles_node(new wml::node("tiles"));
+		std::stringstream ss;
+		ss << std::dec << n;
+		const std::string filename = ss.str() + ".cfg";
+		variant_builder tiles_node;
 		for(int m = n*64; m < level_object_index.size() && m < (n+1)*64; ++m) {
-			tiles_node->add_child(wml::deep_copy(level_object_index[m]));
+			tiles_node.add("tiles", *level_object_index[m]);
 		}
 
-		std::string data;
-		wml::write(tiles_node, data);
-		sys::write_file("data/compiled/tiles/" + filename, data);
+		module::write_file("data/compiled/tiles/" + filename, tiles_node.build().write_json(true));
 	}
 }
 
@@ -600,13 +669,14 @@ void load_compiled_tiles(int index)
 	int starting_index = index*64;
 	char buf[128];
 	sprintf(buf, "%d", index);
-	wml::const_node_ptr node(wml::parse_wml_from_file("data/compiled/tiles/" + std::string(buf) + ".cfg"));
+	variant node(json::parse_from_file("data/compiled/tiles/" + std::string(buf) + ".cfg"));
 	int count = 0;
-	for(wml::node::const_all_child_iterator i = node->begin_children(); i != node->end_children(); ++i) {
+
+	foreach(variant tile_node, node["tiles"].as_list()) {
 		if(starting_index >= compiled_tiles.size()) {
 			compiled_tiles.resize(starting_index+64);
 		}
-		compiled_tiles[starting_index++].reset(new level_object(*i));
+		compiled_tiles[starting_index++].reset(new level_object(tile_node));
 		++count;
 	}
 }
@@ -631,9 +701,9 @@ level_object_ptr level_object::record_zorder(int zorder) const
 	if(i == zorders_.end()) {
 		zorders_.push_back(zorder);
 		if(zorders_.size() > 1) {
-			level_object_ptr result(new level_object(original_level_object_nodes[tile_index_]));
+			level_object_ptr result(new level_object(*original_level_object_nodes[tile_index_]));
 			result->zorders_.push_back(zorder);
-			std::pair<wml::const_node_ptr, int> key(original_level_object_nodes[tile_index_], zorder);
+			std::pair<const_obj_variant_ptr, int> key(original_level_object_nodes[tile_index_], zorder);
 
 			secondary_zorder_objects[key] = result;
 
@@ -656,7 +726,7 @@ level_object_ptr level_object::record_zorder(int zorder) const
 			}
 		}
 	} else if(i != zorders_.begin()) {
-		std::pair<wml::const_node_ptr, int> key(original_level_object_nodes[tile_index_], zorder);
+		std::pair<const_obj_variant_ptr, int> key(original_level_object_nodes[tile_index_], zorder);
 		return secondary_zorder_objects[key];
 	}
 
@@ -685,12 +755,12 @@ void level_object::write_compiled_index(char* buf) const
 
 int level_object::width() const
 {
-	return 32;
+	return BaseTileSize*2;
 }
 
 int level_object::height() const
 {
-	return 32;
+	return BaseTileSize*2;
 }
 
 bool level_object::is_solid(int x, int y) const
@@ -748,6 +818,17 @@ bool level_object::calculate_opaque() const
 	return true;
 }
 
+bool level_object::calculate_uses_alpha_channel() const
+{
+	foreach(int tile, tiles_) {
+		if(is_tile_using_alpha_channel(t_, tile)) {
+			return true;
+		}
+	}
+
+	return false;
+}
+
 bool level_object::calculate_is_solid_color(graphics::color& col) const
 {
 	foreach(int tile, tiles_) {
@@ -766,7 +847,7 @@ bool level_object::calculate_draw_area()
 		draw_area_ = rect_union(draw_area_, get_tile_non_alpha_area(t_, tile));
 	}
 
-	return draw_area_ != rect(0, 0, 16, 16);
+	return draw_area_ != rect(0, 0, BaseTileSize, BaseTileSize);
 }
 
 void level_object::set_palette(unsigned int palette)
@@ -804,3 +885,32 @@ void level_object::get_palettes_used(std::vector<int>& v) const
 		++palette;
 	}
 }
+
+variant level_object::get_value(const std::string& key) const
+{
+	if(key == "id") {
+		return variant(id_);
+	} else if(key == "info") {
+		return variant(info_);
+	} else if(key == "damage") {
+		return variant(damage_);
+	} else if(key == "friction") {
+		return variant(friction_);
+	} else if(key == "traction") {
+		return variant(traction_);
+	} else {
+		return variant();
+	}
+}
+
+UNIT_TEST(level_object_base64)
+{
+	const char* s = "4O0";
+	const char* s2 = s + strlen(s);
+	const int num = base64_unencode(s, s2);
+	char buf[3];
+	base64_encode(num, buf, 3);
+	CHECK_EQ(buf[0], s[0]);
+	CHECK_EQ(buf[1], s[1]);
+	CHECK_EQ(buf[2], s[2]);
+}
