@@ -4,12 +4,11 @@
 #include "formatter.hpp"
 #include "light.hpp"
 #include "raster.hpp"
-#include "wml_node.hpp"
-#include "wml_utils.hpp"
+#include "variant_utils.hpp"
 
-light_ptr light::create_light(const custom_object& obj, wml::const_node_ptr node)
+light_ptr light::create_light(const custom_object& obj, variant node)
 {
-	if(node->attr("type").str() == "circle") {
+	if(node["type"].as_string() == "circle") {
 		return light_ptr(new circle_light(obj, node));
 	} else {
 		return light_ptr();
@@ -21,8 +20,8 @@ light::light(const custom_object& obj) : obj_(obj)
 
 light::~light() {}
 
-circle_light::circle_light(const custom_object& obj, wml::const_node_ptr node)
-  : light(obj), center_(obj.midpoint()), radius_(wml::get_int(node, "radius"))
+circle_light::circle_light(const custom_object& obj, variant node)
+  : light(obj), center_(obj.midpoint()), radius_(node["radius"].as_int())
 {}
 
 circle_light::circle_light(const custom_object& obj, int radius)
@@ -34,13 +33,13 @@ variant light::get_value(const std::string& key) const
 	return variant();
 }
 
-wml::node_ptr circle_light::write() const
+variant circle_light::write() const
 {
-	wml::node_ptr res(new wml::node("light"));
-	res->set_attr("type", "circle");
-	res->set_attr("radius", formatter() << radius_);
+	variant_builder res;
+	res.add("type", "circle");
+	res.add("radius", radius_);
 
-	return res;
+	return res.build();
 }
 
 void circle_light::process()
@@ -63,7 +62,7 @@ void circle_light::draw(const rect& screen_area, const unsigned char* color) con
 	static std::vector<float> y_angles;
 
 	if(x_angles.empty()) {
-		for(float angle = 0; angle < 3.1459*2.0; angle += 0.2) {
+		for(float angle = 0.0f; angle < 3.1459f*2.0f; angle += 0.2f) {
 			x_angles.push_back(cos(angle));
 			y_angles.push_back(sin(angle));
 		}
@@ -90,9 +89,14 @@ void circle_light::draw(const rect& screen_area, const unsigned char* color) con
 
 	glColor4ub(color[0], color[1], color[2], 255);
 
+#if defined(USE_GLES2)
+	gles2::manager gles2_manager(gles2::get_simple_shader());
+	gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, 0, 0, &varray.front());
+#else
 	glDisable(GL_TEXTURE_2D);
 	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
 	glVertexPointer(2, GL_FLOAT, 0, &varray.front());
+#endif
 	glDrawArrays(GL_TRIANGLE_FAN, 0, varray.size()/2);
 
 	varray.clear();
@@ -126,7 +130,15 @@ void circle_light::draw(const rect& screen_area, const unsigned char* color) con
 		carray.push_back(color[2]);
 		carray.push_back(0);
 	}
-
+#if defined(USE_GLES2)
+	{
+		gles2::manager gles2_manager(gles2::get_simple_col_shader());
+		// May need to use a frgament shader implementing phong shading here.
+		gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, 0, 0, &varray.front());
+		gles2::active_shader()->shader()->color_array(4, GL_UNSIGNED_BYTE, GL_TRUE, 0, &carray.front());
+		glDrawArrays(GL_TRIANGLE_STRIP, 0, varray.size()/2);
+	}
+#else
 	glEnableClientState(GL_COLOR_ARRAY);
 	glShadeModel(GL_SMOOTH);
 
@@ -138,6 +150,7 @@ void circle_light::draw(const rect& screen_area, const unsigned char* color) con
 	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 	glEnable(GL_TEXTURE_2D);
 	glShadeModel(GL_FLAT);
+#endif
 
 	glColor4ub(255, 255, 255, 255);
 }
