@@ -4,7 +4,7 @@
 #include <string>
 #include <vector>
 
-#include "wml_node_fwd.hpp"
+#include "variant.hpp"
 
 class level;
 
@@ -13,10 +13,16 @@ struct load_level_manager {
 	~load_level_manager();
 };
 
+namespace loadlevel {
+void load_level_paths();
+void reload_level_paths();
+const std::string& get_level_path(const std::string& name);
+}
+
 void clear_level_wml();
 void preload_level_wml(const std::string& lvl);
-wml::const_node_ptr load_level_wml(const std::string& lvl);
-wml::const_node_ptr load_level_wml_nowait(const std::string& lvl);
+variant load_level_wml(const std::string& lvl);
+variant load_level_wml_nowait(const std::string& lvl);
 
 void preload_level(const std::string& lvl);
 level* load_level(const std::string& lvl);
