@@ -1,39 +1,43 @@
 #include <string>
 #include <iostream>
 
+#include "foreach.hpp"
+#include "surface_cache.hpp"
 #include "loading_screen.hpp"
-#include "wml_node.hpp"
 #include "custom_object_type.hpp"
-#include "wml_utils.hpp"
 #include "raster.hpp"
 #include "graphical_font.hpp"
 #include "geometry.hpp"
 #include "i18n.hpp"
 #include "texture.hpp"
+#include "variant.hpp"
 
 loading_screen::loading_screen (int items) : items_(items), status_(0),
                                              started_at_(SDL_GetTicks())
 {
-	background_ = graphics::texture::get("backgrounds/loading_screen.png");
+	try {
+		background_ = graphics::texture::get("backgrounds/loading_screen.png");
 
-	if(graphics::screen_height() > 0 && float(graphics::screen_width())/float(graphics::screen_height()) <= 1.4) {
-		splash_ = graphics::texture::get("splash.jpg");
-	} else {
-		splash_ = graphics::texture::get("splash-wide.jpg");
+		if(graphics::screen_height() > 0 && float(graphics::screen_width())/float(graphics::screen_height()) <= 1.4) {
+			splash_ = graphics::texture::get("splash.jpg");
+		} else {
+			splash_ = graphics::texture::get("splash-wide.jpg");
+		}
+	} catch(graphics::load_image_error& e) {
 	}
 }
 
-void loading_screen::load (wml::const_node_ptr node)
+void loading_screen::load(variant node)
 {
 	//custom_object_type::get("frogatto_playable");
-	FOREACH_WML_CHILD(preload_node, node, "preload")
+	foreach(variant preload_node, node["preload"].as_list())
 	{
-		draw_and_increment(wml::get_str(preload_node, "message"));
-		if (wml::get_str(preload_node, "type") == "object")
+		draw_and_increment(preload_node["message"].as_string());
+		if(preload_node["type"].as_string() == "object")
 		{
-			custom_object_type::get(wml::get_str(preload_node, "name"));
-		} else if (wml::get_str(preload_node, "type") == "texture") {
-			graphics::texture::get(wml::get_str(preload_node, "name"));
+			custom_object_type::get(preload_node["name"].as_string());
+		} else if(preload_node["type"].as_string() == "texture") {
+			graphics::texture::get(preload_node["name"].as_string());
 		}
 	}
 }
@@ -54,6 +58,9 @@ void loading_screen::draw (const std::string& message)
 	}
 	
 	SDL_GL_SwapBuffers();
+#if defined(__ANDROID__)
+	graphics::reset_opengl_state();
+#endif
 	//SDL_Delay(500); //make it possible to see on fast computers; for debugging
 }
 
@@ -77,8 +84,10 @@ void loading_screen::draw_internal (const std::string& message)
 	
 	const_graphical_font_ptr font = graphical_font::get("door_label");
 	//explicitly translate loading messages
-	rect text_size = font->dimensions(i18n::tr(message));
-	font->draw(screen_w/2 - text_size.w()/2, screen_h/2 + bar_height/2 + 5, i18n::tr(message));
+	if(font) {
+		rect text_size = font->dimensions(i18n::tr(message));
+		font->draw(screen_w/2 - text_size.w()/2, screen_h/2 + bar_height/2 + 5, i18n::tr(message));
+	}
 }
 
 void loading_screen::increment_status ()
