@@ -1,19 +1,5 @@
-#include <SDL.h>
-#ifndef SDL_VIDEO_OPENGL_ES
-#include <GL/glew.h>
-#endif
-#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA)
-#include <GLES/gl.h>
-#ifdef TARGET_PANDORA
-#include <GLES/glues.h>
-#endif
-#else
-#include <GL/gl.h>
-#include <GL/glu.h>
-#endif
-#if defined(__APPLE__) && !(TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE)
-#include <OpenGL/OpenGL.h>
-#endif
+#include "graphics.hpp"
+
 #include <algorithm>
 #include <cmath>
 #include <iostream>
@@ -28,6 +14,8 @@
 #endif
 
 #include "asserts.hpp"
+#include "background_task_pool.hpp"
+#include "checksum.hpp"
 #include "controls.hpp"
 #include "custom_object.hpp"
 #include "custom_object_functions.hpp"
@@ -36,6 +24,8 @@
 #ifndef NO_EDITOR
 #include "editor.hpp"
 #endif
+#include "difficulty.hpp"
+#include "external_text_editor.hpp"
 #include "filesystem.hpp"
 #include "font.hpp"
 #include "foreach.hpp"
@@ -44,10 +34,10 @@
 #include "graphical_font.hpp"
 #include "gui_section.hpp"
 #include "i18n.hpp"
-#include "inventory.hpp"
 #include "iphone_device_info.h"
 #include "of_bridge.h"
 #include "joystick.hpp"
+#include "json_parser.hpp"
 #include "key.hpp"
 #include "level.hpp"
 #include "level_object.hpp"
@@ -55,6 +45,7 @@
 #include "load_level.hpp"
 #include "loading_screen.hpp"
 #include "message_dialog.hpp"
+#include "module.hpp"
 #include "multiplayer.hpp"
 #include "player_info.hpp"
 #include "preferences.hpp"
@@ -68,14 +59,23 @@
 #include "texture_frame_buffer.hpp"
 #include "tile_map.hpp"
 #include "unit_test.hpp"
-#include "wml_node.hpp"
-#include "wml_parser.hpp"
-#include "wml_schema.hpp"
-#include "wml_utils.hpp"
-#include "wml_writer.hpp"
+#include "variant_utils.hpp"
+#include "wm.hpp"
 
-#if defined(TARGET_PANDORA) | defined(TARGET_TEGRA)
+#if defined(__ANDROID__)
+#include "prof.h"
+#endif
+
+#if defined(TARGET_PANDORA) || defined(TARGET_TEGRA)
 #include "eglport.h"
+#elif defined(TARGET_BLACKBERRY)
+#include <EGL/egl.h>
+#endif
+
+#define DEFAULT_MODULE	"frogatto"
+
+#if defined(USE_GLES2)
+window_manager wm;
 #endif
 
 namespace {
@@ -101,16 +101,22 @@ void print_help(const std::string& argv0)
 "      --level LEVEL_FILE       starts the game using the specified level file,\n" <<
 "                                 relative to the level path\n" <<
 "      --level-path=PATH        sets the path to the game level files\n" <<
-"      --native			        one pixel in-game equals one pixel on monitor." <<
 "      --[no-]music             enables/disables game music\n" <<
+"      --native                 one pixel in-game equals one pixel on monitor\n" <<
 "      --relay                  use the server as a relay in multiplayer rather\n" <<
 "                                 than trying to initiate direct connections\n" <<
 "      --[no-]resizable         allows/disallows to resize the game window\n" <<
 "      --scale                  enables an experimental pixel art interpolation\n" <<
 "                                 algorithm for scaling the game graphics (some\n" <<
 "                                 issues with this still have to be solved)\n" <<
-"      --[no-]send-stats        enables/disables sending game statistics over the\n" <<
-"                                 network\n" <<
+"      --[no-]send-stats        enables/disables sending game statistics over\n"
+"                                 the network\n" <<
+"      --server=URL             sets the server to use for the TBS client based\n"
+"                                 on the given url\n" <<
+"      --user=USERNAME          sets the username to use as part of the TBS\n"
+"                                 server and module system\n" <<
+"      --pass=PASSWORD          sets the password to use as part of the TBS\n"
+"                                 server and module system\n" <<
 "      --[no-]sound             enables/disables sound and music support\n" <<
 "      --widescreen             sets widescreen mode, increasing the game view\n" <<
 "                                 area for wide screen displays\n" <<
@@ -126,7 +132,6 @@ void print_help(const std::string& argv0)
 "      --potonly                use power of two-sized textures only\n" <<
 "      --textures16             use 16 bpp textures only (default on iPhone)\n" <<
 "      --textures32             use 32 bpp textures (default on PC/Mac)\n" <<
-
 "\n" <<
 "Developer options:\n" <<
 "      --benchmarks             runs all the engine's benchmarks (intended to\n" <<
@@ -136,6 +141,7 @@ void print_help(const std::string& argv0)
 "                                 of these\n" <<
 "      --benchmarks=NAME        runs a single named benchmark code\n" <<
 "      --[no-]compiled          enable or disable precompiled game data\n" <<
+"      --edit                   starts the game in edit mode.\n" <<
 //"      --profile                FIXME\n" <<
 //"      --profile=FILE           FIXME\n" <<
 "      --show-hitboxes          turns on the display of object hitboxes\n" <<
@@ -144,33 +150,71 @@ void print_help(const std::string& argv0)
 "                                 environment\n" <<
 "      --simiphone              changes various options to emulate an iPhone\n" <<
 "                                 environment\n" <<
+"      --no-autopause           Stops the game from pausing automatically\n" <<
+"                                 when it loses focus\n" <<
 "      --tests                  runs the game's unit tests and exits\n" <<
 "      --no-tests               skips the execution of unit tests on startup\n"
 "      --utility=NAME           runs the specified UTILITY( NAME ) code block,\n" <<
-"                                 such as compile_levels or object_compiler,\n" <<
+"                                 such as compile_levels or compile_objects,\n" <<
 "                                 with the specified arguments\n"
 	;
 }
 
 }
 
-extern "C" int main(int argc, char** argv)
+#if defined(__ANDROID__)
+#include <jni.h>
+#include <android/asset_manager_jni.h>
+AAssetManager* static_assetManager = 0;
+extern "C" void app_set_asset_manager(AAssetManager* assetMan)
 {
-#if defined(TARGET_PANDORA)
-    EGL_Open();
+	static_assetManager = assetMan;	
+}
+namespace sys {
+AAssetManager* GetJavaAssetManager()
+{
+	return static_assetManager;
+}
+}
 #endif
 
-	if(SDL_Init(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK) < 0) {
-		std::cerr << "could not init SDL\n";
+#if defined(__native_client__)
+void register_file_and_data(const char* filename, const char* mode, char* buffer, int size)
+{
+}
+#endif
+
+int load_module(const std::string& mod, std::vector<std::string>* argv)
+{
+	variant mod_info = module::get(mod);
+	if(mod_info.is_null()) {
 		return -1;
 	}
+	module::reload(mod);
+	if(mod_info["arguments"].is_list()) {
+		const std::vector<std::string>& arguments = mod_info["arguments"].as_list_string();
+		argv->insert(argv->end(), arguments.begin(), arguments.end());
+	}
+	preferences::set_preferences_path_from_module(module::get_module_name());
+	return 0;
+}
 
-#ifdef TARGET_OS_HARMATTAN
-	g_type_init();
+#if defined(__native_client__)
+extern "C" int game_main(int argcount, char* argvec[])
+#else
+extern "C" int main(int argcount, char** argvec)
 #endif
-	i18n::init ();
+{
+#if defined(__native_client__)
+	std::cerr << "Running game_main" << std::endl;
 
-//	SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 1);
+	chdir("/frogatto");
+	{
+		char buf[256];
+		const char* const res = getcwd(buf,sizeof(buf));
+		std::cerr << "Current working directory: " << res << std::endl;
+	}
+#endif 
 
 	#ifdef NO_STDERR
 	std::freopen("/dev/null", "w", stderr);
@@ -178,6 +222,12 @@ extern "C" int main(int argc, char** argv)
 	#endif
 
 	std::cerr << "Frogatto engine version " << preferences::version() << "\n";
+	LOG( "After print engine version" );
+
+	#if defined(TARGET_BLACKBERRY)
+		chdir("app/native");
+		std::cout<< "Changed working directory to: " << getcwd(0, 0) << std::endl;
+	#endif
 
 	std::string level_cfg = "titlescreen.cfg";
 	bool unit_tests_only = false, skip_tests = false;
@@ -190,12 +240,81 @@ extern "C" int main(int argc, char** argv)
 	const char* profile_output = NULL;
 	std::string profile_output_buf;
 
+#if defined(__ANDROID__)
+	monstartup("libapplication.so");
+#endif
+
 	std::string orig_level_cfg = level_cfg;
 	std::string override_level_cfg = "";
 
-	preferences::load_preferences();
+	int modules_loaded = 0;
 
-	for(int n = 1; n < argc; ++n) {
+	std::vector<std::string> argv;
+	for(int n = 1; n < argcount; ++n) {
+		argv.push_back(argvec[n]);
+        
+        if(argv.size() >= 2 && argv[argv.size()-2] == "-NSDocumentRevisionsDebugMode" && argv.back() == "YES") {
+            //XCode passes these arguments by default when debugging -- make sure they are ignored.
+            argv.resize(argv.size()-2);
+        }
+	}
+
+	if(sys::file_exists("./master-config.cfg")) {
+		std::cerr << "LOADING CONFIGURATION FROM master-config.cfg" << std::endl;
+		variant cfg = json::parse_from_file("./master-config.cfg");
+		if(cfg.is_map()) {
+			if( cfg["id"].is_null() == false) {
+				std::cerr << "SETTING MODULE PATH FROM master-config.cfg: " << cfg["id"].as_string() << std::endl;
+				preferences::set_preferences_path_from_module(cfg["id"].as_string());
+				//XXX module::set_module_name(cfg["id"].as_string(), cfg["id"].as_string());
+			}
+			if(cfg["arguments"].is_null() == false) {
+				std::vector<std::string> additional_args = cfg["arguments"].as_list_string();
+				argv.insert(argv.begin(), additional_args.begin(), additional_args.end());
+				std::cerr << "ADDING ARGUMENTS FROM master-config.cfg:";
+				for(size_t n = 0; n < cfg["arguments"].num_elements(); ++n) {
+					std::cerr << " " << cfg["arguments"][n].as_string();
+				}
+				std::cerr << std::endl;
+			}
+		}
+	}
+
+	stats::record_program_args(argv);
+
+	for(size_t n = 0; n < argv.size(); ++n) {
+		const int argc = argv.size();
+		const std::string arg(argv[n]);
+		std::string arg_name, arg_value;
+		std::string::const_iterator equal = std::find(arg.begin(), arg.end(), '=');
+		if(equal != arg.end()) {
+			arg_name = std::string(arg.begin(), equal);
+			arg_value = std::string(equal+1, arg.end());
+		}
+		if(arg_name == "--module") {
+			if(load_module(arg_value, &argv) != 0) {
+				std::cerr << "FAILED TO LOAD MODULE: " << arg_value << "\n";
+				return -1;
+			}
+			++modules_loaded;
+		}
+	}
+
+	if(modules_loaded == 0) {
+		if(load_module(DEFAULT_MODULE, &argv) != 0) {
+			std::cerr << "FAILED TO LOAD MODULE: " << DEFAULT_MODULE << "\n";
+			return -1;
+		}
+	}
+
+	preferences::load_preferences();
+	LOG( "After load_preferences()" );
+
+	// load difficulty settings after module, before rest of args.
+	difficulty::manager();
+
+	for(size_t n = 0; n < argv.size(); ++n) {
+		const size_t argc = argv.size();
 		const std::string arg(argv[n]);
 		std::string arg_name, arg_value;
 		std::string::const_iterator equal = std::find(arg.begin(), arg.end(), '=');
@@ -204,7 +323,9 @@ extern "C" int main(int argc, char** argv)
 			arg_value = std::string(equal+1, arg.end());
 		}
 		
-		if(arg_name == "--profile" || arg == "--profile") {
+		if(arg_name == "--module") {
+			// ignore already processed.
+		} else if(arg_name == "--profile" || arg == "--profile") {
 			profile_output_buf = arg_value;
 			profile_output = profile_output_buf.c_str();
 		} else if(arg_name == "--utility") {
@@ -236,6 +357,10 @@ extern "C" int main(int argc, char** argv)
 			server = argv[++n];
 		} else if(arg == "--compiled") {
 			preferences::set_load_compiled(true);
+#ifndef NO_EDITOR
+		} else if(arg == "--edit") {
+			preferences::set_edit_on_start(true);
+#endif
 		} else if(arg == "--no-compiled") {
 			preferences::set_load_compiled(false);
 #if defined(TARGET_PANDORA)
@@ -245,10 +370,10 @@ extern "C" int main(int argc, char** argv)
 			preferences::set_bequ(false);
 #endif
 		} else if(arg == "--help" || arg == "-h") {
-			print_help(std::string(argv[0]));
+			print_help(std::string(argvec[0]));
 			return 0;
 		} else {
-			const bool res = preferences::parse_arg(argv[n]);
+			const bool res = preferences::parse_arg(argv[n].c_str());
 			if(!res) {
 				std::cerr << "unrecognized arg: '" << arg << "'\n";
 				return -1;
@@ -256,8 +381,16 @@ extern "C" int main(int argc, char** argv)
 		}
 	}
 
+	checksum::manager checksum_manager;
+#ifndef NO_EDITOR
+	sys::filesystem_manager fs_manager;
+#endif // NO_EDITOR
+
 	preferences::expand_data_paths();
 
+	background_task_pool::manager bg_task_pool_manager;
+	LOG( "After expand_data_paths()" );
+
 	std::cerr << "Preferences dir: " << preferences::user_data_path() << '\n';
 
 	//make sure that the user data path exists.
@@ -267,8 +400,46 @@ extern "C" int main(int argc, char** argv)
 
 	std::cerr << "\n";
 
-#ifdef TARGET_OS_IPHONE
-	//on the iPhone, try to restore the auto-save if it exists
+	if(utility_program.empty() == false && test::utility_needs_video(utility_program) == false) {
+		test::run_utility(utility_program, util_args);
+		return 0;
+	}
+
+#if defined(TARGET_PANDORA)
+    EGL_Open();
+#endif
+
+#if defined(__ANDROID__)
+	std::freopen("stdout.txt","w",stdout);
+	std::freopen("stderr.txt","w",stderr);
+	std::cerr.sync_with_stdio(true);
+#endif
+
+	LOG( "Start of main" );
+	
+#if !defined(__native_client__)
+	Uint32 sdl_init_flags = SDL_INIT_VIDEO | SDL_INIT_JOYSTICK;
+#if defined(_WINDOWS) || defined(TARGET_OS_IPHONE)
+	sdl_init_flags |= SDL_INIT_TIMER;
+#endif
+	if(SDL_Init(sdl_init_flags) < 0) {
+		std::cerr << "could not init SDL\n";
+		return -1;
+	}
+	LOG( "After SDL_Init" );
+#endif
+
+#ifdef TARGET_OS_HARMATTAN
+	g_type_init();
+#endif
+	i18n::init ();
+	LOG( "After i18n::init()" );
+
+	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);
+	SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 8);
+
+#if defined(TARGET_OS_IPHONE) || defined(TARGET_BLACKBERRY) || defined(__ANDROID__)
+	//on the iPhone and PlayBook, try to restore the auto-save if it exists
 	if(sys::file_exists(preferences::auto_save_file_path()) && sys::read_file(std::string(preferences::auto_save_file_path()) + ".stat") == "1") {
 		level_cfg = "autosave.cfg";
 		sys::write_file(std::string(preferences::auto_save_file_path()) + ".stat", "0");
@@ -281,6 +452,13 @@ extern "C" int main(int argc, char** argv)
 		orig_level_cfg = level_cfg;
 	}
 
+#if defined(USE_GLES2)
+	wm.create_window(preferences::actual_screen_width(),
+		preferences::actual_screen_height(),
+		0,
+		(preferences::resizable() ? SDL_RESIZABLE : 0) | (preferences::fullscreen() ? SDL_FULLSCREEN : 0));
+#else
+
 #if TARGET_OS_IPHONE || TARGET_IPHONE_SIMULATOR
 	int width, height;
 	iphone_screen_res(&width, &height);
@@ -338,8 +516,7 @@ extern "C" int main(int argc, char** argv)
 	}
     EGL_Init();
     preferences::init_oes();
-#else
-#if defined(TARGET_TEGRA)
+#elif defined(TARGET_TEGRA)
 	//if (SDL_SetVideoMode(preferences::actual_screen_width(),preferences::actual_screen_height(),0,preferences::resizable() ? SDL_RESIZABLE : 0|preferences::fullscreen() ? SDL_FULLSCREEN : 0) == NULL) {
 	if (SDL_SetVideoMode(preferences::actual_screen_width(),preferences::actual_screen_height(),0,SDL_FULLSCREEN) == NULL) {
 		std::cerr << "could not set video mode\n";
@@ -347,6 +524,50 @@ extern "C" int main(int argc, char** argv)
 	}
     EGL_Init();
     preferences::init_oes();
+#elif defined(TARGET_BLACKBERRY)
+	if (SDL_SetVideoMode(preferences::actual_screen_width(),preferences::actual_screen_height(),0,SDL_OPENGL|SDL_FULLSCREEN) == NULL) {
+		std::cerr << "could not set video mode\n";
+		return -1;
+	}
+	preferences::init_oes();
+#elif defined(__ANDROID__)
+    SDL_Rect** r = SDL_ListModes(NULL, SDL_FULLSCREEN|SDL_OPENGL);
+    if( r != (SDL_Rect**)0 && r != (SDL_Rect**)-1 ) {
+        preferences::set_actual_screen_width(r[0]->w);
+        preferences::set_actual_screen_height(r[0]->h);
+		if(r[0]->w < 640) {
+        	preferences::set_virtual_screen_width(r[0]->w*2);
+        	preferences::set_virtual_screen_height(r[0]->h*2);
+		} else {
+			preferences::set_virtual_screen_width(r[0]->w);
+			preferences::set_virtual_screen_height(r[0]->h);
+		}
+		preferences::set_control_scheme(r[0]->h >= 1024 ? "ipad_2d" : "android_med");
+    }
+
+    if (SDL_SetVideoMode(preferences::actual_screen_width(),preferences::actual_screen_height(),0,SDL_FULLSCREEN|SDL_OPENGL) == NULL) {
+		std::cerr << "could not set video mode\n";
+		return -1;
+    }
+#elif defined(__native_client__)
+    SDL_Rect** r = SDL_ListModes(NULL, SDL_OPENGL);
+	std::cerr << "Video modes";
+	if(r == (SDL_Rect**)0) {
+		std::cerr << "No modes available";
+		return -1;
+	}
+	if(r == (SDL_Rect**)-1) {
+		std::cerr << "All modes available";
+	} else {
+		for(int i = 0; r[i]; ++i) {
+			std::cerr << r[i]->w << r[i]->h << std::endl;
+		}
+	}
+
+    if (SDL_SetVideoMode(preferences::actual_screen_width(),preferences::actual_screen_height(),0,0) == NULL) {
+		std::cerr << "could not set video mode\n";
+		return -1;
+    }
 #else
 	if (SDL_SetVideoMode(preferences::actual_screen_width(),preferences::actual_screen_height(),0,SDL_OPENGL|(preferences::resizable() ? SDL_RESIZABLE : 0)|(preferences::fullscreen() ? SDL_FULLSCREEN : 0)) == NULL) {
 		std::cerr << "could not set video mode\n";
@@ -354,29 +575,94 @@ extern "C" int main(int argc, char** argv)
 	}
 #endif
 #endif
+
 #endif
 
+#endif // USE_GLES2
+
+#if defined(__GLEW_H__)
+	GLenum glew_status = glewInit();
+	ASSERT_EQ(glew_status, GLEW_OK);
+#endif
+
+#if defined(USE_GLES2)
+	if(glCreateShader == NULL) {
+		const GLubyte* glstrings;
+		if(glGetString && (glstrings = glGetString(GL_VERSION)) != NULL) {
+			std::cerr << "OpenGL version: " << reinterpret_cast<const char *>(glstrings) << std::endl;
+		}
+		std::cerr << "glCreateShader is NULL. Check that your current video card drivers support"
+			<< "an OpenGL version >= 2. Exiting." << std::endl;
+		return 0;
+	}
+	// Has to happen after the call to glewInit().
+	gles2::init_default_shader();
 #endif
 
 //	srand(time(NULL));
 
 	const stats::manager stats_manager;
+#ifndef NO_EDITOR
+	const external_text_editor::manager editor_manager;
+#endif // NO_EDITOR
+
+	std::cerr << std::endl;
+	const GLubyte* glstrings;
+	if((glstrings = glGetString(GL_VENDOR)) != NULL) {
+		std::cerr << "OpenGL vendor: " << reinterpret_cast<const char *>(glstrings) << std::endl;
+	} else {
+		GLenum err = glGetError();
+		std::cerr << "Error in vendor string: " << std::hex << err << std::endl;
+	}
+	if((glstrings = glGetString(GL_VERSION)) != NULL) {
+		std::cerr << "OpenGL version: " << reinterpret_cast<const char *>(glstrings) << std::endl;
+	} else {
+		GLenum err = glGetError();
+		std::cerr << "Error in version string: " << std::hex << err << std::endl;
+	}
+	if((glstrings = glGetString(GL_EXTENSIONS)) != NULL) {
+		std::cerr << "OpenGL extensions: " << reinterpret_cast<const char *>(glstrings) << std::endl;
+	} else {
+		GLenum err = glGetError();
+		std::cerr << "Error in extensions string: " << std::hex << err << std::endl;
+	}
+#ifdef GL_SHADING_LANGUAGE_VERSION
+	if((glstrings = glGetString(GL_SHADING_LANGUAGE_VERSION)) != NULL) {
+		std::cerr << "GLSL Version: " << reinterpret_cast<const char *>(glstrings) << std::endl;
+	} else {
+		GLenum err = glGetError();
+		std::cerr << "Error in GLSL string: " << std::hex << err << std::endl;
+	}
+#endif
+	std::cerr << std::endl;
+
+	GLint stencil_bits = 0;
+	glGetIntegerv(GL_STENCIL_BITS, &stencil_bits);
+	std::cerr << "Stencil bits: " << stencil_bits << std::endl;
+
+#if defined(USE_GLES2)
+	GLfloat min_pt_sz;
+	glGetFloatv(GL_POINT_SIZE_MIN, &min_pt_sz);
+	GLfloat max_pt_sz;
+	glGetFloatv(GL_POINT_SIZE_MAX, &max_pt_sz);
+	std::cerr << "Point size range: " << min_pt_sz << " < size < " << max_pt_sz << std::endl;
+#if !defined(GL_ES_VERSION_2_0)
+	glEnable(GL_POINT_SPRITE);
+	glEnable(GL_VERTEX_PROGRAM_POINT_SIZE);
+#endif
+#endif
 
-	std::cerr
-		<< "\n"
-		<< "OpenGL vendor: " << reinterpret_cast<const char *>(glGetString(GL_VENDOR)) << "\n"
-		<< "OpenGL version: " << reinterpret_cast<const char *>(glGetString(GL_VERSION)) << "\n"
-		<< "OpenGL extensions: " << reinterpret_cast<const char *>(glGetString(GL_EXTENSIONS)) << "\n"
-		<< "\n";
 	glShadeModel(GL_SMOOTH);
 	glEnable(GL_BLEND);
+#if !defined(USE_GLES2)
 	glEnable(GL_TEXTURE_2D);
 	glEnableClientState(GL_VERTEX_ARRAY);
 	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-	
+#endif
+
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-	SDL_WM_SetCaption("Frogatto", "Frogatto");
+	SDL_WM_SetCaption(module::get_module_pretty_name().c_str(), module::get_module_pretty_name().c_str());
 
 	std::cerr << "JOYSTICKS: " << SDL_NumJoysticks() << "\n";
 
@@ -385,9 +671,12 @@ extern "C" int main(int argc, char** argv)
 	{ //manager scope
 	const font::manager font_manager;
 	const sound::manager sound_manager;
+#if !defined(__native_client__)
 	const joystick::manager joystick_manager;
-		
-	#if !TARGET_OS_IPHONE && !TARGET_IPHONE_SIMULATOR
+#endif 
+	
+	#if !TARGET_OS_IPHONE && !TARGET_IPHONE_SIMULATOR && !defined(__native_client__)
+	std::cerr << "SDL_GetVideoSurface()" << std::endl;
 	const SDL_Surface* fb = SDL_GetVideoSurface();
 	if(fb == NULL) {
 		return 0;
@@ -396,40 +685,49 @@ extern "C" int main(int argc, char** argv)
 
 	graphics::texture::manager texture_manager;
 
-	wml::const_node_ptr preloads;
+#ifndef NO_EDITOR
+	editor::manager editor_manager;
+#endif
+
+	variant preloads;
 	loading_screen loader;
-	
 	try {
-		wml::schema::init(wml::parse_wml_from_file("data/schema.cfg"));
+		sound::init_music(json::parse_from_file("data/music.cfg"));
 
 		std::string filename = "data/fonts." + i18n::get_locale() + ".cfg";
 		if (!sys::file_exists(filename))
 			filename = "data/fonts.cfg";
-		graphical_font::init(wml::parse_wml_from_file(filename));
+		std::cerr << "LOADING FONT: " << filename << " -> " << module::map_file(filename) << "\n";
+		graphical_font::init(json::parse_from_file(filename));
 
-		preloads = wml::parse_wml_from_file("data/preload.cfg");
-		int preload_items = std::distance(preloads->begin_child("preload"), preloads->end_child("preload"));
+		preloads = json::parse_from_file("data/preload.cfg");
+		int preload_items = preloads["preload"].num_elements();
 		loader.set_number_of_items(preload_items+7); // 7 is the number of items that will be loaded below
-		loader.load(preloads);
-		loader.draw_and_increment(_("Initializing custom object"));
 		custom_object::init();
 		loader.draw_and_increment(_("Initializing custom object functions"));
-		init_custom_object_functions(wml::parse_wml_from_file("data/functions.cfg"));
+		init_custom_object_functions(json::parse_from_file("data/functions.cfg"));
+		loader.draw_and_increment(_("Initializing textures"));
+		loader.load(preloads);
 		loader.draw_and_increment(_("Initializing tiles"));
-		tile_map::init(wml::parse_wml_from_file("data/tiles.cfg",
-		               wml::schema::get("tiles")));
+		tile_map::init(json::parse_from_file("data/tiles.cfg"));
 		loader.draw_and_increment(_("Initializing GUI"));
 
-		wml::const_node_ptr gui_node = wml::parse_wml_from_file(preferences::load_compiled() ? "data/compiled/gui.cfg" : "data/gui.cfg");
+		variant gui_node = json::parse_from_file(preferences::load_compiled() ? "data/compiled/gui.cfg" : "data/gui.cfg");
 		gui_section::init(gui_node);
 		loader.draw_and_increment(_("Initializing GUI"));
 		framed_gui_element::init(gui_node);
-	} catch(const wml::parse_error& e) {
+
+	} catch(const json::parse_error& e) {
+		std::cerr << "ERROR PARSING: " << e.error_message() << "\n";
 		return 0;
 	}
-	
 	loader.draw(_("Loading level"));
 
+#if defined(__native_client__)
+	while(1) {
+	}
+#endif
+
 	if(!skip_tests && !test::run_tests()) {
 		return -1;
 	}
@@ -437,19 +735,12 @@ extern "C" int main(int argc, char** argv)
 	if(unit_tests_only) {
 		return 0;
 	}
-
 #if defined(__APPLE__) && !(TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE)
 	GLint swapInterval = 1;
 	CGLSetParameter(CGLGetCurrentContext(), kCGLCPSwapInterval, &swapInterval);
 #endif
 
-#ifndef SDL_VIDEO_OPENGL_ES
-	GLenum glew_status = glewInit();
-	ASSERT_EQ(glew_status, GLEW_OK);
-#endif
-
 	loader.finish_loading();
-
 	//look to see if we got any quit events while loading.
 	{
 	SDL_Event event;
@@ -461,7 +752,12 @@ extern "C" int main(int argc, char** argv)
 	}
 
 	formula_profiler::manager profiler(profile_output);
+
+#ifdef USE_GLES2
+	texture_frame_buffer::init(preferences::actual_screen_width(), preferences::actual_screen_height());
+#else
 	texture_frame_buffer::init();
+#endif
 
 	if(run_benchmarks) {
 		if(benchmarks_list.empty() == false) {
@@ -489,6 +785,7 @@ extern "C" int main(int argc, char** argv)
 		}
 #endif
 
+#if !defined(__native_client__)
 		//see if we're loading a multiplayer level, in which case we
 		//connect to the server.
 		multiplayer::manager mp_manager(lvl->is_multiplayer());
@@ -513,6 +810,7 @@ extern "C" int main(int argc, char** argv)
 
 			lvl->set_multiplayer_slot(multiplayer::slot());
 		}
+#endif
 
 		last_draw_position() = screen_position();
 
@@ -535,18 +833,23 @@ extern "C" int main(int argc, char** argv)
 		}
 	}
 
-	} //end manager scope, make managers destruct before calling SDL_Quit
+	level::clear_current_level();
 
+	} //end manager scope, make managers destruct before calling SDL_Quit
 //	controls::debug_dump_controls();
 #if defined(TARGET_PANDORA) || defined(TARGET_TEGRA)
     EGL_Destroy();
 #endif
 
+#if defined(USE_GLES2)
+	wm.destroy_window();
+#endif
+
 	SDL_Quit();
 	
 	preferences::save_preferences();
 	std::cerr << SDL_GetError() << "\n";
-#if !defined(TARGET_OS_HARMATTAN) && !defined(TARGET_TEGRA)
+#if !defined(TARGET_OS_HARMATTAN) && !defined(TARGET_TEGRA) && !defined(TARGET_BLACKBERRY) && !defined(__ANDROID__) && !defined(USE_GLES2)
 	std::cerr << gluErrorString(glGetError()) << "\n";
 #endif
 	return 0;
