@@ -1,10 +1,14 @@
+#ifdef _WINDOWS
+#define WIN32_LEAN_AND_MEAN
+#include <Windows.h>
+#endif
+
 #include <sstream>
 #include <string>
 
-#include <inttypes.h>
+#include <boost/cstdint.hpp>
 #include <numeric>
 #include <stdio.h>
-#include <unistd.h>
 
 #include <boost/asio.hpp>
 #include <boost/regex.hpp>
@@ -287,7 +291,7 @@ void sync_start_time(const level& lvl, boost::function<bool()> idle_fn)
 			boost::array<char, 4096> udp_msg;
 			udp::endpoint endpoint;
 			size_t len = udp_socket->receive_from(boost::asio::buffer(udp_msg), endpoint);
-			if(len == 6 && toupper(udp_msg[0]) == 'A') {
+			if(len == 6 && ::toupper(udp_msg[0]) == 'A') {
 				confirmed_players.insert(udp_msg[5]);
 				if(udp_msg[5] >= 0 && udp_msg[5] < udp_endpoint_peers.size()) {
 					if(endpoint.port() != udp_endpoint_peers[udp_msg[5]]->port()) {
@@ -435,8 +439,6 @@ void sync_start_time(const level& lvl, boost::function<bool()> idle_fn)
 						const int player_num = receive_buf[5];
 						send_confirm_packet(player_num, msg, true);
 					}
-
-					fprintf(stderr, "RECEIVED UDP PACKET: %d/%c\n", len, (len > 0 ? receive_buf[0] : '-'));
 				}
 			}
 
