@@ -9,6 +9,7 @@ namespace {
 std::vector<std::string> create_object_event_names()
 {
 	std::vector<std::string> res;
+	res.push_back("any");
 	res.push_back("start_level");
 	res.push_back("player_death");
 	res.push_back("load");
@@ -41,6 +42,8 @@ std::vector<std::string> create_object_event_names()
 	res.push_back("draw");
 	res.push_back("begin_dialog");
 	res.push_back("cosmic_shift");
+	res.push_back("schedule_finished");
+	res.push_back("outside_level");
 
 	ASSERT_EQ(res.size(), NUM_OBJECT_BUILTIN_EVENT_IDS);
 	return res;
