@@ -2,7 +2,9 @@
 #define OBJECT_EVENTS_HPP_INCLUDED
 
 enum OBJECT_EVENT_ID {
+	OBJECT_EVENT_ANY,
 	OBJECT_EVENT_START_LEVEL,
+	OBJECT_EVENT_PLAYER_DEATH,
 	OBJECT_EVENT_LOAD,
 	OBJECT_EVENT_LOAD_CHECKPOINT,
 	OBJECT_EVENT_CREATE,
@@ -15,7 +17,7 @@ enum OBJECT_EVENT_ID {
 	OBJECT_EVENT_COLLIDE_HEAD,
 	OBJECT_EVENT_COLLIDE_FEET,
 	OBJECT_EVENT_COLLIDE_DAMAGE,
-	OBJECT_EVENT_COLLIDE,
+	OBJECT_EVENT_COLLIDE_SIDE,
 	OBJECT_EVENT_STUCK,
 	OBJECT_EVENT_JUMPED_ON,
 	OBJECT_EVENT_GET_HIT,
@@ -32,6 +34,9 @@ enum OBJECT_EVENT_ID {
 	OBJECT_EVENT_SPAWNED,
 	OBJECT_EVENT_DRAW,
 	OBJECT_EVENT_BEGIN_DIALOG,
+	OBJECT_EVENT_COSMIC_SHIFT,
+	OBJECT_EVENT_SCHEDULE_FINISHED,
+	OBJECT_EVENT_OUTSIDE_LEVEL,
 	NUM_OBJECT_BUILTIN_EVENT_IDS,
 };
 
