@@ -3,19 +3,25 @@
 
 #include <string>
 
-#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA)
+#include "graphics.hpp"
+#include "uri.hpp"
+#include "variant.hpp"
+#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA) || defined(TARGET_BLACKBERRY)
 #include <EGL/egl.h>
-#include <GLES/gl.h>
-#include <GLES/glext.h>
 #endif
 
 namespace game_logic {
 class formula_callable;
 }
 
+#ifdef _WINDOWS
+std::string GetAppDataPath();
+#endif
+
 namespace preferences {
 
 	const std::string& version();
+	const variant& version_decimal();
 	int get_unique_user_id();
 
 	bool parse_arg(const char* arg);
@@ -23,15 +29,22 @@ namespace preferences {
 	bool no_music();
 
 	void set_preferences_path(const std::string& path);
+	void set_preferences_path_from_module(const std::string& name);
 
 	bool setup_preferences_dir();
 
 	const std::string& level_path();
+	bool is_level_path_set();
 	const char* user_data_path();
 	const char* save_file_path();
 	const char* auto_save_file_path();
+	bool editor_save_to_user_preferences();
+
+	std::string dlc_path();
 	void expand_data_paths();
+	void set_save_slot(const std::string& fname);
 	bool show_debug_hitboxes();
+	bool toogle_debug_hitboxes();
 	bool show_iphone_controls(); //iphone control hit rects
 	bool use_pretty_scaling();
 	void set_use_pretty_scaling(bool value);
@@ -39,6 +52,7 @@ namespace preferences {
 	void set_fullscreen(bool value);
 
 	bool resizable();
+	bool proportional_resize();
 	
 	// Reverse A and B buttons for iPhone
 	bool reverse_ab();
@@ -70,8 +84,11 @@ namespace preferences {
 	bool load_compiled();
 
 	void set_load_compiled(bool value);
+	
+	void set_edit_on_start(bool value);
+	bool edit_on_start();
 
-#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA)
+#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA) || defined(TARGET_BLACKBERRY)
 bool use_fbo();
 bool use_bequ();
 void set_fbo( bool value );
@@ -94,6 +111,8 @@ void set_bequ( bool value );
 	void set_virtual_screen_width(int width);
 	void set_virtual_screen_height(int height);
 	
+	bool allow_autopause();
+
 	bool screen_rotated();
 	
 	bool force_no_npot_textures();
@@ -102,18 +121,33 @@ void set_bequ( bool value );
 
 	bool sim_iphone();
 
+	bool no_iphone_controls();
+
 	bool send_stats();
 
+	int force_difficulty();
+
+	bool record_history();
+	void set_record_history(bool value);
+
 	bool relay_through_server();
 	void set_relay_through_server(bool value);
 
+	variant external_code_editor();
+
 	bool run_failing_unit_tests();
 
+	bool serialize_bad_objects();
+
 	game_logic::formula_callable* registry();
 
 	void load_preferences();
 	void save_preferences();
 
+	uri::uri get_tbs_uri();
+	std::string get_username();
+	std::string get_password();
+
 	class editor_screen_size_scope {
 		int width_, height_;
 	public:
@@ -121,7 +155,7 @@ void set_bequ( bool value );
 		~editor_screen_size_scope();
 	};
 
-#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA)
+#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA) || defined(TARGET_BLACKBERRY)
 	void init_oes( void );
 	extern PFNGLBLENDEQUATIONOESPROC           glBlendEquationOES;
 	extern PFNGLGENFRAMEBUFFERSOESPROC         glGenFramebuffersOES;
