@@ -10,16 +10,7 @@
  
  See the COPYING file for more details.
  */
-#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA)
-#include <GLES/gl.h>
-#ifdef TARGET_PANDORA
-#include <GLES/glues.h>
-#endif
-#else
-#include <GL/gl.h>
-#include <GL/glu.h>
-#endif
-#include <SDL.h>
+ #include "graphics.hpp"
 
 #include "asserts.hpp"
 #include "foreach.hpp"
@@ -35,6 +26,70 @@
 namespace graphics
 {
 
+namespace {
+
+static void transform_point(GLfloat out[4], const GLfloat m[16], const GLfloat in[4])
+{
+#define M(row,col)  m[col*4+row]
+	out[0] = M(0, 0) * in[0] + M(0, 1) * in[1] + M(0, 2) * in[2] + M(0, 3) * in[3];
+	out[1] = M(1, 0) * in[0] + M(1, 1) * in[1] + M(1, 2) * in[2] + M(1, 3) * in[3];
+	out[2] = M(2, 0) * in[0] + M(2, 1) * in[1] + M(2, 2) * in[2] + M(2, 3) * in[3];
+	out[3] = M(3, 0) * in[0] + M(3, 1) * in[1] + M(3, 2) * in[2] + M(3, 3) * in[3];
+#undef M
+}
+
+static GLboolean gluProjectf(GLfloat objx, GLfloat objy, GLfloat objz,
+					 const GLfloat model[16], const GLfloat proj[16],
+					 const GLint viewport[4],
+					 GLfloat * winx, GLfloat * winy, GLfloat * winz)
+{
+	// transformation matrix
+	GLfloat in[4], out[4];
+	
+	// Initialise transformation matrix with vector
+	in[0] = objx;
+	in[1] = objy;
+	in[2] = objz;
+	in[3] = 1.0f;
+	transform_point(out, model, in);
+	transform_point(in, proj, out);
+	
+	// Normalise
+	if (in[3] == 0.0f)
+		return GL_FALSE;
+	
+	in[0] /= in[3];
+	in[1] /= in[3];
+	in[2] /= in[3];
+	
+	*winx = viewport[0] + (1.0f + in[0]) * viewport[2] / 2.0f;
+	*winy = viewport[1] + (1.0f + in[1]) * viewport[3] / 2.0f;
+	*winz = (1.0f + in[2]) / 2.0f;
+	return GL_TRUE;
+}
+
+}
+
+void reset_opengl_state()
+{
+	glShadeModel(GL_SMOOTH);
+	glEnable(GL_BLEND);
+	glEnable(GL_TEXTURE_2D);
+#if !defined(USE_GLES2)
+	glEnableClientState(GL_VERTEX_ARRAY);
+	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+#endif
+
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+
+    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
+#if defined(USE_GLES2)
+	glClearColor(0.0,0.0,0.0,0.0);
+	gles2::init_default_shader();
+#endif
+}
+
 bool set_video_mode(int w, int h)
 {
 #ifdef TARGET_OS_HARMATTAN
@@ -49,15 +104,9 @@ SDL_Surface* set_video_mode(int w, int h, int bitsperpixel, int flags)
 {
 	graphics::texture::unbuild_all();
 	SDL_Surface* result = SDL_SetVideoMode(w,h,bitsperpixel,flags);
+	reset_opengl_state();
 	graphics::texture::rebuild_all();
-
-	glShadeModel(GL_SMOOTH);
-	glEnable(GL_BLEND);
-	glEnable(GL_TEXTURE_2D);
-	glEnableClientState(GL_VERTEX_ARRAY);
-	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 	
-	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	return result;
 }
 	
@@ -83,7 +132,7 @@ SDL_Surface* set_video_mode(int w, int h, int bitsperpixel, int flags)
 		return v;
 	}
 	
-#ifdef SDL_VIDEO_OPENGL_ES
+#if defined(SDL_VIDEO_OPENGL_ES) && !defined(__native_client__)
 #define glOrtho glOrthof
 #endif
 	
@@ -96,6 +145,8 @@ SDL_Surface* set_video_mode(int w, int h, int bitsperpixel, int flags)
 		//	real_w = 320;
 		//	real_h = 480;
 		//	rotated = true;
+#elif defined(__native_client__)
+		// do nothing.
 #else
 		const SDL_Surface* fb = SDL_GetVideoSurface();
 		if(fb == NULL) {
@@ -116,9 +167,17 @@ SDL_Surface* set_video_mode(int w, int h, int bitsperpixel, int flags)
 		
 		if(preferences::screen_rotated()) {
 			//		glOrtho(0, 640, 960, 0, -1.0, 1.0);
+#if defined(USE_GLES2) && defined(GL_ES_VERSION_2_0)
+			glOrthof(0, screen_height(), screen_width(), 0, -1.0, 1.0);
+#else
 			glOrtho(0, screen_height(), screen_width(), 0, -1.0, 1.0);
+#endif
 		} else {
+#if defined(USE_GLES2) && defined(GL_ES_VERSION_2_0)
+			glOrthof(0, screen_width(), screen_height(), 0, -1.0, 1.0);
+#else
 			glOrtho(0, screen_width(), screen_height(), 0, -1.0, 1.0);
+#endif
 		}
 		
 		//glOrtho(0, real_w, real_h, 0, -1.0, 1.0);
@@ -135,8 +194,10 @@ SDL_Surface* set_video_mode(int w, int h, int bitsperpixel, int flags)
 		glLoadIdentity();
 		
 		glDisable(GL_DEPTH_TEST);
+#if !defined(USE_GLES2)
 		glDisable(GL_LIGHTING);
 		glDisable(GL_LIGHT0);
+#endif
 		
 		glColor4f(1.0, 1.0, 1.0, 1.0);
 	}
@@ -228,8 +289,14 @@ SDL_Surface* set_video_mode(int w, int h, int bitsperpixel, int flags)
 			texture::get_coord_x(1.0), texture::get_coord_y(0.0),
 			texture::get_coord_x(1.0), texture::get_coord_y(1.0)
 		};
+#if defined(USE_GLES2)
+		gles2::active_shader()->prepare_draw();
+		gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, 0, 0, varray);
+		gles2::active_shader()->shader()->texture_array(2, GL_FLOAT, 0, 0, tcarray);
+#else
 		glVertexPointer(2, GL_FLOAT, 0, varray);
 		glTexCoordPointer(2, GL_FLOAT, 0, tcarray);
+#endif
 		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
 		
 		glPopMatrix();
@@ -303,8 +370,14 @@ SDL_Surface* set_video_mode(int w, int h, int bitsperpixel, int flags)
 				texture::get_coord_x(x2), texture::get_coord_y(y1),
 				texture::get_coord_x(x2), texture::get_coord_y(y2)
 			};
+#if defined(USE_GLES2)
+			gles2::active_shader()->prepare_draw();
+			gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, 0, 0, varray);
+			gles2::active_shader()->shader()->texture_array(2, GL_FLOAT, 0, 0, tcarray);
+#else
 			glVertexPointer(2, GL_FLOAT, 0, varray);
 			glTexCoordPointer(2, GL_FLOAT, 0, tcarray);
+#endif
 			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
 			glPopMatrix();
 		}
@@ -380,8 +453,14 @@ SDL_Surface* set_video_mode(int w, int h, int bitsperpixel, int flags)
 						distort.distort_point(&points[n*2], &points[n*2 + 1]);
 					}
 					
+#if defined(USE_GLES2)
+					gles2::active_shader()->prepare_draw();
+					gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, 0, 0, points);
+					gles2::active_shader()->shader()->texture_array(2, GL_FLOAT, 0, 0, uv);
+#else
 					glVertexPointer(2, GL_FLOAT, 0, points);
 					glTexCoordPointer(2, GL_FLOAT, 0, uv);
+#endif
 					glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
 				}
 			}
@@ -557,8 +636,14 @@ void flush_blit_texture()
 	}
 
 	blit_current_texture->set_as_current_texture();
+#if defined(USE_GLES2)
+	gles2::active_shader()->prepare_draw();
+	gles2::active_shader()->shader()->vertex_array(2, GL_SHORT, 0, 0, &blit_vqueue.front());
+	gles2::active_shader()->shader()->texture_array(2, GL_FLOAT, 0, 0,  &blit_tcqueue.front());
+#else
 	glVertexPointer(2, GL_SHORT, 0, &blit_vqueue.front());
 	glTexCoordPointer(2, GL_FLOAT, 0, &blit_tcqueue.front());
+#endif
 	glDrawArrays(GL_TRIANGLE_STRIP, 0, blit_tcqueue.size()/2);
 
 	blit_current_texture = NULL;
@@ -581,8 +666,14 @@ void blit_queue::do_blit() const
 
 	texture::set_current_texture(texture_);
 
+#if defined(USE_GLES2)
+	gles2::active_shader()->prepare_draw();
+	gles2::active_shader()->shader()->vertex_array(2, GL_SHORT, 0, 0, &vertex_.front());
+	gles2::active_shader()->shader()->texture_array(2, GL_FLOAT, 0, 0,  &uv_.front());
+#else
 	glVertexPointer(2, GL_SHORT, 0, &vertex_.front());
 	glTexCoordPointer(2, GL_FLOAT, 0, &uv_.front());
+#endif
 	glDrawArrays(GL_TRIANGLE_STRIP, 0, uv_.size()/2);
 }
 
@@ -594,8 +685,14 @@ void blit_queue::do_blit_range(short begin, short end) const
 
 	texture::set_current_texture(texture_);
 
+#if defined(USE_GLES2)
+	gles2::active_shader()->prepare_draw();
+	gles2::active_shader()->shader()->vertex_array(2, GL_SHORT, 0, 0, &vertex_[begin]);
+	gles2::active_shader()->shader()->texture_array(2, GL_FLOAT, 0, 0,  &uv_[begin]);
+#else
 	glVertexPointer(2, GL_SHORT, 0, &vertex_[begin]);
 	glTexCoordPointer(2, GL_FLOAT, 0, &uv_[begin]);
+#endif
 	glDrawArrays(GL_TRIANGLE_STRIP, 0, (end - begin)/2);
 }
 
@@ -655,66 +752,88 @@ bool blit_queue::merge(const blit_queue& q, short begin, short end)
 	void draw_rect(const SDL_Rect& r, const SDL_Color& color,
 				   unsigned char alpha)
 	{
-		glDisable(GL_TEXTURE_2D);
-		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-		glColor4ub(color.r,color.g,color.b,alpha);
 		GLfloat varray[] = {
 			r.x, r.y,
 			r.x+r.w, r.y,
 			r.x, r.y+r.h,
 			r.x+r.w, r.y+r.h
 		};
+#if defined(USE_GLES2)
+		glColor4ub(color.r,color.g,color.b,alpha);
+		gles2::manager gles2_manager(gles2::get_simple_shader());
+		gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, 0, 0, varray);
+		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+		glColor4f(1.0, 1.0, 1.0, 1.0);
+#else
+		glDisable(GL_TEXTURE_2D);
+		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+		glColor4ub(color.r,color.g,color.b,alpha);
 		glVertexPointer(2, GL_FLOAT, 0, varray);
 		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
 		//glRecti(r.x,r.y,r.x+r.w,r.y+r.h);
 		glColor4ub(255, 255, 255, 255);
 		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 		glEnable(GL_TEXTURE_2D);
+#endif
 	}
 	
 	void draw_rect(const rect& r, const graphics::color& color)
 	{
-		glDisable(GL_TEXTURE_2D);
-		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-		glColor4ub(color.r(),color.g(),color.b(),color.a());
 		GLfloat varray[] = {
 			r.x(), r.y(),
 			r.x()+r.w(), r.y(),
 			r.x(), r.y()+r.h(),
 			r.x()+r.w(), r.y()+r.h()
 		};
+#if defined(USE_GLES2)
+		glColor4ub(color.r(),color.g(),color.b(),color.a());
+		gles2::manager gles2_manager(gles2::get_simple_shader());
+		gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, 0, 0, varray);
+		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+		glColor4f(1.0, 1.0, 1.0, 1.0);
+#else
+		glDisable(GL_TEXTURE_2D);
+		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+		glColor4ub(color.r(),color.g(),color.b(),color.a());
 		glVertexPointer(2, GL_FLOAT, 0, varray);
 		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
 		//glRecti(r.x(),r.y(),r.x()+r.w(),r.y()+r.h());
 		glColor4ub(255, 255, 255, 255);
 		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 		glEnable(GL_TEXTURE_2D);
+#endif
 	}
 	
 	
 	void draw_hollow_rect(const SDL_Rect& r, const SDL_Color& color,
-						  unsigned char alpha) {
-		
-		glDisable(GL_TEXTURE_2D);
-		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-		glColor4ub(color.r, color.g, color.b, alpha);
+						  unsigned char alpha)
+	{
 		GLfloat varray[] = {
 			r.x, r.y,
 			r.x + r.w, r.y,
 			r.x + r.w, r.y + r.h,
 			r.x, r.y + r.h
 		};
+#if defined(USE_GLES2)
+		glColor4ub(color.r, color.g, color.b, alpha);
+		gles2::manager gles2_manager(gles2::get_simple_shader());
+		gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, 0, 0, varray);
+		glDrawArrays(GL_LINE_LOOP, 0, sizeof(varray)/sizeof(GLfloat)/2);
+		glColor4f(1.0, 1.0, 1.0, 1.0);
+#else
+		glDisable(GL_TEXTURE_2D);
+		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+		glColor4ub(color.r, color.g, color.b, alpha);
 		glVertexPointer(2, GL_FLOAT, 0, varray);
 		glDrawArrays(GL_LINE_LOOP, 0, sizeof(varray)/sizeof(GLfloat)/2);
 		glColor4ub(255, 255, 255, 255);
 		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 		glEnable(GL_TEXTURE_2D);
+#endif
 	}
 
 	void draw_circle(int x, int y, int radius)
 	{
-		glDisable(GL_TEXTURE_2D);
-		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
 		static std::vector<GLfloat> varray;
 		varray.clear();
 		varray.push_back(x);
@@ -730,88 +849,75 @@ bool blit_queue::merge(const blit_queue& q, short begin, short end)
 		varray.push_back(varray[2]);
 		varray.push_back(varray[3]);
 
+#if defined(USE_GLES2)
+		gles2::manager gles2_manager(gles2::get_simple_shader());
+		gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, 0, 0, &varray.front());
+		glDrawArrays(GL_TRIANGLE_FAN, 0, varray.size()/2);
+#else
+		glDisable(GL_TEXTURE_2D);
+		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
 		glVertexPointer(2, GL_FLOAT, 0, &varray.front());
 		glDrawArrays(GL_TRIANGLE_FAN, 0, varray.size()/2);
 		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 		glEnable(GL_TEXTURE_2D);
+#endif
 	}
 
-#ifndef SDL_VIDEO_OPENGL_ES
-	void coords_to_screen(GLdouble sx, GLdouble sy, GLdouble sz,
-						  GLdouble* dx, GLdouble* dy, GLdouble* dz) {
-		GLdouble model[16], proj[16];
+	void coords_to_screen(GLfloat sx, GLfloat sy, GLfloat sz,
+						  GLfloat* dx, GLfloat* dy, GLfloat* dz) 
+	{
+		GLfloat model[16], proj[16];
 		GLint view[4];
 		
-		glGetDoublev(GL_MODELVIEW_MATRIX, model);
-		glGetDoublev(GL_PROJECTION_MATRIX, proj);
+#if defined(USE_GLES2) && defined(GL_ES_VERSION_2_0)
+		glGetFloatv_1(GL_MODELVIEW_MATRIX, model);
+		glGetFloatv_1(GL_PROJECTION_MATRIX, proj);
+#else
+		glGetFloatv(GL_MODELVIEW_MATRIX, model);
+		glGetFloatv(GL_PROJECTION_MATRIX, proj);
+#endif
 		glGetIntegerv(GL_VIEWPORT, view);
 		
-		gluProject(sx, sy, sz, model, proj, view, dx, dy, dz);
+		gluProjectf(sx, sy, sz, model, proj, view, dx, dy, dz);
 	}
 
 	void push_clip(const SDL_Rect& r)
 	{
-		const bool was_enabled_clip = glIsEnabled(GL_SCISSOR_TEST);
-		bool should_enable_clip = true;
+		glEnable(GL_STENCIL_TEST);
+		glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
+		glStencilMask(0xff);
+		glClear(GL_STENCIL_BUFFER_BIT);
 		
-		GLdouble x, y ,z;
-		coords_to_screen(0, graphics::screen_height(), 0, &x, &y, &z);
-		
-		SDL_Rect s = r;
-		s.x += static_cast<Sint16>(x);
-		s.y -= static_cast<Sint16>(y);
-		
-		if(s.x == 0 && s.y == 0 && s.w == screen_width() && s.h == screen_height()) {
-			should_enable_clip = false;
-		}
-		
-		
-		boost::shared_array<GLint> box(new GLint[4]);
-		
-		if(was_enabled_clip) {
-			glGetIntegerv(GL_SCISSOR_BOX, box.get());
-		} else {
-			box[0] = 0;
-			box[1] = 0;
-			box[2] = screen_width();
-			box[3] = screen_height();
-		}
-		clip_rectangles.push_back(box);
-		
-		if(should_enable_clip) {
-			if(!was_enabled_clip) {
-				glEnable(GL_SCISSOR_TEST);
-			}
-			glScissor(s.x, screen_height() - (s.y + s.h), s.w, s.h);
-		} else if(was_enabled_clip) {
-			glDisable(GL_SCISSOR_TEST);
-		}
+		GLfloat varray[] = {
+			r.x, r.y,
+			r.x+r.w, r.y,
+			r.x, r.y+r.h,
+			r.x+r.w, r.y+r.h
+		};
+		glStencilFunc(GL_NEVER, 0x1, 0xff);
+		glStencilOp(GL_REPLACE, GL_KEEP, GL_KEEP);
+#if defined(USE_GLES2)
+		glColor4f(1.0f,1.0f,1.0f,1.0f);
+		gles2::manager gles2_manager(gles2::get_simple_shader());
+		gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, 0, 0, varray);
+ 		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+#else
+		glColor4ub(255, 255, 255, 255);
+		glDisable(GL_TEXTURE_2D);
+		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+		glVertexPointer(2, GL_FLOAT, 0, varray);
+		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+		glEnable(GL_TEXTURE_2D);
+#endif
+		glStencilMask(0);
+		glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
+		glStencilFunc(GL_EQUAL, 0x1, 0x1);
 	}
 	
 	void pop_clip() {
-		const bool was_enabled_clip = glIsEnabled(GL_SCISSOR_TEST);
-		bool should_enable_clip = false;
-		boost::shared_array<GLint> box;
-		
-		if(!clip_rectangles.empty()) {
-			box = *clip_rectangles.rbegin();
-			clip_rectangles.pop_back();
-			
-			if(box[0] != 0 || box[1] != 0 || box[2] != screen_width() || box[3] != screen_height()) {
-				should_enable_clip = true;
-			}
-		}
-		
-		if(should_enable_clip) {
-			if(!was_enabled_clip) {
-				glEnable(GL_SCISSOR_TEST);
-			}
-			glScissor(box[0], box[1], box[2], box[3]);
-		} else if(was_enabled_clip) {
-			glDisable(GL_SCISSOR_TEST);
-		}
+		glDisable(GL_STENCIL_TEST);
 	}
-#endif
 	
 	namespace {
 		int zoom_level = 1;
@@ -861,41 +967,5 @@ bool blit_queue::merge(const blit_queue& q, short begin, short end)
 	{
 		zoom_level = 1;
 	}
-	
-	const SDL_Color& color_black()
-	{
-		static SDL_Color res = {0,0,0,0};
-		return res;
-	}
-	
-	const SDL_Color& color_white()
-	{
-		static SDL_Color res = {0xFF,0xFF,0xFF,0};
-		return res;
-	}
-	
-	const SDL_Color& color_red()
-	{
-		static SDL_Color res = {0xFF,0,0,0};
-		return res;
-	}
-	
-	const SDL_Color& color_green()
-	{
-		static SDL_Color res = {0,0xFF,0,0};
-		return res;
-	}
-	
-	const SDL_Color& color_blue()
-	{
-		static SDL_Color res = {0,0,0xFF,0};
-		return res;
-	}
-	
-	const SDL_Color& color_yellow()
-	{
-		static SDL_Color res = {0xFF,0xFF,0,0};
-		return res;
-	}
-	
+
 }
