@@ -1,9 +1,4 @@
-#ifdef TARGET_PANDORA
-#include <GLES/gl.h>
-#else
-#include <GL/gl.h>
-#endif
-
+#include "graphics.hpp"
 #include "rectangle_rotator.hpp"
 #include <math.h>
 #ifndef M_PI
@@ -11,7 +6,7 @@
 #endif
 #include "unit_test.hpp"
 
-void rotate_rect(GLshort center_x, GLshort center_y, GLshort rotation, GLshort* rect_vertexes){
+void rotate_rect(GLshort center_x, GLshort center_y, float rotation, GLshort* rect_vertexes){
 
 	point p;
 	
