@@ -2,7 +2,7 @@
 #define SETTINGS_DIALOG_HPP_INCLUDED
 
 #include "gui_section.hpp"
-#include <SDL.h>
+#include "graphics.hpp"
 
 class settings_dialog
 {
@@ -11,14 +11,12 @@ public:
 	~settings_dialog ();
 	
 	bool handle_event (const SDL_Event& event); //returns whether the game should be paused
-	void draw () const;
+	void draw (bool in_speech_dialog) const;
 	void reset ();
 	
 private:
 	bool show_window_;
 	bool menu_button_state_;
-	const const_gui_section_ptr menu_button_normal_;
-	const const_gui_section_ptr menu_button_down_;
 };
 
 #endif
