@@ -2,16 +2,15 @@
 #include <map>
 #include <vector>
 
-#include <pthread.h>
-
+#include <boost/bind.hpp>
 #include <boost/shared_ptr.hpp>
 
+#include "foreach.hpp"
+#include "module.hpp"
 #include "preferences.hpp"
+#include "thread.hpp"
 #include "filesystem.hpp"
-#include "SDL.h"
-#if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE
-#include "SDL_mixer.h"
-#endif
+#include "graphics.hpp"
 
 #include "sound.hpp"
 
@@ -19,12 +18,23 @@
 #include "iphone_sound.h"
 #endif
 
+#include "variant_utils.hpp"
+
 namespace sound {
 
 namespace {
 
+struct MusicInfo {
+	MusicInfo() : volume(1.0) {}
+	float volume;
+};
+
+std::map<std::string, MusicInfo> music_index;
+
 float sfx_volume = 1.0;
-float recorded_music_volume = 1.0;
+float user_music_volume = 1.0;
+float engine_music_volume = 1.0;
+float track_music_volume = 1.0;
 const int SampleRate = 44100;
 // number of allocated channels, 
 #if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE
@@ -79,9 +89,10 @@ struct sound_playing {
 	std::string file;
 	const void* object;
 	int	loops;		//not strictly boolean.  -1=true, 0=false
+	float volume;
 };
 
-std::vector<sound_playing> channels_to_sounds_playing;
+std::vector<sound_playing> channels_to_sounds_playing, queued_sounds;
 
 void on_sound_finished(int channel)
 {
@@ -121,7 +132,11 @@ class sound
 		SDL_AudioSpec spec; /* the audio format of the .wav file */
 		SDL_AudioCVT cvt; /* used to convert .wav to output format when formats differ */
 		Uint8 *tmp_buffer;
-		if (SDL_LoadWAV(file.c_str(), &spec, &tmp_buffer, &length) == NULL)
+#if defined(__ANDROID__)
+		if(SDL_LoadWAV_RW(sys::read_sdl_rw_from_asset(module::map_file(file).c_str(), 1, &spec, &tmp_buffer, &length) == NULL)
+#else
+		if (SDL_LoadWAV(module::map_file(file).c_str(), &spec, &tmp_buffer, &length) == NULL)
+#endif
 		{
 			std::cerr << "Could not load sound: " << file << "\n";
 			return; //should maybe die
@@ -261,8 +276,39 @@ typedef std::map<std::string, sound> cache_map;
 #endif
 cache_map cache;
 
+cache_map threaded_cache;
+threading::mutex cache_mutex;
+
+
 bool sound_init = false;
 
+void thread_load(const std::string& file)
+{
+#if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE
+#if defined(__ANDROID__)
+	Mix_Chunk* chunk = Mix_LoadWAV_RW(sys::read_sdl_rw_from_asset(module::map_file("sounds/" + file).c_str()),1);
+#else
+	Mix_Chunk* chunk = Mix_LoadWAV(module::map_file("sounds/" + file).c_str());
+#endif
+	{
+		threading::lock l(cache_mutex);
+		threaded_cache[file] = chunk;
+	}
+
+#else
+	std::string wav_file = file;
+	wav_file.replace(wav_file.length()-3, wav_file.length(), "wav");
+	sound s("sounds_wav/" + wav_file);
+
+	{
+		threading::lock l(cache_mutex);
+		threaded_cache[file] = s;
+	}
+#endif
+}
+
+std::map<std::string, boost::shared_ptr<threading::thread> > loading_threads;
+
 }
 
 manager::manager()
@@ -316,7 +362,7 @@ manager::manager()
     }
 #endif
 
-	set_music_volume(recorded_music_volume);
+	set_music_volume(user_music_volume);
 }
 
 manager::~manager()
@@ -325,6 +371,8 @@ manager::~manager()
 		return;
 	}
 
+	loading_threads.clear();
+
 #if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE
 	Mix_HookMusicFinished(NULL);
 	next_music().clear();
@@ -334,6 +382,14 @@ manager::~manager()
 #endif
 }
 
+void init_music(variant node)
+{
+	foreach(variant music_node, node["music"].as_list()) {
+		const std::string name = music_node["name"].as_string();
+		music_index[name].volume = music_node["volume"].as_decimal(decimal(1.0)).as_float();
+	}
+}
+
 bool ok() { return sound_ok; }
 bool muted() { return mute_; }
 
@@ -341,40 +397,59 @@ void mute (bool flag)
 {
 	mute_ = flag;
 #if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE
-	Mix_VolumeMusic(MIX_MAX_VOLUME*(!flag));
+	Mix_VolumeMusic(user_music_volume*track_music_volume*engine_music_volume*MIX_MAX_VOLUME*(!flag));
 #endif
 }
 
+void preload(const std::string& file)
+{
+	if(!sound_ok) {
+		return;
+	}
+
+	if(loading_threads.count(file)) {
+		return;
+	}
+
+#if defined(__ANDROID__) && SDL_VERSION_ATLEAST(1, 3, 0)
+	boost::shared_ptr<threading::thread> t(new threading::thread("sounds", boost::bind(thread_load, file)));
+#else
+	boost::shared_ptr<threading::thread> t(new threading::thread(boost::bind(thread_load, file)));
+#endif
+	loading_threads[file] = t;
+}
+
 namespace {
 
-int play_internal(const std::string& file, int loops, const void* object)
+int play_internal(const std::string& file, int loops, const void* object, float volume)
 {
 	if(!sound_ok) {
 		return -1;
 	}
 
+	if(!cache.count(file)) {
+		preload(file);
+		queued_sounds.push_back(sound_playing());
+		queued_sounds.back().file = file;
+		queued_sounds.back().loops = loops;
+		queued_sounds.back().object = object;
+		queued_sounds.back().volume = volume;
+		
+		return -1;
+	}
+
 #if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE
-	Mix_Chunk*& chunk = cache[file];
+	Mix_Chunk* chunk = cache[file];
 	if(chunk == NULL) {
-		chunk = Mix_LoadWAV(("sounds/" + file).c_str());
-		if(chunk == NULL) {
-			return -1;
-		}
+		return -1;
 	}
 
 	int result = Mix_PlayChannel(-1, chunk, loops);
 
 #else
 	sound& s = cache[file];
-	if (s == NULL)
-	{
-		std::string wav_file = file;
-		wav_file.replace(wav_file.length()-3, wav_file.length(), "wav");
-		s = sound("sounds_wav/" + wav_file);
-		if (s == NULL)
-		{
-			return -1;
-		}
+	if(s == NULL) {
+		return -1;
 	}
 	
 	int result = sdl_play_sound(&s, loops);
@@ -386,7 +461,7 @@ int play_internal(const std::string& file, int loops, const void* object)
 			channels_to_sounds_playing.resize(result + 1);
 		}
 #if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE
-		Mix_Volume(result, sfx_volume*MIX_MAX_VOLUME); //start sound at full volume
+		Mix_Volume(result, volume*sfx_volume*MIX_MAX_VOLUME); //start sound at full volume
 #endif
 
 		channels_to_sounds_playing[result].file = file;
@@ -399,13 +474,36 @@ int play_internal(const std::string& file, int loops, const void* object)
 
 }
 
-void play(const std::string& file, const void* object)
+void process()
+{
+	bool has_items = false;
+	{
+		threading::lock l(cache_mutex);
+		for(cache_map::const_iterator i = threaded_cache.begin(); i != threaded_cache.end(); ++i) {
+			cache.insert(*i);
+			has_items = true;
+			loading_threads.erase(i->first);
+		}
+
+		threaded_cache.clear();
+	}
+
+	if(has_items) {
+		std::vector<sound_playing> sounds;
+		sounds.swap(queued_sounds);
+		foreach(const sound_playing& sfx, sounds) {
+			play_internal(sfx.file, sfx.loops, sfx.object, sfx.volume);
+		}
+	}
+}
+
+void play(const std::string& file, const void* object, float volume)
 {
 	if(preferences::no_sound() || mute_) {
 		return;
 	}
 
-	play_internal(file, 0, object);
+	play_internal(file, 0, object, volume);
 }
 
 void stop_sound(const std::string& file, const void* object)
@@ -421,6 +519,14 @@ void stop_sound(const std::string& file, const void* object)
 #endif
 		}
 	}
+
+	for(int n = 0; n != queued_sounds.size(); ++n) {
+		if(queued_sounds[n].object == object &&
+		   queued_sounds[n].file == file) {
+			queued_sounds.erase(queued_sounds.begin() + n);
+			--n;
+		}
+	}
 }
 	
 void stop_looped_sounds(const void* object)
@@ -442,16 +548,30 @@ void stop_looped_sounds(const void* object)
 			channels_to_sounds_playing[n].object = NULL;
 		}
 	}
+
+	for(int n = 0; n != queued_sounds.size(); ++n) {
+		if((object == NULL && queued_sounds[n].object != NULL
+		   || queued_sounds[n].object == object) &&
+		   (queued_sounds[n].loops != 0)) {
+			queued_sounds.erase(queued_sounds.begin() + n);
+			--n;
+		} else if(queued_sounds[n].object == object) {
+			//this sound is a looped sound, but make sure it keeps going
+			//until it ends, since this function signals that the associated
+			//object is going away.
+			queued_sounds[n].object = NULL;
+		}
+	}
 }
 	
-int play_looped(const std::string& file, const void* object)
+int play_looped(const std::string& file, const void* object, float volume)
 {
 	if(preferences::no_sound() || mute_) {
 		return -1;
 	}
 
 
-	const int result = play_internal(file, -1, object);
+	const int result = play_internal(file, -1, object, volume);
 	std::cerr << "PLAY: " << object << " " << file << " -> " << result << "\n";
 	return result;
 }
@@ -489,37 +609,53 @@ void set_sound_volume(float volume)
 
 float get_music_volume()
 {
+	return user_music_volume;
+}
+
+namespace {
+void update_music_volume()
+{
+	if(sound_init) {
 #if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE
-	return (float)Mix_VolumeMusic(-1)/MIX_MAX_VOLUME;
+		Mix_VolumeMusic(track_music_volume*user_music_volume*engine_music_volume*MIX_MAX_VOLUME);
 #else
-	return iphone_get_music_volume();
+		iphone_set_music_volume(track_music_volume*user_music_volume*engine_music_volume);
 #endif
+	}
+}
 }
 
 void set_music_volume(float volume)
 {
-	recorded_music_volume = volume;
-
-	if(sound_init) {
-#if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE
-		Mix_VolumeMusic(volume*MIX_MAX_VOLUME);
-#else
-		iphone_set_music_volume(volume);
-#endif
-	}
+	user_music_volume = volume;
+	update_music_volume();
 }
 
-void cancel_looped(int handle)
+void set_engine_music_volume(float volume)
 {
-	if(handle >= 0 && handle < channels_to_sounds_playing.size()) {
-		channels_to_sounds_playing[handle].object = NULL;
-	}
+	engine_music_volume = volume;
+	update_music_volume();
+}
 
-#if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE
-	Mix_HaltChannel(handle);
-#else
-	sdl_stop_channel(handle);
-#endif
+float get_engine_music_volume()
+{
+	return engine_music_volume;
+}
+
+namespace {
+std::map<std::string,std::string>& get_music_paths() {
+	static std::map<std::string,std::string> res;
+	return res;
+}
+}
+
+void load_music_paths() {
+    #if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
+        #define MUSIC_DIR_NAME "music_aac/"
+    #else
+        #define MUSIC_DIR_NAME "music/"
+    #endif
+	module::get_unique_filenames_under_dir(MUSIC_DIR_NAME, &get_music_paths());
 }
 
 void play_music(const std::string& file)
@@ -532,6 +668,23 @@ void play_music(const std::string& file)
 		return;
 	}
 
+    std::string song_file = file;
+
+#if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
+	song_file.replace(song_file.length()-3, song_file.length(), "m4a");
+#endif
+
+	if(get_music_paths().empty()) {
+		load_music_paths();
+	}
+
+	std::map<std::string, std::string>::const_iterator itor = module::find(get_music_paths(), song_file);
+	if(itor == get_music_paths().end()) {
+		std::cerr << "FILE NOT FOUND: " << song_file << std::endl;
+		return;
+	}
+	const std::string& path = itor->second;
+
 #if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE
 	if(current_mix_music) {
 		next_music() = file;
@@ -539,16 +692,19 @@ void play_music(const std::string& file)
 		return;
 	}
 
-	if(file.empty()) {
+	current_music_name() = file;
+#if defined(__ANDROID__)
+	current_mix_music = Mix_LoadMUS_RW(sys::read_sdl_rw_from_asset(path.c_str()));
+#else
+	current_mix_music = Mix_LoadMUS(path.c_str());
+#endif
+	if(!current_mix_music) {
+		std::cerr << "Mix_LoadMUS ERROR loading " << path << ": " << Mix_GetError() << "\n";
 		return;
 	}
 
-	current_music_name() = file;
-	current_mix_music = Mix_LoadMUS(("music/" + file).c_str());
-	if(!current_mix_music) {
-		std::cerr << "Mix_LoadMUS ERROR loading " << file << ": " << Mix_GetError() << "\n";
-		return;
-	}
+	track_music_volume = music_index[file].volume;
+	update_music_volume();
 
 	Mix_FadeInMusic(current_mix_music, -1, 500);
 #else
@@ -564,10 +720,10 @@ void play_music(const std::string& file)
 	}
 	
 	current_music_name() = file;
-	std::string aac_file = file;
-	aac_file.replace(aac_file.length()-3, aac_file.length(), "m4a");
-	if (!sys::file_exists("music_aac/" + aac_file)) return;
-	iphone_play_music(("music_aac/" + aac_file).c_str(), -1);
+	if (!sys::file_exists(path)) return;
+	track_music_volume = music_index[file].volume;
+	update_music_volume();
+	iphone_play_music((path).c_str(), -1);
 	iphone_fade_in_music(350);
 	playing_music = true;
 #endif
@@ -586,10 +742,27 @@ void play_music_interrupt(const std::string& file)
 	
 	next_music() = current_music_name();
 	
+    std::string song_file = file;
+    
+#if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
+	song_file.replace(song_file.length()-3, song_file.length(), "m4a");
+#endif
+
 	current_music_name() = file;
 
-#if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE
 
+	if(get_music_paths().empty()) {
+		load_music_paths();
+	}
+
+	std::map<std::string, std::string>::const_iterator itor = module::find(get_music_paths(), song_file);
+	if(itor == get_music_paths().end()) {
+		std::cerr << "FILE NOT FOUND: " << song_file << std::endl;
+		return;
+	}
+	const std::string& path = itor->second;
+
+#if !TARGET_IPHONE_SIMULATOR && !TARGET_OS_IPHONE
 	//note that calling HaltMusic will result in on_music_finished being
 	//called, which releases the current_music pointer.
 	Mix_HaltMusic();
@@ -597,7 +770,12 @@ void play_music_interrupt(const std::string& file)
 		return;
 	}
 
-	current_mix_music = Mix_LoadMUS(("music/" + file).c_str());
+#if defined(__ANDROID__)
+	current_mix_music = Mix_LoadMUS_RW(sys::read_sdl_rw_from_asset(path.c_str()));
+#else
+	current_mix_music = Mix_LoadMUS(path.c_str());
+#endif
+
 	if(!current_mix_music) {
 		std::cerr << "Mix_LoadMUS ERROR loading " << file << ": " << Mix_GetError() << "\n";
 		return;
@@ -605,9 +783,7 @@ void play_music_interrupt(const std::string& file)
 
 	Mix_PlayMusic(current_mix_music, 1);
 #else
-	std::string aac_file = file;
-	aac_file.replace(aac_file.length()-3, aac_file.length(), "m4a");
-	iphone_play_music(("music_aac/" + aac_file).c_str(), 0);
+	iphone_play_music((path).c_str(), 0);
 	playing_music = true;
 #endif
 }
