@@ -2,6 +2,8 @@
 #include <stack>
 #include <limits.h>
 
+#include <boost/tokenizer.hpp>
+
 #include "controls.hpp"
 #include "color_utils.hpp"
 #include "draw_scene.hpp"
@@ -14,15 +16,17 @@
 #include "preferences.hpp"
 #include "raster.hpp"
 #include "speech_dialog.hpp"
+#include "iphone_controls.hpp"
+#include "asserts.hpp"
 
 namespace {
 #if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
 	const int OptionHeight = 70;
-	const int OptionWidth = 200;
+	const int OptionMinWidth = 200;
 	const int OptionXPad = 20;
 #else
 	const int OptionHeight = 50;
-	const int OptionWidth = 150;
+	const int OptionMinWidth = 150;
 	const int OptionXPad = 10;
 #endif
 	const int OptionsBorder = 20; // size of the border around the options window
@@ -32,12 +36,18 @@ namespace {
 
 speech_dialog::speech_dialog()
   : cycle_(0), left_side_speaking_(false), horizontal_position_(0), text_char_(0), option_selected_(0),
+#if defined(__ANDROID__)
+    joystick_button_pressed_(false),
+    joystick_up_pressed_(false),
+    joystick_down_pressed_(false),
+#else // !defined(__ANDROID__)
     joystick_button_pressed_(true),
     joystick_up_pressed_(true),
     joystick_down_pressed_(true),
+#endif // defined(__ANDROID__)
 	expiration_(-1)
 {
-#if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
+#if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE || defined(__ANDROID__)
 	option_selected_ = -1;
 #endif
 }
@@ -48,16 +58,11 @@ speech_dialog::~speech_dialog()
 
 bool speech_dialog::handle_mouse_move(int x, int y)
 {
-	if(preferences::screen_rotated()) {
-		x = preferences::actual_screen_width() - x;
-		std::swap(x, y);
-		x *= 2;
-		y *= 2;
-	}
+	translate_mouse_coords(&x, &y);
 	rect box(
-		preferences::virtual_screen_width() - OptionsX - OptionWidth - OptionsBorder,
+		preferences::virtual_screen_width() - OptionsX - option_width_ - OptionsBorder,
 		preferences::virtual_screen_height() - OptionsY - OptionHeight*options_.size() - OptionsBorder,
-		OptionWidth + OptionsBorder*2, OptionHeight*options_.size() + OptionsBorder*2
+		option_width_ + OptionsBorder*2, OptionHeight*options_.size() + OptionsBorder*2
 	);
 	//std::cerr << "Options box: " << box << " : " << x << " : " << y << "\n";
 	if (point_in_rect(point(x, y), box))
@@ -103,7 +108,9 @@ bool speech_dialog::key_press(const SDL_Event& event)
 			}
 		}
 	
-#if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
+#if defined(__ANDROID__)
+		// XXX: todo
+#elif TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE || TARGET_BLACKBERRY
 		if(event.type == SDL_MOUSEBUTTONDOWN)
 		{
 			last_mouse = event.button.which;
@@ -129,12 +136,18 @@ bool speech_dialog::key_press(const SDL_Event& event)
 		return false; // only keydown and mousebuttondown should be handled by the rest of the function
 	}
 
+	return scroll_text();
+}
+
+bool speech_dialog::scroll_text()
+{
 	if(text_char_ < num_chars()) {
 		text_char_ = num_chars();
 		return false;
 	}
 
 	if(text_.size() > 2) {
+		markup_.erase(markup_.begin());
 		text_.erase(text_.begin());
 		text_char_ = text_.front().size();
 		return false;
@@ -175,6 +188,7 @@ bool speech_dialog::process()
 		}
 	}
 
+#if !defined(__ANDROID__)
 	if(expiration_ <= 0) {
 		joystick::update();
 
@@ -185,19 +199,32 @@ bool speech_dialog::process()
 		if(!joystick_down_pressed_ && joystick::down()) {
 			move_down();
 		}
-
-		if(!joystick_button_pressed_ && (joystick::button(0) || joystick::button(10))) {
-			return true;
-		}
 	}
 
 	joystick_up_pressed_ = joystick::up();
 	joystick_down_pressed_ = joystick::down();
-	joystick_button_pressed_ = joystick::button(0) || joystick::button(1);
+#endif 
 
 	return cycle_ == expiration_;
 }
 
+bool speech_dialog::detect_joystick_press()
+{
+#if defined(__ANDROID__)
+    return false;
+#else //!defined(__ANDROID__)
+	const bool new_press = joystick::button(0) || joystick::button(1);
+	const bool is_pressed = new_press && !joystick_button_pressed_;
+	joystick_button_pressed_ = new_press;
+
+	if(is_pressed) {
+		return scroll_text();
+	} else {
+		return false;
+	}
+#endif
+}
+
 void speech_dialog::draw() const
 {
 	static const const_gui_section_ptr top_corner = gui_section::get("speech_dialog_top_corner");
@@ -284,10 +311,31 @@ void speech_dialog::draw() const
 	for(int n = 0; n < 2 && n < text_.size() && nchars > 0; ++n) {
 		std::string str(text_[n].begin(), text_[n].begin() +
 		                  std::min<int>(nchars, text_[n].size()));
-		//currently the color of speech is hard coded.
-		glColor4ub(255, 187, 10, 255);
 		//move the first line slightly up so that accents don't mess up centering
-		rect area = font->draw(text_left_align[n], ypos - 2, str);
+		rect area = font->dimensions(str);
+		area = rect(text_left_align[n], ypos - 2, area.w(), area.h());
+
+		//draw the font by chunks of markup.
+		int xadj = 0;
+		const std::vector<TextMarkup>& markup = markup_[n];
+		for(int m = 0; m != markup.size(); ++m) {
+			const int begin_index = markup[m].begin;
+			const int end_index = std::min<int>(str.size(), m+1 == markup.size() ? str.size() : markup[m+1].begin);
+			if(begin_index >= end_index) {
+				continue;
+			}
+
+			if(markup[m].color) {
+				markup[m].color->set_as_current_color();
+			} else {
+				//default color of fonts.
+				glColor4ub(255, 187, 10, 255);
+			}
+
+			const rect r = font->draw(text_left_align[n] + xadj, ypos - 2, std::string(str.begin() + begin_index, str.begin() + end_index));
+			xadj += r.w();
+		}
+
 		glColor4f(1.0, 1.0, 1.0, 1.0);
 		//add some space between the lines
 		ypos = area.y2() + 4;
@@ -303,9 +351,9 @@ void speech_dialog::draw() const
 	if(text_char_ == num_chars() && options_.empty() == false) {
 		//const_gui_section_ptr options_panel = gui_section::get("speech_portrait_pane");
 		const_framed_gui_element_ptr options_panel = framed_gui_element::get("regular_window");
-		int xpos = graphics::screen_width() - OptionsX - OptionWidth - OptionsBorder*2;
+		int xpos = graphics::screen_width() - OptionsX - option_width_ - OptionsBorder*2;
 		int ypos = graphics::screen_height() - OptionsY - OptionHeight*options_.size() - OptionsBorder*2;
-		options_panel->blit(xpos, ypos, OptionsBorder*2 + OptionWidth, OptionsBorder*2 + OptionHeight*options_.size(), true);
+		options_panel->blit(xpos, ypos, OptionsBorder*2 + option_width_, OptionsBorder*2 + OptionHeight*options_.size(), true);
 
 		xpos += OptionsBorder + OptionXPad;
 		ypos += OptionsBorder;
@@ -315,7 +363,7 @@ void speech_dialog::draw() const
 		foreach(const std::string& option, options_) {
 #if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
 			if(index == option_selected_) {
-				graphics::draw_rect(rect(xpos-OptionXPad, ypos, OptionWidth, OptionHeight), graphics::color(0xC74545FF));
+				graphics::draw_rect(rect(xpos-OptionXPad, ypos, option_width_, OptionHeight), graphics::color(0xC74545FF));
 				glColor4ub(255, 187, 10, 255); //reset color to what it was, since draw_rect changes it
 			}
 #endif
@@ -338,7 +386,7 @@ void speech_dialog::draw() const
 
 void speech_dialog::set_speaker_and_flip_side(const_entity_ptr e)
 {
-	std::cerr << "set speaker\n";
+	//std::cerr << "set speaker\n";
 	left_side_speaking_ = !left_side_speaking_;
 	set_speaker(e, left_side_speaking_);
 }
@@ -361,18 +409,65 @@ void speech_dialog::set_side(bool left_side)
 
 void speech_dialog::set_text(const std::vector<std::string>& text)
 {
-	text_ = text;
+	text_.clear();
+	markup_.clear();
+	for (int i = 0; i < text.size(); i++) {
+		typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
+		//split text[i] at newline characters, add each line separately
+		tokenizer lines(text[i], boost::char_separator<char> ("\n"));
+		for (tokenizer::iterator iter = lines.begin(); iter != lines.end(); ++iter) {
+			std::string txt = *iter;
+			std::vector<TextMarkup> markup;
+			TextMarkup m;
+			m.begin = 0;
+			markup.push_back(m);
+			static const std::string BeginEmStr = "<em>";
+			static const std::string EndEmStr = "</em>";
+			std::string::iterator begin_em = std::search(txt.begin(), txt.end(), BeginEmStr.begin(), BeginEmStr.end());
+			while(begin_em != txt.end()) {
+				const int begin_index = begin_em - txt.begin();
+				txt.erase(begin_em, begin_em + BeginEmStr.size());
+
+				TextMarkup m;
+				m.begin = begin_index;
+				m.color.reset(new graphics::color(255, 255, 255, 255));
+				markup.push_back(m);
+
+				std::string::iterator end_em = std::search(txt.begin(), txt.end(), EndEmStr.begin(), EndEmStr.end());
+				if(end_em != txt.end()) {
+					const int end_index = end_em - txt.begin();
+					txt.erase(end_em, end_em + EndEmStr.size());
+
+					TextMarkup m;
+					m.begin = end_index;
+					markup.push_back(m);
+				}
+
+				begin_em = std::search(txt.begin(), txt.end(), BeginEmStr.begin(), BeginEmStr.end());
+			}
+
+			text_.push_back(txt);
+			markup_.push_back(markup);
+		}
+	}
 	text_char_ = 0;
 }
 
 void speech_dialog::set_options(const std::vector<std::string>& options)
 {
 	options_ = options;
-#if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
+#if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE  || defined(__ANDROID__)
 	option_selected_ = -1;
 #else
 	option_selected_ = 0;
 #endif
+	option_width_ = OptionMinWidth;
+	const_graphical_font_ptr font = graphical_font::get("default");
+	foreach(const std::string& option, options_) {
+		rect area = font->dimensions(option);
+		if (area.w()+OptionXPad*2 > option_width_)
+			option_width_ = area.w()+OptionXPad*2;
+	}
 }
 
 int speech_dialog::num_chars() const
