@@ -1,11 +1,14 @@
 #ifndef SPEECH_DIALOG_HPP_INCLUDED
 #define SPEECH_DIALOG_HPP_INCLUDED
 
-#include "SDL.h"
+#include <boost/shared_ptr.hpp>
+
+#include "graphics.hpp"
 
 #include <string>
 #include <vector>
 
+#include "color_utils.hpp"
 #include "entity.hpp"
 #include "gui_section.hpp"
 
@@ -16,6 +19,7 @@ public:
 	speech_dialog();
 	~speech_dialog();
 
+	bool detect_joystick_press();
 	bool key_press(const SDL_Event& e);
 	bool process();
 	void draw() const;
@@ -33,11 +37,20 @@ private:
 	void move_up();
 	void move_down();
 
+	bool scroll_text();
+
 	int cycle_;
 	const_entity_ptr left_, right_;
 	bool left_side_speaking_;
 	int horizontal_position_;
 
+	struct TextMarkup {
+		int begin;
+		boost::shared_ptr<graphics::color> color;
+	};
+
+	std::vector<std::vector<TextMarkup> > markup_;
+
 	std::vector<std::string> text_;
 	int text_char_;
 
