@@ -1,9 +1,6 @@
 #include <iostream>
 #include <map>
-#include <SDL.h>
-#ifndef SDL_VIDEO_OPENGL_ES
-#include <GL/glew.h>
-#endif
+#include "graphics.hpp"
 
 #include "asserts.hpp"
 #include "concurrent_cache.hpp"
@@ -13,6 +10,7 @@
 #include "formula_callable.hpp"
 #include "formula_function.hpp"
 #include "hi_res_timer.hpp"
+#include "module.hpp"
 #include "surface.hpp"
 #include "surface_cache.hpp"
 #include "surface_formula.hpp"
@@ -105,7 +103,7 @@ void run_formula(surface surf, const std::string& algo)
 
 	const int ticks = SDL_GetTicks();
 	surface_formula_symbol_table table(surf);
-	game_logic::formula f(algo, &table);
+	game_logic::formula f(variant(algo), &table);
 	bool locked = false;
 	if(SDL_MUSTLOCK(surf.get())) {
 		const int res = SDL_LockSurface(surf.get());
@@ -194,7 +192,7 @@ GLuint compile_shader(const std::string& shader_file, GLuint type)
 #ifndef SDL_VIDEO_OPENGL_ES
 	id = glCreateShader(type);
 
-	const std::string file_data = sys::read_file("data/shaders/" + shader_file);
+	const std::string file_data = sys::read_file(module::map_file("data/shaders/" + shader_file));
 
 	const char* file_str = file_data.c_str();
 	glShaderSource(id, 1, &file_str, NULL);
@@ -214,7 +212,7 @@ GLuint get_gl_shader(const std::vector<std::string>& vertex_shader_file,
                      const std::vector<std::string>& fragment_shader_file)
 {
 #ifndef SDL_VIDEO_OPENGL_ES
-	if(vertex_shader_file.empty() || fragment_shader_file.empty()) {
+	if(vertex_shader_file.empty() && fragment_shader_file.empty()) {
 		return 0;
 	}
 
