@@ -4,12 +4,7 @@
 #include <string>
 #include <vector>
 
-#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA)
-#include <GLES/gl.h>
-#else
-#include <GL/gl.h>
-#endif
-
+#include "graphics.hpp"
 #include "surface.hpp"
 
 graphics::surface get_surface_formula(graphics::surface input, const std::string& algo);
