@@ -10,14 +10,14 @@
 #include "formula.hpp"
 #include "formula_callable.hpp"
 #include "formula_function.hpp"
+#include "json_parser.hpp"
 #include "multi_tile_pattern.hpp"
 #include "point_map.hpp"
 #include "random.hpp"
 #include "string_utils.hpp"
+#include "thread.hpp"
 #include "tile_map.hpp"
-#include "wml_node.hpp"
-#include "wml_parser.hpp"
-#include "wml_utils.hpp"
+#include "variant_utils.hpp"
 
 namespace {
 
@@ -45,21 +45,22 @@ bool match_regex(boost::array<char, 4> str, const boost::regex* re) {
 static const int TileSize = 32;
 
 struct is_whitespace {
-	bool operator()(char c) const { return isspace(c); }
+	bool operator()(char c) const { return util::c_isspace(c); }
 };
 
 }
 
 struct tile_pattern {
-	explicit tile_pattern(wml::const_node_ptr node)
-	  : tile(new level_object(node)), reverse(node->attr("reverse").str() != "no"),
-	    empty(node->attr("empty").str() == "yes"),
-		filter_formula(game_logic::formula::create_optional_formula(node->attr("filter")))
+	explicit tile_pattern(variant node, const std::string& id)
+	  : tile_id(id),
+	    tile(new level_object(node, id.c_str())), reverse(node["reverse"].as_bool(true)),
+	    empty(node["empty"].as_bool(false)),
+		filter_formula(game_logic::formula::create_optional_formula(node["filter"]))
 	{
 		variations.push_back(tile);
 
-		pattern_str = node->attr("pattern");
-		std::string pattern_str = node->attr("pattern");
+		pattern_str = node["pattern"].as_string();
+		std::string pattern_str = node["pattern"].as_string();
 		pattern_str.erase(std::remove_if(pattern_str.begin(), pattern_str.end(), is_whitespace()), pattern_str.end());
 
 		static std::vector<std::string> patterns;
@@ -71,7 +72,7 @@ struct tile_pattern {
 		//the main pattern is always the very middle one.
 		int main_tile = patterns.size()/2;
 
-		int width = wml::get_int(node, "pattern_width", sqrt(static_cast<float>(patterns.size())));
+		int width = node["pattern_width"].as_int(sqrt(static_cast<float>(patterns.size())));
 		assert(width != 0);
 		int height = patterns.size()/width;
 
@@ -79,7 +80,7 @@ struct tile_pattern {
 		int left = -width/2;
 
 		//pattern with size 12 is a special case
-		if(patterns.size() == 12 && !node->has_attr("pattern_width")) {
+		if(patterns.size() == 12 && !node.has_key("pattern_width")) {
 			width = 3;
 			height = 4;
 			top = -1;
@@ -99,24 +100,22 @@ struct tile_pattern {
 			surrounding_tiles.push_back(surrounding_tile(x, y, patterns[n]));
 		}
 
-		wml::node::const_child_iterator i1 = node->begin_child("variation");
-		wml::node::const_child_iterator i2 = node->end_child("variation");
-		while(i1 != i2) {
-			variations.push_back(level_object_ptr(new level_object(i1->second)));
-			++i1;
+		foreach(variant var, node["variation"].as_list()) {
+			variations.push_back(level_object_ptr(new level_object(var, tile_id.c_str())));
 		}
 
-		i1 = node->begin_child("tile");
-		i2 = node->end_child("tile");
-		while(i1 != i2) {
+		foreach(variant var, node["tile"].as_list()) {
 			added_tile t;
-			t.object = level_object_ptr(new level_object(i1->second));
-			t.zorder = wml::get_int(i1->second, "zorder");
+			level_object_ptr new_object(new level_object(var, tile_id.c_str()));
+
+			t.object = new_object.get();
+			t.zorder = var["zorder"].as_int();
 			added_tiles.push_back(t);
-			++i1;
+			
 		}
 	}
 
+	std::string tile_id;
 	const boost::regex* current_tile_pattern;
 
 	struct surrounding_tile {
@@ -210,12 +209,12 @@ void tile_map::load_all()
 {
 	for(std::map<std::string, std::vector<std::string> >::const_iterator i = files_index.begin(); i != files_index.end(); ++i) {
 		foreach(const std::string& s, i->second) {
-			load(s);
+			load(s, i->first);
 		}
 	}
 }
 
-void tile_map::load(const std::string& fname)
+void tile_map::load(const std::string& fname, const std::string& tile_id)
 {
 	if(files_loaded.count(fname)) {
 		return;
@@ -223,30 +222,35 @@ void tile_map::load(const std::string& fname)
 
 	files_loaded.insert(fname);
 
-	wml::const_node_ptr node = wml::parse_wml_from_file("data/tiles/" + fname);
+	variant node = json::parse_from_file("data/tiles/" + fname);
 
-	palette_scope palette_setter(node->attr("palettes"));
+	palette_scope palette_setter(parse_variant_list_or_csv_string(node["palettes"]));
 
-	wml::node::const_child_iterator p1 = node->begin_child("tile_pattern");
-	wml::node::const_child_iterator p2 = node->end_child("tile_pattern");
-	for(; p1 != p2; ++p1) {
-		const wml::const_node_ptr& p = p1->second;
-		patterns.push_back(tile_pattern(p));
+	foreach(variant pattern, node["tile_pattern"].as_list()) {
+		patterns.push_back(tile_pattern(pattern, tile_id));
 	}
 
-	const int start = multi_tile_pattern::get_all().size();
-	multi_tile_pattern::load(node);
-	//std::cerr << "TILE_MAP_LOAD: " << fname << " -> " << (multi_tile_pattern::get_all().size() - start) << ": " << multi_tile_pattern::get_all().size() << "\n";
+	multi_tile_pattern::load(node, tile_id);
 
 	++current_patterns_version;
 }
 
-void tile_map::init(wml::const_node_ptr node)
+const std::vector<std::string>& tile_map::get_files(const std::string& tile_id)
+{
+	if(files_index.count(tile_id)) {
+		return files_index.find(tile_id)->second;
+	} else {
+		static const std::vector<std::string> empty;
+		return empty;
+	}
+}
+
+void tile_map::init(variant node)
 {
 	files_index.clear();
 
-	for(wml::node::const_attr_iterator i = node->begin_attr(); i != node->end_attr(); ++i) {
-		files_index[i->first] = util::split(i->second);
+	foreach(const variant_pair& value, node.as_map()) {
+		files_index[value.first.as_string()] = util::split(value.second.as_string());
 	}
 
 	patterns.clear();
@@ -257,8 +261,55 @@ void tile_map::init(wml::const_node_ptr node)
 	++current_patterns_version;
 }
 
+#ifndef NO_EDITOR
+namespace {
+std::set<tile_map*>& all_tile_maps() {
+	static std::set<tile_map*>* all = new std::set<tile_map*>;
+	return *all;
+}
+threading::mutex& all_tile_maps_mutex() {
+	static threading::mutex* m = new threading::mutex;
+	return *m;
+}
+
+void create_tile_map(tile_map* t) {
+	threading::lock l(all_tile_maps_mutex());
+	all_tile_maps().insert(t);
+}
+void destroy_tile_map(tile_map* t) {
+	threading::lock l(all_tile_maps_mutex());
+	all_tile_maps().erase(t);
+}
+std::set<tile_map*> copy_tile_maps() {
+	threading::lock l(all_tile_maps_mutex());
+	std::set<tile_map*> result = all_tile_maps();
+	return result;
+}
+}
+
+void tile_map::prepare_rebuild_all()
+{
+	const std::set<tile_map*> maps = copy_tile_maps();
+	foreach(tile_map* m, maps) {
+		m->node_ = m->write();
+	}
+}
+
+void tile_map::rebuild_all()
+{
+	const std::set<tile_map*> maps = copy_tile_maps();
+	foreach(tile_map* m, maps) {
+		*m = tile_map(m->node_);
+	}
+}
+#endif
+
 tile_map::tile_map() : xpos_(0), ypos_(0), x_speed_(100), y_speed_(100), zorder_(0), patterns_version_(-1)
 {
+#ifndef NO_EDITOR
+	create_tile_map(this);
+#endif
+
 	//turn off reference counting
 	add_ref();
 
@@ -267,19 +318,27 @@ tile_map::tile_map() : xpos_(0), ypos_(0), x_speed_(100), y_speed_(100), zorder_
 	pattern_index_.back().matching_patterns.push_back(&get_regex_from_pool(""));
 }
 
-tile_map::tile_map(wml::const_node_ptr node)
-  : xpos_(wml::get_int(node, "x")), ypos_(wml::get_int(node, "y")),
-	x_speed_(wml::get_int(node, "x_speed", 100)), y_speed_(wml::get_int(node, "y_speed", 100)),
-    zorder_(wml::get_int(node, "zorder"))
+tile_map::tile_map(variant node)
+  : xpos_(node["x"].as_int()), ypos_(node["y"].as_int()),
+	x_speed_(node["x_speed"].as_int(100)), y_speed_(node["y_speed"].as_int(100)),
+    zorder_(node["zorder"].as_int())
+
+#ifndef NO_EDITOR
+	, node_(node)
+#endif
 {
+#ifndef NO_EDITOR
+	create_tile_map(this);
+#endif
+
 	//turn off reference counting
 	add_ref();
 
-	const std::string& unique_tiles = node->attr("unique_tiles");
+	const std::string& unique_tiles = node["unique_tiles"].as_string_default();
 	foreach(const std::string& tile, util::split(unique_tiles)) {
 		const std::vector<std::string>& files = files_index[tile];
 		foreach(const std::string& file, files) {
-			load(file);
+			load(file, tile);
 		}
 	}
 
@@ -288,7 +347,7 @@ tile_map::tile_map(wml::const_node_ptr node)
 	pattern_index_.back().matching_patterns.push_back(&get_regex_from_pool(""));
 
 	{
-	const std::string& tiles_str = node->attr("tiles");
+	const std::string& tiles_str = node["tiles"].as_string();
 	std::vector<std::string> lines;
 	lines.reserve(std::count(tiles_str.begin(), tiles_str.end(), '\n')+1);
 
@@ -307,11 +366,11 @@ tile_map::tile_map(wml::const_node_ptr node)
 			}
 
 			//We want to copy [ptr,end) to tile_string. First strip the spaces.
-			while(ptr != end && isspace(*ptr)) {
+			while(ptr != end && util::c_isspace(*ptr)) {
 				++ptr;
 			}
 
-			while(end != ptr && isspace(*(end-1))) {
+			while(end != ptr && util::c_isspace(*(end-1))) {
 				--end;
 			}
 
@@ -351,6 +410,23 @@ tile_map::tile_map(wml::const_node_ptr node)
 	}
 }
 
+tile_map::tile_map(const tile_map& o)
+{
+#ifndef NO_EDITOR
+	create_tile_map(this);
+#endif
+
+	add_ref();
+	*this = o;
+}
+
+tile_map::~tile_map()
+{
+#ifndef NO_EDITOR
+	destroy_tile_map(this);
+#endif
+}
+
 void tile_map::build_patterns()
 {
 	std::vector<const boost::regex*> all_regexes;
@@ -455,17 +531,18 @@ const std::vector<const tile_pattern*>& tile_map::get_patterns() const
 	return patterns_;
 }
 
-wml::node_ptr tile_map::write() const
+variant tile_map::write() const
 {
-	wml::node_ptr res(new wml::node("tile_map"));
-	res->set_attr("x", formatter() << xpos_);
-	res->set_attr("y", formatter() << ypos_);
-	res->set_attr("x_speed", formatter() << x_speed_);
-	res->set_attr("y_speed", formatter() << y_speed_);
-	res->set_attr("zorder", formatter() << zorder_);
+	variant_builder res;
+	res.add("x", xpos_);
+	res.add("y", ypos_);
+	res.add("x_speed", x_speed_);
+	res.add("y_speed", y_speed_);
+	res.add("zorder", zorder_);
 
 	std::vector<boost::array<char, 4> > unique_tiles;
 	std::ostringstream tiles;
+	bool first = true;
 	foreach(const std::vector<int>& row, map_) {
 		
 		//cut off any empty cells at the end.
@@ -475,7 +552,11 @@ wml::node_ptr tile_map::write() const
 		}
 
 
-		tiles << "\n";
+		if(!first) {
+			tiles << "\n";
+		}
+
+		first = false;
 		for(int i = 0; i != size; ++i) {
 			if(i) {
 				tiles << ",";
@@ -500,7 +581,7 @@ wml::node_ptr tile_map::write() const
 		unique_str << unique_tiles[n].data();
 	}
 
-	res->set_attr("unique_tiles", unique_str.str());
+	res.add("unique_tiles", unique_str.str());
 
 	std::ostringstream variations;
 	foreach(const std::vector<int>& row, variations_) {
@@ -517,9 +598,9 @@ wml::node_ptr tile_map::write() const
 		}
 	}
 
-	res->set_attr("tiles", tiles.str());
-	res->set_attr("variations", variations.str());
-	return res;
+	res.add("tiles", tiles.str());
+	res.add("variations", variations.str());
+	return res.build();
 }
 
 const char* tile_map::get_tile_from_pixel_pos(int xpos, int ypos) const
@@ -645,8 +726,8 @@ int random_hash(int x, int y, int z, int n)
 
 void tile_map::apply_matching_multi_pattern(int& x, int y,
   const multi_tile_pattern& pattern,
-  point_map<level_object_ptr>& mapping,
-  std::map<point_zorder, level_object_ptr>& different_zorder_mapping) const
+  point_map<level_object*>& mapping,
+  std::map<point_zorder, level_object*>& different_zorder_mapping) const
 {
 
 	if(pattern.chance() < 100 && random_hash(x, y, zorder_, 0)%100 > pattern.chance()) {
@@ -683,7 +764,7 @@ void tile_map::apply_matching_multi_pattern(int& x, int y,
 			for(int ypos = 0; ypos < chosen_pattern.height() && match; ++ypos) {
 				const multi_tile_pattern::tile_info& info = chosen_pattern.tile_at(xpos, ypos);
 				foreach(const multi_tile_pattern::tile_entry& entry, info.tiles) {
-					level_object_ptr ob = entry.tile;
+					level_object* ob = entry.tile.get();
 					if(ob) {
 						if(entry.zorder == INT_MIN || entry.zorder == zorder_) {
 							mapping.insert(point(x + xpos, y + ypos), ob);
@@ -708,8 +789,8 @@ void tile_map::build_tiles(std::vector<level_tile>* tiles, const rect* r) const
 		}
 	}
 
-	point_map<level_object_ptr> multi_pattern_matches;
-	std::map<point_zorder, level_object_ptr> different_zorder_multi_pattern_matches;
+	point_map<level_object*> multi_pattern_matches;
+	std::map<point_zorder, level_object*> different_zorder_multi_pattern_matches;
 
 	//std::cerr << "MULTIPATTERNS: " << multi_patterns_.size() << "/" << multi_tile_pattern::get_all().size() << "\n";
 	foreach(const multi_tile_pattern* p, multi_patterns_) {
@@ -727,8 +808,8 @@ void tile_map::build_tiles(std::vector<level_tile>* tiles, const rect* r) const
 	}
 
 	//add all tiles in different zorders to our own.
-	for(std::map<point_zorder, level_object_ptr>::const_iterator i = different_zorder_multi_pattern_matches.begin(); i != different_zorder_multi_pattern_matches.end(); ++i) {
-		const level_object_ptr& obj = i->second;
+	for(std::map<point_zorder, level_object*>::const_iterator i = different_zorder_multi_pattern_matches.begin(); i != different_zorder_multi_pattern_matches.end(); ++i) {
+		const level_object* obj = i->second;
 		const int x = i->first.first.x;
 		const int y = i->first.first.y;
 
@@ -759,7 +840,7 @@ void tile_map::build_tiles(std::vector<level_tile>* tiles, const rect* r) const
 		for(int x = -1; x <= width; ++x) {
 			const int xpos = xpos_ + x*TileSize;
 
-			const level_object_ptr& obj = multi_pattern_matches.get(point(x, y));
+			const level_object* obj = multi_pattern_matches.get(point(x, y));
 			if(obj) {
 				level_tile t;
 				t.x = xpos;
@@ -796,7 +877,7 @@ void tile_map::build_tiles(std::vector<level_tile>* tiles, const rect* r) const
 				variation_num = 0;
 			}
 			assert(p->variations[variation_num]);
-			t.object = p->variations[variation_num];
+			t.object = p->variations[variation_num].get();
 			t.face_right = face_right;
 			if(t.object->flipped()) {
 				t.face_right = !t.face_right;
@@ -813,7 +894,7 @@ void tile_map::build_tiles(std::vector<level_tile>* tiles, const rect* r) const
 				if(a.zorder) {
 					t.zorder = a.zorder;
 				}
-				t.object = a.object;
+				t.object = a.object.get();
 				t.face_right = face_right;
 				if(t.object->flipped()) {
 					t.face_right = !t.face_right;
