@@ -1,7 +1,4 @@
-#include <SDL.h>
-#ifndef SDL_VIDEO_OPENGL_ES
-#include <GL/glew.h>
-#endif
+#include "graphics.hpp"
 
 #include <iostream>
 #include <math.h>
@@ -15,15 +12,11 @@
 #include "level.hpp"
 #include "raster.hpp"
 #include "string_utils.hpp"
+#include "variant_utils.hpp"
 #include "water.hpp"
-#include "wml_node.hpp"
-#include "wml_utils.hpp"
-#include "color_utils.hpp"
 
-#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA)
+#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA) || defined(TARGET_BLACKBERRY)
 #include <EGL/egl.h>
-#include <GLES/gl.h>
-#include <GLES/glext.h>
 #define glBlendEquationOES          preferences::glBlendEquationOES
 #endif
 
@@ -36,54 +29,52 @@ water::water()
 {
 }
 
-water::water(wml::const_node_ptr water_node) :
-  zorder_(wml::get_int(water_node, "zorder", WaterZorder)),
-  current_x_formula_(game_logic::formula::create_optional_formula(water_node->attr("current_x_formula"))),
-  current_y_formula_(game_logic::formula::create_optional_formula(water_node->attr("current_y_formula")))
+water::water(variant water_node) :
+  zorder_(water_node["zorder"].as_int(WaterZorder)),
+  current_x_formula_(game_logic::formula::create_optional_formula(water_node["current_x_formula"])),
+  current_y_formula_(game_logic::formula::create_optional_formula(water_node["current_y_formula"]))
 {
-	FOREACH_WML_CHILD(area_node, water_node, "area") {
-		const rect r(area_node->attr("rect"));
-		std::vector<std::string> str = util::split(area_node->attr("color"));
+	foreach(variant area_node, water_node["area"].as_list()) {
+		const rect r(area_node["rect"]);
+		std::vector<int> color_vec = area_node["color"].as_list_int();
 		unsigned char color[4];
 		for(int n = 0; n != 4; ++n) {
-			if(n < str.size()) {
-				color[n] = atoi(str[n].c_str());
+			if(n < color_vec.size()) {
+				color[n] = static_cast<unsigned char>(color_vec[n]);
 			} else {
 				color[n] = 0;
 			}
 		}
 
-		variant obj;
-		obj.serialize_from_string(area_node->attr("object"));
+		variant obj = area_node["object"];
 		areas_.push_back(area(r, color, obj));
 	}
 }
 
-wml::node_ptr water::write() const
+variant water::write() const
 {
-	wml::node_ptr result(new wml::node("water"));
-	result->set_attr("zorder", formatter() << zorder_);
+	variant_builder result;
+	result.add("zorder", zorder_);
 	foreach(const area& a, areas_) {
-		wml::node_ptr node(new wml::node("area"));
-		node->set_attr("rect", a.rect_.to_string());
-		node->set_attr("color", formatter()
-		               << static_cast<int>(a.color_[0]) << ","
-		               << static_cast<int>(a.color_[1]) << ","
-		               << static_cast<int>(a.color_[2]) << ","
-		               << static_cast<int>(a.color_[3]));
+		variant_builder area_node;
+		area_node.add("rect", a.rect_.write());
+		std::vector<variant> color_vec;
+		color_vec.reserve(4);
+		for(int n = 0; n != 4; ++n) {
+			color_vec.push_back(variant(static_cast<int>(a.color_[n])));
+		}
+		area_node.add("color", variant(&color_vec));
+		area_node.add("object", a.obj_);
 
-		std::string obj;
-		a.obj_.serialize_to_string(obj);
-		node->set_attr("object", obj);
-
-		result->add_child(node);
+		result.add("area", area_node.build());
 	}
 
-	return result;
+	return result.build();
 }
 
 void water::add_rect(const rect& r, const unsigned char* color, variant obj)
 {
+	std::cerr << "ADD WATER: " << r << "\n";
 	areas_.push_back(area(r, color, obj));
 }
 
@@ -170,13 +161,15 @@ bool water::draw_area(const water::area& a, int x, int y, int w, int h) const
 
 	unsigned char water_color[] = {a.color_[0], a.color_[1], a.color_[2], a.color_[3]};
 	
-	glBlendFunc(GL_ONE, GL_ONE);
-	#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA)
-	if (glBlendEquationOES)
+#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA) || defined(TARGET_BLACKBERRY)
+	if (glBlendEquationOES) {
 		glBlendEquationOES(GL_FUNC_REVERSE_SUBTRACT_OES);
-	#elif defined(GL_OES_blend_subtract)
+	}
+#elif defined(GL_OES_blend_subtract)
 	glBlendEquationOES(GL_FUNC_REVERSE_SUBTRACT_OES);
-	#else
+#elif defined(USE_GLES2)
+	glBlendEquation(GL_FUNC_REVERSE_SUBTRACT);
+#else
 	if(GLEW_EXT_blend_equation_separate && (GLEW_ARB_imaging || GLEW_VERSION_1_4)) {
 		glBlendEquation(GL_FUNC_REVERSE_SUBTRACT);
 	} else {
@@ -185,9 +178,12 @@ bool water::draw_area(const water::area& a, int x, int y, int w, int h) const
 		water_color[1] = (max_color - water_color[1])/8;
 		water_color[2] = (max_color - water_color[2])/8;
 	}
-	#endif
+#endif
+
+#if !defined(USE_GLES2)
 	glDisable(GL_TEXTURE_2D);
 	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+#endif
 	
 	GLfloat vertices[] = {
 		waterline_rect.x, waterline_rect.y, //shallow water colored
@@ -204,24 +200,36 @@ bool water::draw_area(const water::area& a, int x, int y, int w, int h) const
 #else
 	glColor4ub(water_color[0], water_color[1], water_color[2], water_color[3]);
 #endif	
+#if defined(USE_GLES2)
+	gles2::manager gles2_manager(gles2::get_simple_shader());
+	gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, 0, 0, vertices);
+#else
 	glVertexPointer(2, GL_FLOAT, 0, vertices);
+#endif
+	glBlendFunc(GL_ONE, GL_ONE);
 	glDrawArrays(GL_TRIANGLE_STRIP, 0, sizeof(vertices)/sizeof(GLfloat)/2);
-	#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA)
-	if (glBlendEquationOES)
+#if defined(TARGET_OS_HARMATTAN) || defined(TARGET_PANDORA) || defined(TARGET_TEGRA) || defined(TARGET_BLACKBERRY)
+	if (glBlendEquationOES) {
 		glBlendEquationOES(GL_FUNC_ADD_OES);
-	#elif defined(GL_OES_blend_subtract)
+	}
+#elif defined(GL_OES_blend_subtract)
 	glBlendEquationOES(GL_FUNC_ADD_OES);
-	#else
-	if (GLEW_EXT_blend_equation_separate && (GLEW_ARB_imaging || GLEW_VERSION_1_4))
+#elif defined(USE_GLES2)
+	glBlendEquation(GL_FUNC_ADD);
+#else
+	if (GLEW_EXT_blend_equation_separate && (GLEW_ARB_imaging || GLEW_VERSION_1_4)) {
 		glBlendEquation(GL_FUNC_ADD);
-	#endif
+	}
+#endif
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
 	glLineWidth(2.0);
 
 	typedef std::pair<int, int> Segment;
 
+#if !defined(USE_GLES2)
 	glEnableClientState(GL_COLOR_ARRAY);
+#endif
 
 	const int EndSegmentSize = 20;
 
@@ -239,9 +247,18 @@ bool water::draw_area(const water::area& a, int x, int y, int w, int h) const
 			255, 255, 255, 255,
 			255, 255, 255, 0,
 		};
+#if defined(USE_GLES2)
+		{
+			gles2::manager gles2_manager(gles2::get_simple_col_shader());
+			gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, GL_FALSE, 0, varray);
+			gles2::active_shader()->shader()->color_array(4, GL_UNSIGNED_BYTE, GL_TRUE, 0, vcolors);
+			glDrawArrays(GL_LINE_STRIP, 0, 4);
+		}
+#else
 		glVertexPointer(2, GL_FLOAT, 0, varray);
 		glColorPointer(4, GL_UNSIGNED_BYTE, 0, vcolors);
 		glDrawArrays(GL_LINE_STRIP, 0, 4);
+#endif
 	
 		//draw a second line, in a different color, just below the first
 		glColor4f(0.0, 0.9, 0.75, 0.5);
@@ -257,16 +274,26 @@ bool water::draw_area(const water::area& a, int x, int y, int w, int h) const
 			0, 230, 200, 128,
 			0, 230, 200, 0,
 		};
+#if defined(USE_GLES2)
+		{
+			gles2::manager gles2_manager(gles2::get_simple_col_shader());
+			gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, 0, 0, varray2);
+			gles2::active_shader()->shader()->color_array(4, GL_UNSIGNED_BYTE, GL_TRUE, 0, vcolors2);
+			glDrawArrays(GL_LINE_STRIP, 0, 4);
+		}
+#else
 		glVertexPointer(2, GL_FLOAT, 0, varray2);
 		glColorPointer(4, GL_UNSIGNED_BYTE, 0, vcolors2);
 		glDrawArrays(GL_LINE_STRIP, 0, 4);
+#endif
 	}
 
+#if !defined(USE_GLES2)
 	glDisableClientState(GL_COLOR_ARRAY);
-
-	glColor4f(1.0, 1.0, 1.0, 1.0);
 	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 	glEnable(GL_TEXTURE_2D);
+#endif
+	glColor4f(1.0, 1.0, 1.0, 1.0);
 
 	return true;
 }
