@@ -3,18 +3,18 @@
 #include <vector>
 
 #include "preferences.hpp"
+#include "variant.hpp"
 #include "water_particle_system.hpp"
-#include "wml_utils.hpp"
 
-water_particle_system_info::water_particle_system_info(wml::const_node_ptr node)
-: number_of_particles(wml::get_int(node, "number_of_particles", 1500)),
-repeat_period(wml::get_int(node, "repeat_period", 1000)),
-velocity_x(wml::get_int(node, "velocity_x")),
-velocity_y(wml::get_int(node, "velocity_y", -5)),
-velocity_rand(wml::get_int(node, "velocity_rand", 3)),
-dot_size(wml::get_int(node, "dot_size", 1)*(preferences::double_scale() ? 2 : 1))
+water_particle_system_info::water_particle_system_info(variant node)
+: number_of_particles(node["number_of_particles"].as_int(1500)),
+repeat_period(node["repeat_period"].as_int(1000)),
+velocity_x(node["velocity_x"].as_int()),
+velocity_y(node["velocity_y"].as_int(-5)),
+velocity_rand(node["velocity_rand"].as_int(3)),
+dot_size(node["dot_size"].as_int(1)*(preferences::double_scale() ? 2 : 1))
 {
-	irgba = graphics::color(node->attr("color")).value();
+	irgba = graphics::color(node["color"]).value();
 
 	if(dot_size > 1 && preferences::xypos_draw_mask) {
 		//if we are clipping our drawing granularity, then we have a small
@@ -23,7 +23,7 @@ dot_size(wml::get_int(node, "dot_size", 1)*(preferences::double_scale() ? 2 : 1)
 	}
 }
 
-water_particle_system_factory::water_particle_system_factory (wml::const_node_ptr node)
+water_particle_system_factory::water_particle_system_factory(variant node)
  : info(node)
 {
 	
@@ -112,16 +112,24 @@ void water_particle_system::draw(const rect& screen_area, const entity& e) const
 	if(vertices.empty()) {
 		return;
 	}
+
+	glColor4f(info_.rgba[0]/255.0, info_.rgba[1]/255.0, info_.rgba[2]/255.0, info_.rgba[3]/255.0);
+#if defined(USE_GLES2)
+	glPointSize(info_.dot_size);
+	gles2::manager gles2_manager(gles2::get_simple_shader());
+	gles2::active_shader()->shader()->vertex_array(2, GL_SHORT, GL_FALSE, 0, &vertices.front());
+	glDrawArrays(GL_POINTS, 0, vertices.size()/2);
+#else
 	glDisable(GL_TEXTURE_2D);
 	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
 	glPointSize(info_.dot_size);
-	glColor4f(info_.rgba[0]/255.0, info_.rgba[1]/255.0, info_.rgba[2]/255.0, info_.rgba[3]/255.0);
 
 	glVertexPointer(2, GL_SHORT, 0, &vertices.front());
 	glDrawArrays(GL_POINTS, 0, vertices.size()/2);
 
 	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 	glEnable(GL_TEXTURE_2D);
+#endif
 	glColor4f(1.0, 1.0, 1.0, 1.0);
 }
 
