@@ -3,9 +3,9 @@
 #include <vector>
 
 #include "weather_particle_system.hpp"
-#include "wml_utils.hpp"
+#include "variant_utils.hpp"
 
-weather_particle_system_factory::weather_particle_system_factory (wml::const_node_ptr node)
+weather_particle_system_factory::weather_particle_system_factory (variant node)
  : info(node)
 {
 	
@@ -49,8 +49,10 @@ void weather_particle_system::process(const entity& e)
 
 void weather_particle_system::draw(const rect& area, const entity& e) const
 {
+#if !defined(USE_GLES2)
 	glDisable(GL_TEXTURE_2D);
 	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+#endif
 	glLineWidth(info_.line_width);
 	glColor4f(info_.rgba[0]/255.0, info_.rgba[1]/255.0, info_.rgba[2]/255.0, info_.rgba[3]/255.0);
 	int offset_x = area.x() - area.x()%info_.repeat_period;
@@ -77,10 +79,38 @@ void weather_particle_system::draw(const rect& area, const entity& e) const
 			my_y += info_.repeat_period;
 		} while (my_y < area.y()+area.h());
 	}
+#if defined(USE_GLES2)
+	gles2::manager gles2_manager(gles2::get_simple_shader());
+	gles2::active_shader()->shader()->vertex_array(2, GL_FLOAT, 0, 0, &vertices.front());
+#else
 	glVertexPointer(2, GL_FLOAT, 0, &vertices.front());
+#endif
 	glDrawArrays(GL_LINES, 0, vertices.size()/2);
+#if !defined(USE_GLES2)
 	//glDisable(GL_SMOOTH);
 	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 	glEnable(GL_TEXTURE_2D);
+#endif
 	glColor4f(1.0, 1.0, 1.0, 1.0);
 }
+
+variant weather_particle_system::get_value(const std::string& key) const
+{
+	if(key == "velocity_x") {
+		return variant(decimal(direction[0]));
+	} else if(key == "velocity_y") {
+		return variant(decimal(direction[1]));
+	} else {
+		return variant();
+	}
+}
+
+void weather_particle_system::set_value(const std::string& key, const variant& value)
+{
+	if(key == "velocity_x") {
+		direction[0] = value.as_decimal().as_float();
+	} else if(key == "velocity_y") {
+		direction[1] = value.as_decimal().as_float();
+	}
+}
+
